#import pickle
import os
from   string             import lstrip, rstrip, split, strip, lower, find

from coupledDevsV2        import coupledDevsV2        as CoupledDEVS
from atomicDevsV2         import atomicDevsV2         as AtomicDEVS
from stateDevsV2          import stateDevsV2          as SeqState
from portDevsV2           import portDevsV2           as Port
from channelV2            import channelV2            as Channel
from containsStateV2      import containsStateV2      as ContainsState      
from containsPortV2       import containsPortV2       as ContainsPort   
from containsModelV2      import containsModelV2      as ContainsModel     
from internalTransitionV2 import internalTransitionV2 as InternalTransition 
from externalTransitionV2 import externalTransitionV2 as ExternalTransition
from insDevsV2            import insDevsV2            as ModelInstance
from eventDevsV2          import eventDevsV2          as DevsEvent
from DevsState            import DevsState
from DevsExperiment       import DevsExperiment
#from PathTool             import PathTool
#from DataStructure        import *
    
class DevsCodeGenerator(object):

  def __init__(self, asgRoot, saveFileName): #, pathTool):
    self.asgRoot       = asgRoot
    self.code          = ''
    self.indentLevel   = 0
    self.indentUnit    = '  '
    self.saveFileName  = saveFileName
    pathName, fileName = os.path.split(saveFileName)
    packName, fileExt  = split(fileName, '.')
    self.packName      = packName
    
  def genCode(self):
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'package ' + self.packName
    self.indentLevel = self.indentLevel + 1
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'import devs.*;'
    self.code = self.code + '\n' + indentation + 'import externalfunctions.*;'
    self.code = self.code + '\n' + indentation + 'import simulator.*;'
    self.code = self.code + '\n'

    modelList = self.asgRoot.listNodes['atomicDevsV2'] \
                + self.asgRoot.listNodes['coupledDevsV2'] \
                + self.asgRoot.listNodes['eventDevsV2'] \
                + self.asgRoot.listNodes['DevsExperiment']
                
    for model in modelList:    
      if(self.isAtomicModel(model)):
        atomicGenerator = AtomicCodeGenerator( model, self.indentLevel, self.indentUnit)
        atomicGenerator.genCode()
        print atomicGenerator.code
        self.code = self.code + atomicGenerator.code
        
      elif(self.isCoupledModel(model)):
        coupledGenerator = CoupledCodeGenerator( model, self.indentLevel, self.indentUnit)
        coupledGenerator.genCode()
        print coupledGenerator.code
        self.code = self.code + coupledGenerator.code
       
      elif(self.isEventModel(model)):
        eventGenerator = EventCodeGenerator( model, self.indentLevel, self.indentUnit)
        eventGenerator.genCode()        
        self.code = self.code + eventGenerator.code
      elif(self.isExperimentModel(model)):
        experimentGenerator = ExperimentCodeGenerator(model, self.indentLevel, self.indentUnit)
        experimentGenerator.genCode()
        self.code = self.code + experimentGenerator.code
    self.indentLevel = self.indentLevel - 1
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'end ' + self.packName + ';'
    
  def isAtomicModel(self, model):
    if(isinstance(model, AtomicDEVS)):
      return True
    return False
     
  def isCoupledModel(self, model):
    if(isinstance(model, CoupledDEVS)):
      return True
    return False
    
  def isEventModel(self, model):
    if(isinstance(model, DevsEvent)):
      return True
    return False
  def isExperimentModel(self, model):  
    if(isinstance(model, DevsExperiment)):
       return True
    return False

  def saveCode(self): #, path, fileName):
    codeFile = open( self.saveFileName, 'w') #os.path.join( path, fileName), "w")
    codeFile.write(self.code)
    codeFile.close()

class AtomicCodeGenerator(object):
  def __init__(self, model, indentLevel, indentUnit):
    self.name          = model.name.getValue()
    self.model         = model
    self.code          = ''
    self.indentLevel   = indentLevel
    self.indentUnit    = indentUnit
    self.modelState    = self.getModelState()
    self.seqStates     = self.getSeqStates()
    self.ports         = self.getPorts()
    
  def genCode(self):
    #self.indentLevel = self.indentLevel + 1
    indentation = self.indentLevel * self.indentUnit #* self.indentLevel
    
    #Generate Model State Class
    self.code = self.code + '\n' + indentation + 'class '
    if(self.modelState == None):
        self.code = self.code + self.name + 'State'        
    else:
        self.code = self.code + self.modelState.name.getValue()
        self.indentLevel = self.indentLevel + 1
        indentation = self.indentLevel * self.indentUnit
        attrList = self.modelState.attributes.getValue()
        for attr in attrList:
            if(str(attr.getValue()[1])=='Text'):
		txtValue = rstrip(str(attr.getValue()[2]))
		pos = find(txtValue, ':')
		if(pos!=-1): #customized type with initialized value like Integer:randint(ia, ib)
		  typ, val = split(txtValue, ':')
		  self.code = self.code + '\n' + indentation + typ + ' ' + str(attr.getValue()[0]) + ' = ' + val #+ ';'
		else:
                  self.code = self.code + '\n' + indentation + rstrip(str(attr.getValue()[2])) + ' ' + str(attr.getValue()[0]) + '()'
            elif(str(attr.getValue()[1])=='Float'):
                self.code = self.code + '\n' + indentation + 'Real ' + str(attr.getValue()[0]) + ' = ' + str(attr.getValue()[2]) # + ';'
            elif(str(attr.getValue()[1])=='List'):
                self.code = self.code + '\n' + indentation + 'list ' + str(attr.getValue()[0]) + '()' # = ' + str(attr.getValue()[2]) + ';'                
            else:
                self.code = self.code + '\n' + indentation + str(attr.getValue()[1]) + ' ' + str(attr.getValue()[0]) + ' = ' + str(attr.getValue()[2])
	
            if(attr.getValue()[3][1]==1):
                self.code = self.code + ' \"ID\" '
            self.code = self.code + ';'

        self.indentLevel = self.indentLevel - 1
        indentation = self.indentLevel * self.indentUnit
    self.indentLevel = self.indentLevel + 1
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + self.name + '.SeqStates seqState(start=' + self.name + '.SeqStates.'
    for sq in self.seqStates:
        if(sq.initial.getValue()[1]):
           self.code = self.code + sq.name.getValue()
    self.code = self.code + ');'   
    self.indentLevel = self.indentLevel - 1
    indentation = self.indentLevel * self.indentUnit    
    self.code = self.code + '\n' + indentation + 'end '
    if(self.modelState == None):
        self.code = self.code + self.name + 'State;\n'        
    else:
        self.code = self.code + self.modelState.name.getValue()+';\n'


    self.code = self.code + '\n' + indentation + 'class ' + self.name
    
    self.indentLevel = self.indentLevel + 1
    indentation = self.indentLevel * self.indentUnit #* self.indentLevel
    self.code = self.code + '\n' + indentation + 'extends AtomicDEVS;' 
    
    parmList = self.model.parameters.getValue()    
    for p in parmList:
        self.code = self.code + '\n' + indentation + 'parameter ' + str(p.getValue()[1]) + ' ' + str(p.getValue()[0]) + '=' + str(p.getValue()[2])+';'
    # Always add a name to the model
    self.code = self.code + '\n' + indentation + 'parameter String name;'
    
    for port in self.ports:
        if(port.portType.getValue()[1]): #output port
            self.code = self.code + '\n' + indentation + 'output DevsPort ' + port.name.getValue() + ';'
        else:
            self.code = self.code + '\n' + indentation + 'input DevsPort ' + port.name.getValue() + ';'

    if(self.modelState == None):
        print 'could not find model state ... '
        self.code = self.code + '\n' + indentation + self.name + 'State state();'        
    else:
        self.code = self.code + '\n' + indentation + self.modelState.name.getValue() + ' state();'        
    
    self.code = self.code + '\n' + indentation + 'type SeqStates = enumeration('
    first = 1
    for state in self.seqStates:
        if(first == 1):
            self.code = self.code + state.name.getValue()
            first = -1;
        else:
            self.code = self.code + ', ' + state.name.getValue()
    self.code = self.code + ');'
    
    self.genInitFunction()
    
    if(self.hasExternalTransitions()):
        self.genExternalTransition()
    if(self.hasInternalTransitions()):
        self.genInternalTransition()
    if(self.hasOutputs()):
        self.genOutputFunction()
    self.genTimeAdvance()
    
    self.indentLevel = self.indentLevel - 1
    
    indentation = self.indentLevel * self.indentUnit #* self.indentLevel
    self.code = self.code + '\n' + indentation + 'end ' + self.name + ';\n'
    
  def genInitFunction(self):	  
      initCode = rstrip(self.model.init.getValue())
      if(len(initCode)>0):
	  self.code = self.code + '\n'
          indentation = self.indentLevel * self.indentUnit
          self.code = self.code + '\n' + indentation + 'function initialization'
          self.code = self.code + '\n' + indentation + 'algorithm'
          self.indentLevel = self.indentLevel + 1
          indentation = self.indentLevel * self.indentUnit
          lines = split(initCode, '\n')
          for line in lines:
              self.code = self.code + '\n' + indentation + line #stateOutput
          self.indentLevel = self.indentLevel - 1
          indentation = self.indentLevel * self.indentUnit
          self.code = self.code + '\n' + indentation + 'end initialization;\n'
      
  def genExternalTransition(self):
      self.code = self.code + '\n'
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'function extTransition'
      self.code = self.code + '\n' + indentation + 'algorithm'
      haveExtTrans = False
      firstState = True
      for seqState in self.seqStates:
          extTransList = self.getExtTransitions(seqState)
          if(len(extTransList)>0):
              self.indentLevel = self.indentLevel + 1
              indentation = self.indentLevel * self.indentUnit
              haveExtTrans = True
              if(firstState):
                  self.code = self.code + '\n' + indentation + 'if( state.seqState == SeqStates.' + seqState.name.getValue() + ' ) then'
                  firstState = False
              else:
                  self.code = self.code + '\n' + indentation + 'elseif( state.seqState == SeqStates.' + seqState.name.getValue() + ' ) then'
                  
          stateExtAction = rstrip(seqState.extAction.getValue())
          if(len(stateExtAction)>0):
              self.indentLevel = self.indentLevel + 1
              indentation = self.indentLevel * self.indentUnit
	      lines = split(stateExtAction, '\n')
	      for line in lines:
                self.code = self.code + '\n' + indentation + line
              self.indentLevel = self.indentLevel - 1
              
          firstCondition = True
          hasCondition = False
          for extTrans in extTransList:
              condition = rstrip(extTrans.condition .getValue())
              action    = rstrip(extTrans.action.getValue())
              targetState = self.getTransitionTarget(extTrans)              
              if(len(condition)>0):
                  hasCondition = True
                  self.indentLevel = self.indentLevel + 1
                  indentation = self.indentLevel * self.indentUnit                                
                  if(firstCondition):
                      self.code = self.code + '\n' + indentation + 'if( ' + condition + ' ) then'
                      if(len(action)>0):
                          self.indentLevel = self.indentLevel + 1
                          indentation = self.indentLevel * self.indentUnit
			  lines = split(action, '\n')
			  for line in lines:
                            self.code = self.code + '\n' + indentation + line
                          self.indentLevel = self.indentLevel -1
                      firstCondition = False
                  else:
                      self.code = self.code + '\n' + indentation + 'elseif( ' + condition + ' ) then'
                      if(len(action)>0):
                          self.indentLevel = self.indentLevel + 1
                          indentation = self.indentLevel * self.indentUnit
			  lines = split(action, '\n')
			  for line in lines:
                            self.code = self.code + '\n' + indentation + line
                          self.indentLevel = self.indentLevel -1
                          
                  self.indentLevel = self.indentLevel + 1
                  indentation = self.indentLevel * self.indentUnit
                  self.code = self.code + '\n' + indentation + 'state.seqState := SeqStates.' + targetState.name.getValue() + ';'
                  self.indentLevel = self.indentLevel - 1
                            
              else:
                  self.indentLevel = self.indentLevel + 1
                  indentation = self.indentLevel * self.indentUnit
                  self.code = self.code + '\n' + indentation + 'state.seqState := SeqStates.' + targetState.name.getValue() + ';'
                  self.indentLevel = self.indentLevel - 1
                  
              if(len(condition)>0):
                  self.indentLevel = self.indentLevel - 1
                  
          if(hasCondition):
              self.indentLevel = self.indentLevel + 1              
              indentation = self.indentLevel * self.indentUnit
              self.code = self.code + '\n' + indentation + 'end if;'              
              self.indentLevel = self.indentLevel - 1
              
          if(len(extTransList)>0):
              self.indentLevel = self.indentLevel - 1
                      
      #if(len(extTransList)>0):
      if(haveExtTrans):
          self.indentLevel = self.indentLevel + 1
          indentation = self.indentLevel * self.indentUnit
          self.code = self.code + '\n' + indentation + 'end if;'              
          self.indentLevel = self.indentLevel - 1
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'end extTransition;\n'


  def genInternalTransition(self):
      self.code = self.code + '\n'
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'function intTransition'
      self.code = self.code + '\n' + indentation + 'algorithm'
      firstState = True
      hasIntTrans = False
      for seqState in self.seqStates:
          intTransList = self.getIntTransitions(seqState)
          if(len(intTransList)>0):
              hasIntTrans = True
              self.indentLevel = self.indentLevel + 1
              indentation = self.indentLevel * self.indentUnit
              if(firstState):
                  self.code = self.code + '\n' + indentation + 'if( state.seqState == SeqStates.' + seqState.name.getValue() + ' ) then'
                  firstState = False
              else:
                  self.code = self.code + '\n' + indentation + 'elseif( state.seqState == SeqStates.' + seqState.name.getValue() + ' ) then'
          
          stateIntAction = rstrip(seqState.intAction.getValue())
          if(len(stateIntAction)>0):
              self.indentLevel = self.indentLevel + 1
              indentation = self.indentLevel * self.indentUnit
	      lines = split(stateIntAction, '\n')
	      for line in lines:
                self.code = self.code + '\n' + indentation + line
              self.indentLevel = self.indentLevel - 1
              
          firstCondition = True
          hasCondition = False
          for intTrans in intTransList:
              condition = rstrip(intTrans.condition .getValue())
              action    = rstrip(intTrans.action.getValue())
              targetState = self.getTransitionTarget(intTrans)              
              if(len(condition)>0):
                  hasCondition = True
                  self.indentLevel = self.indentLevel + 1
                  indentation = self.indentLevel * self.indentUnit                                
                  if(firstCondition):
                      self.code = self.code + '\n' + indentation + 'if( ' + condition + ' ) then'
                      if(len(action)>0):
                          self.indentLevel = self.indentLevel + 1
                          indentation = self.indentLevel * self.indentUnit
			  lines = split(action, '\n')
			  for line in lines:
                            self.code = self.code + '\n' + indentation + line
                          self.indentLevel = self.indentLevel -1
                      firstCondition = False
                  else:
                      self.code = self.code + '\n' + indentation + 'elseif( ' + condition + ' ) then'
                      if(len(action)>0):
                          self.indentLevel = self.indentLevel + 1
                          indentation = self.indentLevel * self.indentUnit
			  lines = split(action, '\n')
			  for line in lines:
                            self.code = self.code + '\n' + indentation + line
                          self.indentLevel = self.indentLevel -1
                          
                  self.indentLevel = self.indentLevel + 1
                  indentation = self.indentLevel * self.indentUnit
                  self.code = self.code + '\n' + indentation + 'state.seqState := SeqStates.' + targetState.name.getValue() + ';'
                  self.indentLevel = self.indentLevel - 1
                            
              else:
                  self.indentLevel = self.indentLevel + 1
                  indentation = self.indentLevel * self.indentUnit
                  self.code = self.code + '\n' + indentation + 'state.seqState := SeqStates.' + targetState.name.getValue() + ';'
                  self.indentLevel = self.indentLevel -1
                             
              if(len(condition)>0):
                self.indentLevel = self.indentLevel - 1
          if(hasCondition):
              self.indentLevel = self.indentLevel + 1              
              indentation = self.indentLevel * self.indentUnit
              self.code = self.code + '\n' + indentation + 'end if;'              
              self.indentLevel = self.indentLevel - 1
                      
          if(len(intTransList)>0):
              self.indentLevel = self.indentLevel - 1
              
      if(hasIntTrans):
          self.indentLevel = self.indentLevel + 1
          indentation = self.indentLevel * self.indentUnit
          self.code = self.code + '\n' + indentation + 'end if;'              
          self.indentLevel = self.indentLevel -1

              
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'end intTransition;\n'
      
  def genOutputFunction(self):
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'function outputFnc'
      self.indentLevel = self.indentLevel + 1
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'DevsEvent evt = null;'
      self.indentLevel = self.indentLevel - 1
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'algorithm'
      for seqState in self.seqStates:
          stateOutput = rstrip(seqState.output.getValue())
          if(len(stateOutput)>0):
              self.indentLevel = self.indentLevel + 1
              indentation = self.indentLevel * self.indentUnit
              self.code = self.code + '\n' + indentation + 'if( state.seqState == SeqStates.'\
                        + seqState.name.getValue() + ') then'
              self.indentLevel = self.indentLevel + 1
              indentation = self.indentLevel * self.indentUnit
              lines = split(stateOutput, '\n')
              for line in lines:
                self.code = self.code + '\n' + indentation + line #stateOutput
              self.indentLevel = self.indentLevel - 1
              
              indentation = self.indentLevel * self.indentUnit
              self.code = self.code + '\n' + indentation + 'end if;'
              self.indentLevel = self.indentLevel - 1
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'end outputFnc;\n'

  def genTimeAdvance(self):
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'function timeAdvance'
      self.indentLevel = self.indentLevel + 1
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'output Integer timespan;'
      self.indentLevel = self.indentLevel - 1
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'algorithm'
      self.indentLevel = self.indentLevel + 1
      for seqState in self.seqStates:          
          stateTime = rstrip(seqState.timeAdvance.getValue())
          if(len(stateTime)>0):
              indentation = self.indentLevel * self.indentUnit
              self.code = self.code + '\n' + indentation + 'if( state.seqState == SeqStates.'\
                        + seqState.name.getValue() + ') then'
              self.indentLevel = self.indentLevel + 1
              indentation = self.indentLevel * self.indentUnit
              lines = split(stateTime, '\n')
              for line in lines:
                self.code = self.code + '\n' + indentation + line #stateOutput
              self.indentLevel = self.indentLevel - 1
              indentation = self.indentLevel * self.indentUnit
              self.code = self.code + '\n' + indentation + 'end if;'
      self.indentLevel = self.indentLevel - 1
      indentation = self.indentLevel * self.indentUnit
      self.code = self.code + '\n' + indentation + 'end timeAdvance;\n'
                  
  def hasExternalTransitions(self):
      for sq in self.seqStates:
          extTransList = self.getExtTransitions(sq)
          if(len(extTransList)>0):
              return True
      return False
  
  def hasInternalTransitions(self):
      for sq in self.seqStates:
          intTransList = self.getIntTransitions(sq)
          if(len(intTransList)>0):
              return True
      return False
  
  def hasOutputs(self):
      for sq in self.seqStates:
          outputText = rstrip(sq.output.getValue())
          if(len(outputText)>0):
              return True
      return False
      
  def getSeqStates(self):
    states = []
    for child in self.model.devChildren:
      if(isinstance(child, SeqState)):
        states.append(child)
    return states

  def getModelState(self):
    for child in self.model.devChildren:
        if(isinstance(child, DevsState)):
            return child
    return None

  def getPorts(self):
    ports = []
    for child in self.model.devChildren:
      if(isinstance(child, Port)):
        ports.append(child)
    return ports
  
  #Return a list of outgoing internal transitions of a given state
  def getIntTransitions(self, state):
    intTrans = []
    for conn in state.out_connections_:
      if(isinstance(conn, InternalTransition)):    
        intTrans.append(conn)
    return intTrans
    
  #Return a list of outgoing external transitions of a given state
  def getExtTransitions(self, state):
    extTrans = []
    for conn in state.out_connections_:
      if(isinstance(conn, ExternalTransition)):
        extTrans.append(conn)
    return extTrans
     
  # Get the target state of a internal or external transition  
  def getTransitionTarget(self, transition):
    return transition.out_connections_[0]
    
#Generate code for coupled Devs model        
class CoupledCodeGenerator(object):
  def __init__(self, model, indentLevel, indentUnit):
    self.model     = model
    self.name      = self.model.name.getValue()
    self.indentLevel = indentLevel
    self.indentUnit = indentUnit
    self.code      = ''
    self.instances = self.getInstances()
    self.ports     = self.getPorts()

  def getInstances(self):
    insList = []
    for child in self.model.devChildren:
      if(isinstance(child, ModelInstance)):
        insList.append(child)
    return insList
    
  #Return ports of the coupled model
  def getPorts(self):
    ports = []
    for child in self.model.devChildren:
      if(isinstance(child, Port)):
        ports.append(child)
    return ports
  #Return all the ports of the model including instances and the coupled model
  def getAllPorts(self):
    ports = []
    for inst in self.instances: 
      for child in inst.devChildren:
        if(isinstance(child, Port)):
          ports.append(child)
          
    return ports+self.ports 

  #Generate instance code    
  def genInstanceCode(self, inst):
    insCode = ''
    #insCode = inst.type.getValue() + ' ' + inst.name.getValue() + '("' + str(inst.name.getValue()) + '"' \
    insCode = inst.type.getValue() + ' ' + inst.name.getValue() + '(' \
               + rstrip(inst.parameters.getValue()) + ')'
    return insCode
      
  #Generate channel code for each port
  def genPortCode(self, port):
    output = []
    outList = port.out_connections_
    if(len(outList)>0):
      for conn in outList:
        portCode = ''
        if(isinstance(conn, Channel)):
          sourceParent = port.devParent.name.getValue()
          targetPort = conn.out_connections_[0]
          targetParent = targetPort.devParent.name.getValue()          
          #print 'channel'
          if(sourceParent==self.name):
            portCode += port.name.getValue()
          else:
            portCode += sourceParent + '.' + port.name.getValue()
          if(targetParent==self.name):
            portCode += ', '+ targetPort.name.getValue()
          else:
            portCode += ', ' + targetParent + '.' + targetPort.name.getValue()
        output.append(portCode)
            
    return output
   
  def genCode(self):
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'class ' + self.name
    self.indentLevel = self.indentLevel + 1
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'extends CoupledDEVS;'
    clVarList = self.model.classVariables.getValue()
    
    for var in clVarList:
        self.code = self.code + '\n' + indentation \
                  + var.getValue()[1] + ' ' + var.getValue()[0] + ' = ' + var.getValue()[2] + ' \"ClassVariable\";' 
      
    parmList = self.model.parameters.getValue()
    for p in parmList:
        self.code = self.code + '\n' + indentation + 'parameter '\
                  + p.getValue()[1] + ' ' + p.getValue()[0] + ';'
    self.code = self.code + '\n' + indentation + 'parameter String name;'
                                       
    attrList = self.model.attributes.getValue()
    for attr in attrList:
        self.code = self.code + '\n' + indentation \
                  + attr.getValue()[1] + ' ' + attr.getValue()[0] + ' = ' + str(attr.getValue()[2]) + ';'
  
    for port in self.ports:
        if(port.portType.getValue()[1]):
            self.code = self.code + '\n' + indentation + 'output DevsPort ' + port.name.getValue() + ';'
        else:
            self.code = self.code + '\n' + indentation + 'input DevsPort ' + port.name.getValue() + ';'
    for inst in self.instances:
        self.code = self.code + '\n' + indentation + self.genInstanceCode(inst) + ';'
    
    self.indentLevel = self.indentLevel - 1
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'equation'
    self.indentLevel = self.indentLevel + 1
    indentation = self.indentLevel * self.indentUnit
    
    portList = self.getAllPorts()
    for port in portList:
        portCode = self.genPortCode(port)
        for code in portCode:
            self.code = self.code + '\n' + indentation + 'connect( ' + code + ');' 
      
    self.indentLevel = self.indentLevel - 1
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'end ' + self.name + ';\n'

#Generate code for Event or Auxilary class
class EventCodeGenerator(object):
  def __init__(self, model, indentLevel, indentUnit):
    self.model = model
    self.name  = self.model.name.getValue()
    self.indentLevel = indentLevel
    self.indentUnit = indentUnit
    self.code      = ''

    
  def genCode(self):
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'class ' + self.name
    self.indentLevel = self.indentLevel + 1
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'extends DevsEvent;'
    
    clVarList = self.model.classVariables.getValue()
    for var in clVarList:
        self.code = self.code + '\n' + indentation \
                  + var.getValue()[1] + ' ' + var.getValue()[0] + ' = ' + var.getValue()[2] + ' \"ClassVariable\";' 
      
    parmList = self.model.parameters.getValue()
    for p in parmList:
        #TODO HERE
        if(str(p.getValue()[1])=='Float'):
            self.code = self.code + '\n' + indentation + 'parameter Real ' + str(p.getValue()[0]) + ';'
        elif(str(p.getValue()[1])=='List'):
            self.code = self.code + '\n' + indentation + 'parameter list ' + str(p.getValue()[0]) + '()' + ';'
        else:
            self.code = self.code + '\n' + indentation + 'parameter ' + p.getValue()[1] + ' ' + p.getValue()[0] + ';'
                                       
    attrList = self.model.attributes.getValue()
    for attr in attrList:
        #print str(attr)
        #key, keyValue = attr.getValue()[3]
        #print keyValue
        #print attr.getValue()[3][1]
        #print dir(attr)
#        self.code = self.code + '\n' + indentation \
#                  + attr.getValue()[1] + ' ' + attr.getValue()[0] + ' = ' + str(attr.getValue()[2])
        if(str(attr.getValue()[1])=='Text'):
	    txtValue = rstrip(str(attr.getValue()[2]))
	    pos = find(txtValue, ':')
	    if(pos!=-1): #customized type with initialized value like Integer:randint(ia, ib)
		typ, val = split(txtValue, ':')
		self.code = self.code + '\n' + indentation + typ + ' ' + str(attr.getValue()[0]) + ' = ' + val
	    else:
                self.code = self.code + '\n' + indentation + rstrip(str(attr.getValue()[2])) + ' ' + str(attr.getValue()[0]) + '()'
        elif(str(attr.getValue()[1])=='Float'):
            self.code = self.code + '\n' + indentation + 'Real ' + str(attr.getValue()[0]) + ' = ' + str(attr.getValue()[2])
        elif(str(attr.getValue()[1])=='List'):
            self.code = self.code + '\n' + indentation + 'list ' + str(attr.getValue()[0]) + '()' # = ' + str(attr.getValue()[2]) + ';'
        else:
            print("Got unknown type: " + str(attr.getValue()[1]))
            self.code = self.code + '\n' + indentation + str(attr.getValue()[1]) + ' ' + str(attr.getValue()[0]) + ' = ' + str(attr.getValue()[2])


        if(attr.getValue()[3][1]==1):
            self.code = self.code + ' \"ID\" '
        self.code = self.code + ';'
    
    
    self.indentLevel = self.indentLevel - 1
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'end ' + self.name + ';\n'
    
#Generate code for Event or Auxilary class
class ExperimentCodeGenerator(object):
  def __init__(self, model, indentLevel, indentUnit):
    self.model = model
    self.name  = self.model.experimentName.getValue()
    self.indentLevel = indentLevel
    self.indentUnit = indentUnit
    self.code      = ''

    
  def genCode(self):
    indentation = self.indentLevel * self.indentUnit
    if(len(strip(self.model.package.getValue()))>0):
      self.code = self.code  + '\n' + indentation + 'import ' + self.model.package.getValue() + ';'    
    self.code = self.code + '\n' + indentation + 'class ' + self.name
    self.indentLevel = self.indentLevel + 1
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'extends DevsExperiment;'
    
    self.code = self.code + '\n' + indentation + self.model.model.getValue() + ' '\
              + lower(strip(self.model.model.getValue()))
    self.code += 'Model('
    if rstrip(self.model.parameters.getValue()) != "":
        self.code += rstrip(self.model.parameters.getValue()) + ', '
    self.code += 'name=\"' + self.name + '\");' 
    
    self.code = self.code + '\n' + indentation + 'Simulator sim(simModel=' + lower(strip(self.model.model.getValue())) + 'Model);'    
    #self.code = self.code + '\n' + indentation + 'Integer i = sim.simulate(verbose=true, trace=true);'
    self.code = self.code + '\n' + indentation + 'Boolean verbose = ' + str(bool(self.model.verbose.getValue()[1])) + ';'
    self.code = self.code + '\n' + indentation + 'Boolean trace = ' + str(bool(self.model.trace.getValue()[1])) + ';'
    self.code = self.code + '\n' + indentation + 'Real end_time = ' + str(self.model.end_time.getValue()) + ';'
    self.indentLevel = self.indentLevel - 1
    indentation = self.indentLevel * self.indentUnit
    self.code = self.code + '\n' + indentation + 'end ' + self.name + ';\n'
    
