from Tkinter import *
from lib.DevsSimInterfaces import *
 
class DevsPlotter(Frame, DevsEventListener):
  def __init__( self
              , parent=None              
              , rowList=[]
              , eventList=[]
              , title='DEVS Plotter'
              , rowTitle='state'
              , plotterScale=1.0):
    Frame.__init__(self, parent)    
    #self.pack(expand=YES, fill=BOTH)
    self.parent         = parent
    self.rowList        = rowList
    self.eventList      = eventList
    self.title          = title
    self.rowTitle       = rowTitle
    self.scale          = plotterScale
    self.showTrace      = True
    self.showText       = True
    self.startNumber    = 0
    self.leftMargin     = 80
    self.rightMargin    = 40
    self.topMargin      = 30
    self.bottomMargin   = 30
    self.plotWidth      = 0
    self.plotHeight     = 0
    self.sourceY        = {} 
    canv=Canvas(self, relief=SUNKEN) #, bg='brown')
    canv.config(width=300, height=200)
    #canv.config(highlightthickness=3) # no pixels to border    
    canv.pack(side=TOP, expand=YES, fill=BOTH)
    canv.bind('<Configure>', self.initPanel)
    #canv.takefocus=1
    self.canvas = canv
    self.makeOperationArea()   
    
  def makeOperationArea(self):
    opArea = Frame(self, cursor='hand2', relief=RIDGE, bd=2)
    opArea.pack(side=BOTTOM, fill=X)        
  
    toolbar = Frame(opArea, cursor='hand2', relief=RIDGE, bd=2)
    toolbar.grid(row=0, column=1, sticky=NSEW) #pack(side=RIGHT) #, fill=X)        
    Button(toolbar, text='|<<', command=self.toBegin).grid(row=0, column=0) #.pack(side=RIGHT)
    Button(toolbar, text='<<',  command=self.previousPage).grid(row=0, column=1) #.pack(side=RIGHT)
    Button(toolbar, text='<',   command=self.previousOne).grid(row=0, column=2) #.pack(side=RIGHT)
    Button(toolbar, text='>',   command=self.nextOne).grid(row=0, column=3) #.pack(side=RIGHT)
    Button(toolbar, text='>>',  command=self.nextPage).grid(row=0, column=4) #.pack(side=RIGHT)
    Button(toolbar, text='>>|', command=self.toEnd).grid(row=0, column=5) #.pack(side=RIGHT)
#    Button(toolbar, text='New Window', command=self.separateWindow).grid(row=0, column=6) #.pack(side=RIGHT)
    Button(toolbar, text='PostScript', command=self.SaveToPostScript).grid(row=0, column=6) #.pack(side=RIGHT)    
    optionbar = Frame(opArea, cursor='hand2', relief=RIDGE, bd=2)
    optionbar.grid(row=0, column=0, sticky=NSEW) #pack(side=LEFT) #, fill=X)        
    l = Label(optionbar, text='Scale:')
    #L.config(relief=SUNKEN)
    l.grid(row=0, column=0, sticky=E) #pack(side=LEFT)
    self.scaleEntry = Entry(optionbar, width=6)
    #scale.config(anchor=W)
    self.scaleEntry.insert(END, str(self.scale))
    self.scaleEntry.grid(row=0, column=1, sticky=W) #pack(side=LEFT)
    self.scaleEntry.bind('<KeyPress-Return>', self.setScale)
    
    self.showTraceVar = IntVar()
    ch=Checkbutton( optionbar
                 , text='Line'
                 , variable=self.showTraceVar
                 , command=self.showTraceLine)
    ch.select()
    #ch.config(anchor=W, width=20)
    ch.grid(row=0, column=2) #pack(side=LEFT)

    self.showTextVar = IntVar()
    ch=Checkbutton( optionbar
                 , text='Text'
                 , variable=self.showTextVar
                 , command=self.showTraceText)
    ch.select()
    #ch.config(anchor=W, width=20)
    ch.grid(row=0, column=3) #pack(side=LEFT)
    
    opArea.rowconfigure(0, weight=1)
    opArea.columnconfigure(0, weight=1)
    opArea.columnconfigure(1, weight=1)
    optionbar.rowconfigure(0, weight=1)
    optionbar.columnconfigure(0, weight=1)
    optionbar.columnconfigure(1, weight=1)
    optionbar.columnconfigure(2, weight=2) 
    optionbar.columnconfigure(3, weight=2)        
    #self.showTraceVars.append(var)
        
    #sbar = Scrollbar(opArea)
    #listbox = Listbox(opArea, relief=SUNKEN)
    #sbar.config(command=listbox.yview)
    #listbox.config(yscrollcommand=sbar.set)
    #listbox.config(selectmode=SINGLE, setgrid=1, width=4, height=1)
    #listbox.bind('<Double-1>', self.listHandler)
    #sbar.pack(side=RIGHT, fill=Y)
    #listbox.pack(side=LEFT, fill=Y)
    #self.modelListbox = listbox
  
  def showTraceLine(self):
    self.showTrace = self.showTraceVar.get()
    self.updateDisplay()
    #print 'trace value: ', self.showTraceVar.get()

  def showTraceText(self):
    self.showText = self.showTextVar.get()
    #print self.showText
    self.updateDisplay()
    
  def separateWindow(self):
    win = Toplevel()
    plotter = DevsPlotter( parent=win
                         , rowList=self.rowList
                         , eventList=self.eventList
                         , title=self.title
                         , rowTitle=self.rowTitle
                         , plotterScale=self.scale)
    plotter.pack(expand=YES, fill=BOTH)
    plotter.mainloop()

  
  def setScale(self, event):
    scale = self.scaleEntry.get()
    if(len(scale)>0):
      self.scale = float(scale)
      self.updateDisplay()
    #self.scale = scale
  
  def setTrace(self, trace):
    self.showTrace = trace
    
  def initPanel(self, event):
    #print 'event.height', event.height
    #print 'event.width', event.width
    self.canvas.delete('all')
    row = len(self.rowList) + 1
    evtNumber   = len(self.eventList)
    rowInterval = (event.height-self.topMargin-self.bottomMargin)/row
    self.plotHeight = event.height-self.topMargin-self.bottomMargin
    self.plotWidth = event.width-self.leftMargin-self.rightMargin

    self.canvas.create_text( event.width/2
                           , self.topMargin/2
                           , text=self.title
                           , fill='blue'
                           , font=("Helvetica", "16"))

    for i in range(0, row):  
      if( i == row-1):
        self.canvas.create_line( self.leftMargin
                               , self.topMargin+rowInterval*(i+1) 
                               , self.leftMargin+self.plotWidth 
                               , self.topMargin+rowInterval*(i+1) 
                               , width=3 
                               , fill='blue' 
                               , arrow=LAST 
                               , arrowshape=(8,10,3))
        #self.canvas.create_text( self.leftMargin
        #                       , self.topMargin+rowInterval*(i+1)+3
        #                       , text='Event'
        #                       , anchor=NE)
        self.canvas.create_text( self.leftMargin+self.plotWidth 
                               , self.topMargin+rowInterval*(i+1)+5
                               , text='Time'
                               , anchor=N)
      else:
        self.canvas.create_line( self.leftMargin
                               , self.topMargin+rowInterval*(i+1)
                               , self.leftMargin+self.plotWidth 
                               , self.topMargin+rowInterval*(i+1)
                               , fill='grey')                               
        self.canvas.create_text( self.leftMargin/8
                               , self.topMargin+rowInterval*(i+1)
                               , width=self.leftMargin-4
                               , text=self.rowList[i]
                               , anchor=W
                               , fill='blue')
        self.sourceY[self.rowList[i]] = self.topMargin+rowInterval*(i+1)

    self.canvas.create_line( self.leftMargin
                           , self.topMargin
                           , self.leftMargin
                           , self.topMargin+rowInterval*row
                           , width=3
                           , fill='blue'
                           , arrow=FIRST
                           , arrowshape=(8,10,3))
    self.canvas.create_text( self.leftMargin/2
                           , self.topMargin
                           , text=self.rowTitle)
                           
    #if(len(self.eventList)>0):
    self.updateDisplay()
    
  def updateDisplay(self):     
    #for evt in self.eventList:
    #  print evt
    #print self.sourceY.keys()
    if(len(self.eventList) > 0):
      tag = 'content'
      self.canvas.delete(tag)
      baseTime = self.eventList[self.startNumber].time
      preEvt = None
      for evt in self.eventList[self.startNumber:]:
        #print evt 
        #print evt.source, self.startNumber
        if self.scale*(evt.time-baseTime)< self.plotWidth:
          if(evt.time>0.0):
            ovalMark = self.canvas.create_oval( int(self.leftMargin+self.scale*int(evt.time - baseTime))-2
                                              , self.sourceY[evt.source]-2
                                              , int(self.leftMargin+self.scale*int(evt.time - baseTime))+2
                                              , self.sourceY[evt.source]+2
                                              , fill=''
                                              , outline="green")
            self.canvas.addtag_withtag(tag, ovalMark)                       

          if(self.showText):
            if(preEvt):
              txtContent = self.canvas.create_text( self.leftMargin+self.scale*int(evt.time - baseTime)
                                                  , self.sourceY[preEvt.source]
                                                  , text=evt.content
                                                  , font=("Helvetica", "8"))
              self.canvas.addtag_withtag(tag, txtContent)                       
          txtTime = self.canvas.create_text( self.leftMargin+self.scale*int(evt.time - baseTime)
                                           , self.topMargin+self.plotHeight
                                           , text=evt.time
                                           , anchor=N
                                           , font=("Helvetica", "8"))
          self.canvas.addtag_withtag(tag, txtTime)
          if( not (preEvt!=None and preEvt.time==evt.time)):                                                                
            lineMark = self.canvas.create_line( self.leftMargin + self.scale*int(evt.time - baseTime)
                                              , self.topMargin
                                              , self.leftMargin + self.scale*int(evt.time - baseTime)
                                              , self.topMargin + self.plotHeight - 3
                                              , fill='grey')                               
            self.canvas.addtag_withtag(tag, lineMark) 
          if(self.showTrace):
            if(preEvt):
              traceMark1 = self.canvas.create_line( self.leftMargin + self.scale*int(preEvt.time - baseTime)
                                                  , self.sourceY[preEvt.source]
                                                  , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                  , self.sourceY[preEvt.source]
                                                  , fill='black')
              self.canvas.addtag_withtag(tag, traceMark1) 
              endOvalMark = self.canvas.create_oval( int(self.leftMargin+self.scale*int(evt.time - baseTime))-3
                                                   , self.sourceY[preEvt.source]-3
                                                   , int(self.leftMargin+self.scale*int(evt.time - baseTime))+3
                                                   , self.sourceY[preEvt.source]+3
                                                   , fill='black')
              self.canvas.addtag_withtag(tag, endOvalMark)

              startOvalMark = self.canvas.create_oval( int(self.leftMargin+self.scale*int(evt.time - baseTime))-2
                                                     , self.sourceY[evt.source]-2
                                                     , int(self.leftMargin+self.scale*int(evt.time - baseTime))+2
                                                     , self.sourceY[evt.source]+2
                                                     , fill=''
																										 , outline='green')
              self.canvas.addtag_withtag(tag, startOvalMark)
                      
              if(evt.content.find("Int Trans")!=-1):
                if(self.sourceY[preEvt.source] > self.sourceY[evt.source]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)+10
                                                      , (self.sourceY[preEvt.source]+self.sourceY[evt.source])/2
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[evt.source]                                     
                                                      , fill='green'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     
                elif(self.sourceY[preEvt.source] < self.sourceY[evt.source]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)-10
                                                      , (self.sourceY[preEvt.source]+self.sourceY[evt.source])/2
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[evt.source]                                            
                                                      , fill='green'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2) 
                else: #equal
                  traceMark2 = self.canvas.create_line( self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)-10
                                                      , self.sourceY[preEvt.source]-10
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]-20
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)+10
                                                      , self.sourceY[preEvt.source]-10
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[evt.source]                                     
                                                      , fill='green'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     

              elif(evt.content.find("Ext Trans")!=-1):
                if(self.sourceY[preEvt.source] > self.sourceY[evt.source]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)+10
                                                      , (self.sourceY[preEvt.source]+self.sourceY[evt.source])/2
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[evt.source]                                             
                                                      , fill='red'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     
                elif(self.sourceY[preEvt.source] < self.sourceY[evt.source]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)-10
                                                      , (self.sourceY[preEvt.source]+self.sourceY[evt.source])/2
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[evt.source]                                             
                                                      , fill='red'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2) 
                else: #equal
                  traceMark2 = self.canvas.create_line( self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)-10
                                                      , self.sourceY[preEvt.source]-10
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]-20
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)+10
                                                      , self.sourceY[preEvt.source]-10
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[evt.source]                                     
                                                      , fill='red'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     

              else:
                if(self.sourceY[preEvt.source] > self.sourceY[evt.source]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)+10
                                                      , (self.sourceY[preEvt.source]+self.sourceY[evt.source])/2
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[evt.source]                                             
                                                      , fill='black'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     
                elif(self.sourceY[preEvt.source] < self.sourceY[evt.source]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)-10
                                                      , (self.sourceY[preEvt.source]+self.sourceY[evt.source])/2
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[evt.source]                                             
                                                      , fill='black'
                                                      , arrow=LAST
                                                      , smooth=True)                                                    
                  self.canvas.addtag_withtag(tag, traceMark2) 
                else: #equal
                  traceMark2 = self.canvas.create_line( self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)-10
                                                      , self.sourceY[preEvt.source]-10
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[preEvt.source]-20
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)+10
                                                      , self.sourceY[preEvt.source]-10
                                                      , self.leftMargin + self.scale*int(evt.time - baseTime)
                                                      , self.sourceY[evt.source]                                     
                                                      , fill='black'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     

            preEvt = evt
        else:
          break

  def newEvent(self, event): 
    #print 'New event: %s' % event     
    self.eventList.append(event)
    #eventNumber = len(self.eventList)
    if(self.scale*(event.time-self.eventList[self.startNumber].time)>self.plotWidth):
      self.startNumber = self.startNumber + 1
    #print 'Start Number:', self.startNumber
    self.updateDisplay()
    
  def setEventSource(self, eventSource):
    eventSource.addEventListener(self)
    
  def previousOne(self):
    if(self.startNumber==0):
      return
    self.startNumber=self.startNumber-1    
    self.updateDisplay()
    
  def nextOne(self):
    if(self.startNumber==len(self.eventList)-1):
      return
    self.startNumber=self.startNumber+1
    self.updateDisplay()
    
  def toBegin(self):
    self.startNumber=0
    self.updateDisplay()
    
  def toEnd(self):
    self.startNumber=len(self.eventList)-1
    self.updateDisplay()

  def previousPage(self):
    if(self.startNumber==0):
      return    
    else:
      oldStart = self.startNumber
      self.startNumber -= 1
      xDistance = self.scale*(self.eventList[oldStart].time - self.eventList[self.startNumber].time)
      while(self.startNumber>0 and xDistance < self.plotWidth):
        self.startNumber -= 1
        xDistance = self.scale*(self.eventList[oldStart].time - self.eventList[self.startNumber].time)
    self.updateDisplay()
    
  def nextPage(self):
    maxNumber = len(self.eventList)-1
    if(self.startNumber==maxNumber):
      return    
    else:
      oldStart = self.startNumber
      self.startNumber += 1
      xDistance = self.scale*(self.eventList[self.startNumber].time - self.eventList[oldStart].time)
      while(self.startNumber<maxNumber and xDistance < self.plotWidth):
        self.startNumber += 1
        xDistance = self.scale*(self.eventList[self.startNumber].time - self.eventList[oldStart].time)
    self.updateDisplay()

  def SaveToPostScript(self):
# asksaveasfilename 
    import tkFileDialog
    import os
    psfile = tkFileDialog.asksaveasfilename(initialdir = os.getcwd(), defaultextension = '.ps')
    #outfile = open(psfile, 'w')
    print psfile
    #newtag=100
    #self.canvas.addtag_all(newtag)
    self.canvas.postscript(file=psfile)
    #outfile.close();
#    path = ''
#    path = tkFileDialog.askdirectory( title = "Please select a directory to save the generated code"
#                                    , initialdir = '' 
#                                    )
#    if(len(path)>0):                                    
#      if( not os.path.exists(path) ):
#        os.mkdir(path)
#    else:
#      path = os.getcwd()    
#    return path       
if __name__ == '__main__':
  win = Tk()
  rowList = 'model0', 'model1', 'model2','model3','model4','model5'
  canv = DevsPlotter(win, rowList)
  canv.pack(side=TOP)
  #Button(win, text='test', command=(lambda c=canv: dictTest(c))).pack(side=BOTTOM)
  #canv.canvas.bind('<Configure>', lambda event: canv.initPanel(event))
  win.mainloop()
    

