import tkFileDialog
import tkMessageBox
import os
import distutils.file_util

class PathTool(object):   

  def __init__(self, devsLibPath='pyDEVS'):
    self.devsLibPath = os.path.join(os.getcwd(), devsLibPath)
    self.basePath    = os.path.join(os.getcwd(), 'devscode')
    self.modelPath   = os.path.join(self.basePath, 'model')
    self.libPath     = os.path.join(self.basePath, 'lib')
    
  def setDevsLibPath(self, devsLibPath):
    self.devsLibPath = os.path.normpath(devsLibPath)
    
  def initPaths(self):
    self.initPath(self.basePath)
    self.initPath(self.modelPath)
    self.initPath(self.libPath)  
    
  def initPath(self, pathName):
    initFile = os.path.join(pathName, '__init__.py')
    f = open( initFile, 'w' )
    f.write( '""" __init__.py\n\nMakes imports easy\n"""' )
    f.close()
 
  def isPathInitialized(self, pathName):
    initFile = os.path.join(pathName, '__init__.py')
    if(os.path.exists(initFile)):
      return True
    else:
      return False

  def askPath(self):
    path = ''
    path = tkFileDialog.askdirectory( title = "Please select a directory to save the generated code"
                                    , initialdir = '' 
                                    )
    if(len(path)>0):                                    
      if( not os.path.exists(path) ):
        os.mkdir(path)
    else:
      path = os.getcwd()    
    return path
    
    
  def makePath(self, path):
    os.mkdir(path)
  
  def makePaths(self):
    self.makePath(self.basePath)
    self.makePath(self.modelPath)
    self.makePath(self.libPath)
   
  # delete a path and all its contents including subdirectories
  def deletePath(self, path):
    names = os.listdir(path)
    for name in names:
      pathName = os.path.join(path, name)
      if not os.path.isdir(pathName):
        os.remove(pathName)
      else:
        self.deletePath(pathName)

    os.rmdir(path)  
   
  def copyDevsLibFiles(self):
    while( not os.path.exists(os.path.join(self.devsLibPath, 'DEVS.py'))):
      tkMessageBox.showerror(title = "Directory Error !"
                        , message = 'The devsLibPath does not exist, please set it to the path where the DEVS.py is')
      
      self.devsLibPath = tkFileDialog.askdirectory( title = "Please select the directory where DEVS.py is"
                                    , initialdir = '' 
                                    )

            
    for file in os.listdir( self.devsLibPath ):
      distutils.file_util.copy_file(os.path.join(self.devsLibPath,file), 
                                    os.path.join(self.libPath,file) )
 
  def preparePaths(self):
    path = self.askPath()
    self.basePath  = os.path.join(path, 'devscode')
    self.modelPath = os.path.join(self.basePath, 'model')
    self.libPath   = os.path.join(self.basePath, 'lib')

    if( not os.path.exists(path)):
      os.mkdir(os.path.normpath(path))
    else:
      if(os.path.exists(self.basePath)):
        self.deletePath(self.basePath)

    self.makePaths()
    self.initPaths()
    self.copyDevsLibFiles()
 
  def getModelPath(self):
    return self.modelPath
    
  def getBasePath(self):
    return self.basePath
    
  def getLibPath(self):
    return self.libPath
    
if __name__=='__main__':
  pathTool = PathTool()
  #Set the path where you can find the DEVS.py and Simulator.py
  pathTool.setDevsLibPath('/home/moncs/hsong9/pystudy/DEVSEnv/lib')
  #pathTool.askPath()
  pathTool.preparePaths()
  #pathTool.copyDevsLibFiles()

#modelpath = os.path.join( path, 'models' )
#libpath   = os.path.join( path, 'lib' )
# If the path exists, empty the models directory
#if( os.path.exists(path)): 
#  if(os.path.
# If the path does not exists, create it 
#else:
#  os.mkdir(os.path.normpath( path ))

#if( not os.path.exists(modelpath)):
#  os.mkdir(modelpath)

#if( not os.path.exists(libpath)):
#  os.mkdir(libpath)

  


#  print path
#  tkMessageBox.showerror(title = "Directory Error !"
#                        , message = 'The directory you selected does not exist, please try again')
#  return
  # Even the path returned from the tkFileDialog is also
  # needed to be processed by the normpath function, or
  # you will fail and get an error from os.mkdir()

#  print 'does not exist'
#  print pathFile
  
