"""
codeGenDevsV2.py

Overview:
  Author: Denis Dube
  Date:   March 20, 2005
  Module: codeGenDevsV2.py
  Description: Updated for use with a re-worked version of the AToM3 
              implementation of DEVS.
              
Original Source:
 Author: Ernesto Posse
 Date:   October 26, 2002
 Module: code_gen.py and devs_utils.py
 Description: A set of utilities used by the devs meta-model in ATOM3 to
   generato code for the Python DEVS simulator
   

# How to use the codge generator:
# Assuming your inside a buttons model, where self is an AToM3 instance:
import codeGenDevsV2
codeGenDevsV2.genCode( self.ASGroot )
"""

# Make the imports just like the ones in the old Devs :D
from coupledDevsV2        import coupledDevsV2        as coupled_devs
from atomicDevsV2         import atomicDevsV2         as atomic_devs
from stateDevsV2          import stateDevsV2          as state
from portDevsV2           import portDevsV2           as port
from channelV2            import channelV2            as channel
from containsStateV2      import containsStateV2      as contains_state      
from containsPortV2       import containsPortV2       as contains_port   
from containsModelV2      import containsModelV2      as contains_model     
from internalTransitionV2 import internalTransitionV2 as internal_transition 
from externalTransitionV2 import externalTransitionV2 as external_transition 
from DevsCodeGenerator    import DevsCodeGenerator
from PathTool             import PathTool

import os
import os.path
import string
import tkFileDialog
import distutils.file_util

BASEDIR = os.path.split( __file__ )[ 0 ] # <-- Exactly which dir this file is in

def getIO():
  if( IOstuff.INSTANCE == None ):
    IOstuff.INSTANCE = IOstuff()
  return IOstuff.INSTANCE

def genCode( ASGroot ):
  print '\n<----Generating DEVS code---->\n\n'

  pathTool = PathTool()
  #Set the path where you can find the DEVS.py and Simulator.py
  pathTool.setDevsLibPath('/home/moncs/hsong9/py
  study/DEVSEnv/lib')
  pathTool.askPath()
  pathTool.preparePaths()
  codeGenerator = DevsCodeGenerator(ASGroot, pathTool)
  codeGenerator.genCode()
    

def genCodeRecursive( node, path , isInitial=False ):
  """
  Generates Python DEVS Simulator code
  """  
  if isinstance(node, coupled_devs) or isinstance(node, atomic_devs):
    IO = getIO()
    
    # Initial call is at the root    
    if( isInitial ):  IO.copySimulationEnvironment()
    # Otherwise, we create a dir structure one level deeper
    else:   
      path = IO.setPath( path, node.name.getValue()+"_devs_model" )
      IO.initPath()
     
    #print node.name.getValue(), path
    if( isinstance(node, coupled_devs)):
      #print 'compiling coupled devs', node 
      compile_coupled_devs(node)
    else:
      #print 'compiling atomic devs', node 
      compile_atomic_devs(node)
          
    for submodel in get_submodels(node):
      #print 'Generating submodel', submodel.name.getValue(), '\n'
      genCodeRecursive( submodel, path )
      
class AtomicTemplate(object):
  def __init__(self, name):
    self.name = name
    self.headInfo='# AtomicDevs model : %s \n' % self.name    
    self.imports=[]
    self.imports.append('\nfrom lib.DEVS import *')
    self.imports.append('\nfrom lib.Simulator import *')
    self.states        = []
    self.initfunc      = []
    self.intTransition = []
    self.extTransition = []
    self.timeAdvance   = []
    self.outputFunc    = []
     
    
class CoupledTemplate(object):
  def __init__(self, name):
    self.name=name


# find_root loos at all the nodes in the graph and choses the one which is
# supposed to be the root. If there is more than one root or no nodes,
# a message is printed on the console
def find_root(root):  
  if( not root.__dict__.has_key( 'listNodes' ) ): return None
  coupled_models = root.listNodes['coupledDevsV2']
  atomic_models  = root.listNodes['atomicDevsV2']
  roots = []
  for node in coupled_models + atomic_models:
    parent = get_node_parent(node)
    if parent == None:
      roots.append(node)
  if roots == []:
    print "There is no root!"
  elif len(roots) > 1:
    print "There is more than one root!"
  if roots != []:
    return roots[0]
  return None



# Function: check_atomic_devs
# Input: node: atomic_devs
# Output: boolean; 1 if the model is legal, 0 if not
def check_atomic_devs(node):
  return check_states(node) and check_ports(node) and \
         check_internal_transitions(node) and check_channels(node)


# Function: check_channels
# Input: node: atomic_devs
# Output: boolean; 1 if the model is legal, 0 if not
def check_channels(node):
  result = 1
  node_name = node.name.getValue()
  ports = filter(lambda child: isinstance(child, port), node.devChildren)
  names  = map(lambda s: s.name.getValue(), ports)
  for some_port in ports:
    port_name = some_port.name.getValue()
    port_type_enum = some_port.port_type.getValue()
    port_type = port_type_enum[0][port_type_enum[1]]
    incomming_channels = len(get_incomming_channels(some_port))
    outgoing_channels = len(get_outgoing_channels(some_port))
    if port_type == 'Input':
      if incomming_channels == 0:
        print "The node "+node_name+" has an input port ("+port_name+") with no incomming channels"
        result = 0
      if outgoing_channels != 0:
        print "The node "+node_name+" has an input port ("+port_name+") with outgoing channels"
        result = 0
    elif port_type == 'Output':
      if outgoing_channels == 0:
        print "The node "+node_name+" has an output port ("+port_name+") with no outgoing channels channels"
        result = 0
      if incomming_channels != 0:
        print "The node "+node_name+" has an output port ("+port_name+") with incomming channels channels"
        result = 0
  return result


# Function: check_internal_transitions
# Input: node: atomic_devs
# Output: boolean; 1 if the model is legal, 0 if not
def check_internal_transitions(node):
  result = 1
  states = filter(lambda child: isinstance(child, state), node.devChildren) 
  return result


# Function: check_ports
# Input: node: atomic_devs
# Output: boolean; 1 if the model is legal, 0 if not
def check_ports(node):
  result = 1
  node_name = node.name.getValue()
  ports = filter(lambda child: isinstance(child, port), node.devChildren)
  names  = map(lambda s: s.name.getValue(), ports)
  for name in names:
    occurrences = len(filter(lambda p: p == name, names))
    if occurrences > 1:
      print "The node "+node_name+" has more than one port labelled "+name
      result = 0
  return result


# Function: check_states
# Input: node: atomic_devs
# Output: boolean; 1 if the model is legal, 0 if not
def check_states(node):
  result = 1
  node_name = node.name.getValue()
  states = filter(lambda child: isinstance(child, state), node.devChildren)
  names  = map(lambda s: s.name.getValue(), states)
  for name in names:
    same_name_list = map(lambda n: n == name, names)
    occurrences = reduce(lambda a,b: a+b, same_name_list)
    if occurrences > 1:
      print "The node "+node_name+" has more than one state labelled "+name
      result = 0
  initials = filter(lambda s: s.initial.getValue()[1], states)
  if len(initials) == 0:
    print "The node " + node_name + " has no initial state."
    result = 0
  elif len(initials) > 1:
    print "The node " + node_name + " has more than one initial state; I'm taking " + initials[0] + " as initial."
    result = 0
  return result


# Function: compile_coupled_devs
# Input: node: coupled_devs
# Output: none
# Description: generates a file with the name of the coupled devs model
#   with the simulator code for the model.
def compile_coupled_devs(node):
  name = node.name.getValue()
  file = getIO().fopen(name)
  #file = open( os.path.join( getIO().getPath(), name + ".py"), "w")

  submodels = get_submodels(node)
  input_ports   = get_input_ports(node)
  output_ports  = get_output_ports(node)
  couplings     = get_couplings(node)
  
  file.write("# Code generated by the graph grammar devs_code_gen in the tool ATOM3 for a classic DEVS simulator"+chr(10))
  file.write("# Graph Grammar by:  Ernesto Posse"+chr(10))
  file.write("# ATOM3 by:          Juan De Lara"+chr(10))
  file.write("# DEVS simulator by: Jean-Sebastien Bolduc"+chr(10))
  file.write(chr(10))
  file.write("from DEVS import *"+chr(10))
  file.write("from Simulator import *"+chr(10))
  file.write(chr(10))

  for submodel in submodels:
    file.write("from "+submodel.name.getValue()+"_devs_model."
               +submodel.name.getValue()+" import *"+chr(10))
  file.write(chr(10))
  file.write("class "+name+"(CoupledDEVS):"+chr(10))
  file.write(chr(10))
  file.write("  def __init__(self):"+chr(10))
  file.write("    CoupledDEVS.__init__(self)"+chr(10))
  file.write("    self.INFINITY = 99999"+chr(10))
  for inport in input_ports:
    file.write("    self."+inport.name.getValue()+" = self.addInPort()"+chr(10))
  for outport in output_ports:
    file.write("    self."+outport.name.getValue()+" = self.addOutPort()"+chr(10))
  file.write(chr(10))
  for submodel in submodels:
    file.write("    self."+submodel.name.getValue()
               +" = self.addSubModel("+submodel.name.getValue()+"())"+chr(10))
  file.write(chr(10))
  for coupling in couplings:
    sender_out  = "self."+coupling[0]  #Assumes its in the form "model.port"
    receiver_in = "self."+coupling[1]  #Assumes its in the form "model.port"
    file.write("    self.connectPorts("+sender_out+", "+receiver_in+")"+chr(10))
  file.write(chr(10))
  file.write("  def select(self,imm):"+chr(10))
  file.write("    return imm[0]"+chr(10))
  file.write(chr(10))
  file.write("import sys"+chr(10))
  file.write("simtime = 1000"+chr(10))
  file.write("if len(sys.argv) > 1:"+chr(10))
  file.write("  simtime = float(sys.argv[1])"+chr(10))
  file.write("  S = Simulator("+name+"())"+chr(10))
  file.write("  S.simulate(simtime)"+chr(10))
  file.write(chr(10))

  file.close()


# Function: compile_atomic_devs
# Input: node: atomic_devs
# Output: none
# Description: generates a file with the name of the atomic devs model
#   with the simulator code for the model.
def compile_atomic_devs(node):
  name = node.name.getValue()
  file = getIO().fopen(name)
  #file = open( os.path.join( getIO().getPath(), name + ".py"), "w")

  if not check_atomic_devs(node):
    print "Something is wrong with the model "+name

  input_ports   = get_input_ports(node)
  output_ports  = get_output_ports(node)

  delta_ext = compile_external_transition(node)
  delta_int = compile_internal_transition(node)
  lambda_fn = compile_output_function(node)
  ta        = compile_time_advance(node)
  initial_state = get_initial_state(node)

  file.write("# Code generated by the graph grammar devs_code_gen in the tool ATOM3 for a classic DEVS simulator"+chr(10))
  file.write("# Graph Grammar by:  Ernesto Posse"+chr(10))
  file.write("# ATOM3 by:          Juan De Lara"+chr(10))
  file.write("# DEVS simulator by: Jean-Sebastien Bolduc"+chr(10))
  file.write(chr(10))
  file.write("from DEVS import *"+chr(10))
  file.write("from Simulator import *"+chr(10))
  file.write(chr(10))
  file.write("class "+name+"(AtomicDEVS):"+chr(10))
  file.write(chr(10))
  file.write("  def __init__(self):"+chr(10))
  file.write("    AtomicDEVS.__init__(self)"+chr(10))
  file.write("    self.INFINITY = 99999"+chr(10))
  file.write("    self.state    = \'"+initial_state+"\'"+chr(10))
  file.write("    self.elapsed  = 0.0"+chr(10))

  for inport in input_ports:
    file.write("    self."+inport.name.getValue()+" = self.addInPort()"+chr(10))
  for outport in output_ports:
    file.write("    self."+outport.name.getValue()+" = self.addOutPort()"+chr(10))
  file.write(chr(10))
  file.write("  def extTransition(self):"+chr(10))
  file.write("    INFINITY = self.INFINITY"+chr(10))
  file.write("    s = self.state"+chr(10))
  file.write("    e = self.elapsed"+chr(10))
  for inport in input_ports:
    file.write("    "+inport.name.getValue()+" = self.peek(self."+inport.name.getValue()+")"+chr(10))
  file.write(indent(delta_ext, 4))
  file.write(chr(10))
  file.write("  def intTransition(self):"+chr(10))
  file.write("    INFINITY = self.INFINITY"+chr(10))
  file.write("    s = self.state"+chr(10))
  file.write(indent(delta_int, 4))
  file.write(chr(10))
  file.write("  def outputFnc(self):"+chr(10))
  file.write("    INFINITY = self.INFINITY"+chr(10))
  file.write("    s = self.state"+chr(10))
  for outport in output_ports:
    file.write("    "+outport.name.getValue()+" = None"+chr(10))
  file.write(indent(lambda_fn, 4))
  for outport in output_ports:
    file.write("    self.poke(self."+outport.name.getValue()+", "+outport.name.getValue()+")"+chr(10))
  file.write(chr(10))
  file.write("  def timeAdvance(self):"+chr(10))
  file.write("    INFINITY = self.INFINITY"+chr(10))
  file.write("    s = self.state"+chr(10))
  file.write(indent(ta, 4))
  file.write(chr(10))

  file.close()


# compile_external_transition(node) returns a string which is the python code
# that should go inside the external transition method.
# node is an atomic devs model
def compile_external_transition(node):
  code = ""
  states = filter(lambda child: isinstance(child, state), node.devChildren)
  input_ports = get_input_ports(node)
  inports_names = map(lambda p: p.name.getValue(), input_ports)
  transitions_table = []
  # collect transitions info
  for some_state in states:
    source = some_state.name.getValue()
    targets = []
    for rel in some_state.out_connections_:
      if isinstance(rel, external_transition):
        target = rel.out_connections_[0].name.getValue()
        guard_name = rel.condition.getValue()[0]
        guard_code = rel.condition.getValue()[4]
        targets.append((guard_name,guard_code,target))
    if targets != []:
      transitions_table.append((source, targets))
  # generate code
  first = 1
  for (source,targets) in transitions_table:
    if first:
      code = code + "if s == \'"+source+"\':\n"
      first = 0
    else:
      code = code + "elif s == \'"+source+"\':\n"
    guard_number = 1
    for (guard_name, guard_code, target) in targets:
      parameters = "e"
      if len(inports_names) >= 1:
        parameters = parameters + ", "+string_of_name_list(inports_names)
      complete_guard_name = "guard"+repr(guard_number)+"_"+guard_name
      code = code + "  def "+complete_guard_name+"("+parameters+"):\n"
      code = code + indent(string.strip(guard_code), 4) + "\n\n"
      guard_number = guard_number + 1
    guard_number = 1
    first_guard = 1
    for (guard_name, guard_code, target) in targets:
      parameters = "e"
      if len(inports_names) >= 1:
        parameters = parameters + ", "+string_of_name_list(inports_names)
      complete_guard_name = "guard"+repr(guard_number)+"_"+guard_name
      if first_guard:
        code = code + "  if "+complete_guard_name+"("+parameters+"):\n"
        first_guard = 0
      else:
        code = code + "  elif "+complete_guard_name+"("+parameters+"):\n"
      code = code + "    return \'"+target+"\'\n\n"
      guard_number = guard_number + 1
  return code


# compile_internal_transition(node) returns a string which is the python code
# that should go inside the internal transition method.
# node is an atomic devs model
def compile_internal_transition(node):
  def get_target_state(some_state):
    for rel in some_state.out_connections_:
      if isinstance(rel, internal_transition):
        return get_target(rel)
    return None
  s = ""
  states = filter(lambda child: isinstance(child, state), node.devChildren)
  all_target_states = []
  for some_state in states:
    target = get_target_state(some_state)
    if target == None:
      all_target_states.append((some_state.name.getValue(), 'None'))
    else:
      all_target_states.append((some_state.name.getValue(),
                                target.name.getValue()))
  if len(states) > 0:
    s = s + "if s == \'" + all_target_states[0][0] + "\':\n"
    s = s + "  return \'" + all_target_states[0][1] + "\'\n\n"
    for (some_state, target_state) in all_target_states[1:]:
      s = s + "elif s == \'" + some_state + "\':\n"
      s = s + "  return \'" + target_state + "\'\n\n"
  return s

# compile_output_function(node) returns a string which is the python code
# that should go inside the output function method.
# node is an atomic devs model
def compile_output_function(node):
  s = ""
  states = filter(lambda child: isinstance(child, state), node.devChildren)
  all_lambdas = map(lambda some_state:
                    (some_state.name.getValue(),
                     string.strip(some_state.output.getValue()[4])),
                    states)
  if len(states) > 0:
    s = s + "if s == \'" + all_lambdas[0][0] + "\':\n"
    s = s + indent(all_lambdas[0][1], 2) + "\n\n"
    for (some_state, lam) in all_lambdas[1:]:
      s = s + "elif s == \'" + some_state + "\':\n"
      s = s + indent(lam, 2) + "\n\n"
  return s

# compile_time_advance(node) returns a string which is the python code
# that should go inside the time-advance method.
# node is an atomic devs model
def compile_time_advance(node):
  s = ""
  states = filter(lambda child: isinstance(child, state), node.devChildren)
  all_taus = map(lambda some_state:
                 (some_state.name.getValue(),
                  string.strip(some_state.timeAdvance.getValue()[4])),
                 states)
  if len(states) > 0:
    s = s + "if s == \'" + all_taus[0][0] + "\':\n"
    s = s + indent(all_taus[0][1], 2) + "\n\n"
    for (some_state, tau) in all_taus[1:]:
      s = s + "elif s == \'" + some_state + "\':\n"
      s = s + indent(tau, 2) + "\n\n"
  return s



  
# get_couplings returns a list of port pairs, representing the couplings in a
# coupled devs model (the given node).
def get_couplings(node):
  #print "get_couplings("+repr(node)+")"
  node_name = node.name.getValue()
  #print "  node name = "+node_name
  subchannels = filter(lambda child: isinstance(child, channel), node.devChildren)
  couplings = []
  for subchannel in subchannels:
    print "  subchannel: "+repr(subchannel)
    source = get_source(subchannel)
    target = get_target(subchannel)
    source_name = source.name.getValue()
    target_name = target.name.getValue()
    print "    source: "+source_name
    print "    target: "+target_name
    source_parent_name = source.node_parent.name.getValue()
    target_parent_name = target.node_parent.name.getValue()
    if source_parent_name == node_name:
      qualified_source_name = source_name
    else:
      qualified_source_name = source_parent_name+"."+source_name
    if target_parent_name == node_name:
      qualified_target_name = target_name
    else:
      qualified_target_name = target_parent_name+"."+target_name
    couplings.append((qualified_source_name,qualified_target_name))
  return couplings


# Function: get_initial_state
# Input: node: atomic_devs
# Output: string; the name of the initial state, or "None" if there is none.
def get_initial_state(node):
  states = filter(lambda child: isinstance(child, state), node.devChildren)
  initials = []
  for s in states:
    if s.initial.getValue()[1]:
      initials.append(s.name.getValue())
  if len(initials) == 0:
    return "None"
  return initials[0]




# get_incomming_channels(port)
def get_incomming_channels(port):
  return filter(lambda rel: isinstance(rel, channel), port.in_connections_)

# get_outgoing_channels(port)
def get_outgoing_channels(port):
  return filter(lambda rel: isinstance(rel, channel), port.out_connections_)


# get_sub_models returns the immediate submodels (i.e. shallow get)
def get_submodels(entity):
  submodels = []
  if isinstance(entity, coupled_devs):
    for child in entity.devChildren:
      if isinstance(child, atomic_devs) or isinstance(child, coupled_devs):
        submodels.append(child)
  return submodels

# get_input_ports
def get_input_ports(entity):
  ports = get_ports(entity)
  in_ports = []
  for port in ports:
    enum_value = port.portType.getValue();
    if enum_value[0][enum_value[1]] == "Input":
      in_ports.append(port)
  return in_ports

# get_output_ports
def get_output_ports(entity):
  ports = get_ports(entity)
  out_ports = []
  for port in ports:
    enum_value = port.portType.getValue();
    if enum_value[0][enum_value[1]] == "Output":
      out_ports.append(port)
  return out_ports

# get_ports(entity) returns a list of the ports of a non-terminal entity.
# its argument and its results are the semantic objects.
def get_ports(entity):
  ports = None
  if entity != None and is_entity(entity) and not is_terminal(entity):
    ports = []
    for child in entity.devChildren:
      if child.getClass() == "port":
        ports.append(child)
  return ports

# get_sourceget_targets(r) returns the target of a given relation. It assumes
# the relation to be fully specified.
def get_target(relation):
  try:
    return relation.out_connections_[0]
  except IndexError:
    return None
  
# get_source(r) returns the source of a given relation. It assumes the
# relation to be fully specified.
def get_source(relation):
  try:
    return relation.in_connections_[0]
  except IndexError:
    return None
  

# get_node_parent gets the parent (duh!) of a given semantic object, bypassing
# the relation to the parent; i.e. for a state, finds the atomic devs to
# which it belongs, and for an atomic devs or coupled devs, finds the
# coupled devs it belongs to, if any. Similarly for ports.
def get_node_parent(semantic_object):
  try:
    if semantic_object.node_parent != None and is_legal(semantic_object.node_parent):
      return semantic_object.node_parent
  except AttributeError:
    pass
  node_parent = None
  if is_entity(semantic_object):
    node_parent = lookup_entitys_parent(semantic_object)
  elif is_link(semantic_object):
    node_parent = lookup_links_parent(semantic_object)
  return node_parent


# is_entity
def is_entity(semantic_object):
  return isinstance(semantic_object, state) or \
          isinstance(semantic_object, port) or \
          isinstance(semantic_object, atomic_devs) or \
          isinstance(semantic_object, coupled_devs)

# is_link
def is_link(semantic_object):
  return isinstance(semantic_object, internal_transition) or \
          isinstance(semantic_object, external_transition) or \
          isinstance(semantic_object, channel) or \
          is_containment_relation(semantic_object)

# is_containment_relation returns 1 if the given semantic object is a
# link with ype contains_<x> where <x> is in {state, model, port.
def is_containment_relation(relation):
  return isinstance(relation, contains_state) or \
          isinstance(relation, contains_port) or \
          isinstance(relation, contains_model)

# is_terminal returns true (1) for entities that are not composite, i.e.
# states and ports, and false (0) for composite entities, i.e. atomic_devs
# and coupled_devs models.
def is_terminal(semantic_object):
  return isinstance(semantic_object, state) or \
          isinstance(semantic_object, port) or \
          is_link(semantic_object)


# lookup_entitys_parent looks up for the parent of an entity in the graph.
def lookup_entitys_parent(entity):
  node_parent = None
  for relation in entity.in_connections_:
    if is_containment_relation(relation):
      try:
        node_parent = relation.in_connections_[0]
        if not is_terminal(node_parent):
          break
      except IndexError:
        pass
  return node_parent


# lookup_links_parent looks up for the parent of a link in the graph.
def lookup_links_parent(link):
  node_parent = None
  if is_containment_relation(link):
    try:
      node_parent = link.in_connections_[0]
    except IndexError:
      pass
  elif isinstance(link, internal_transition) or isinstance(link, external_transition):
    found = 0
    for source in link.in_connections_:
      node_parent = get_node_parent(source)
      if node_parent != None:
        found = 1
        break
    if not found:
      for target in link.out_connections_:
        node_parent = get_node_parent(target)
        if node_parent != None:
          break
  elif isinstance(link, channel):
    try:
      source_port = link.in_connections_[0]
      target_port = link.out_connections_[0]
      source_parent = get_node_parent(source_port)
      target_parent = get_node_parent(target_port)
      source_grandparent = get_node_parent(source_parent)
      target_grandparent = get_node_parent(target_parent)
      if source_parent == target_parent:
        node_parent = source_parent
      else:
        if source_grandparent == target_grandparent:
          node_parent = source_grandparent
        elif source_parent == target_grandparent:
          node_parent = source_parent
        elif target_parent == source_grandparent:
          node_parent = target_parent
    except AttributeError:
      pass
  return node_parent


  
# Function: indent
# Input: code:string, num:int
# Output: string; the code indented by num spaces
def indent(code, num):
  new_code = ''
  lead = ''
  i = 0
  while i < num:
    lead = lead + ' '
    i = i + 1
  import StringIO
  strio = StringIO.StringIO(code)
  line = strio.readline()
  while line != '':
    new_line = lead + line
    new_code = new_code + new_line
    line = strio.readline()
  strio.close()
  return new_code


# Function: string_of_name_list
# Input: l: string list
# Output: string; a string with the given names separated by commas
def string_of_name_list(l):
  if len(l) == 0:
    return ""
  elif len(l) == 1:
    return l[0]
  else:
    return l[0]+", "+string_of_name_list(l[1:])

