# MuModelica Compiler Project
# Author: Steven Xu
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University

# This is the AST module

#from ASTExp import *

# ------------------- declaration ----------------------
class TypePrefix:
    #visibility: Enum{'DEFAULT', 'PUBLIC', 'PROTECTED'}
    #isFlow: Boolean
    #variability: Enum{None, 'DISCRETE', 'PARAMETER', 'CONSTANT'}
    #             None means 'CONTINUOUS'
    #causality: Enum{None, 'INPUT', 'OUTPUT'}
    #             None means neither
    def __init__(self, visibilityP='DEFAULT', isflowP=False, \
                 variabilityP=None, causalityP=None):
        self.visibility=visibilityP
        self.isflow=isflowP
        self.variability=variabilityP
        self.causality=causalityP

    def getVisibility(self):
        return self.visibility

    def setVisibility(self, value):
        self.visibility=value

    def getFlow(self):
	return self.isflow

    def setFlow(self, value):
	self.isflow=value

    def getVariability(self):
	return self.variability

    def setVariability(self, value):
	self.variability=value

    def getCausality(self):
	return self.causality

    def setCausality(self, value):
	self.causality=value

    # check if two type prefixes are identical
    def isIdenticalTo(self, tp):
	if not self.getFlow()==tp.getFlow():
	    return False
	if not self.getVariability()==tp.getVariability():
	    return False
	if not self.getCausality()==tp.getCausality():
	    return False
	return True

    def __str__(self):
	result=''
	if self.isflow==True:
	    result=result+'flow '
	if self.variability=='CONSTANT':
	    result=result+'constant '
	elif self.variability=='DISCRETE':
	    result=result+'discrete '
	elif self.variability=='PARAMETER':
	    result=result+'parameter '
	if self.causality=='INPUT':
	    result=result+'input '
	elif self.causality=='OUTPUT':
	    result=result+'output '
	return result

class Declaration:
    def __init__(self, lineNumP, colNumP, identP, arrSubP=None, \
                 modificationP=None, commentP=None):
	self.lineNum=lineNumP
	self.colNum=colNumP
	self.ident=identP
        self.arrSub=arrSubP
        self.modification=modificationP 
        self.comment=commentP

    def getName(self):
	return self.ident

    def getArrSub(self):
	return self.arrSub

    def getModification(self):
	return self.modification

    def __str__(self):
	result=str(self.ident)
	if not self.arrSub==None:
	    result=result+str(self.arrSub)
	if not self.modification==None:
	    result=result + str(self.modification)
	#if not len(self.modifications)>0:
	#    for i in self.modifications:
	#        result=result+str(i) 
	if not self.comment==None:
	    result=result+' '+str(self.comment)
	return result

    def accept(self, v):
	v.visitDeclaration(self)
   
#-----------------ClassModifier & ClassModification-------------------

# abstract class: Modification
class Modification:
    def __init__(self, lineNumP, colNumP):
        raise "Abstract class Modification can not be instantiated!"

class ClassModification(Modification):
    'Represent a class_modification of the form, e.g., Real a(unit="V")=3;'
    def __init__(self, lineNumP, colNumP, argumentListP, \
		modificationExpP):
	self.lineNum=lineNumP
	self.colNum=colNumP
	self.argumentList=argumentListP # list of Argument
	self.modificationExp=modificationExpP
	# ASTComponent::EqualsModification
	# an EqualsModification following argumentList

    def getArgumentList(self):
	return self.argumentList
 
    def getEqualsModification(self):
	return self.modificationExp

    def __str__(self):
	result='('
	temp=''
	for i in self.argumentList:
	    temp=temp+str(i)+', '
	result=result+temp[0:len(temp)-2]+')'
	if not self.modificationExp==None:
	    #result=result+'='+str(self.modificationExp)
	    result=result+str(self.modificationExp)
	return result

    def accept(self, v):
	v.visitClassModification(self)

class EqualsModification(Modification):
    def __init__(self, lineNumP, colNumP, expP):
	self.lineNum=lineNumP
	self.colNum=colNumP
	self.exp=expP

    def getExp(self):
	return self.exp

    def __str__(self):
	return '=' + str(self.exp)

    def accept(self, v):
	v.visitEqualsModification(self)

class AssignModification(Modification):
    def __init__(self, lineNumP, colNumP, expP):
	self.lineNum=lineNumP
	self.colNum=colNumP
        self.exp=expP

    def getExp(self):
        return self.exp

    def __str__(self):
        return ':=' + str(self.exp)

    def accept(self, v):
	v.visitAssignModification(self)

class Argument:
    def __init__(self, lineNumP, colNumP):
        raise "Abstract class Argument can not be instantiated!"

class ElementModification(Argument):
    def __init__(self, lineNumP, colNumP, compRefP, \
                 modificationP, commentP=None, \
                 hasEachP=False, isFinalP=False):
	self.lineNum=lineNumP
	self.colNum=colNumP
	self.compRef=compRefP
	self.modification=modificationP # Modification
	self.comment=commentP # StringComment
	self.hasEach=hasEachP
	self.isFinal=isFinalP
        # 'redeclare' not yet supported

    def getCompRef(self):
	return self.compRef

    def setCompRef(self, cr):
	self.compRef=cr

    def getModification(self):
	return self.modification

    def __str__(self):
	result=''
	if self.hasEach==True:
	    result=result+'each '
	if self.isFinal==True:
            result=result+'final '
	result=result+str(self.compRef)+str(self.modification)
	if not self.comment==None:
	    result=result+str(self.comment)
        return result

    def accept(self, v):
        v.visitElementModification(self)

class ElementRedeclaration(Argument):
    def __init__(self, lineNumP, colNumP, classDefinitionP, \
		componentClause1P, replaceableClauseP, \
		hasEachP=False, isFinalP=False):
	self.lineNum=lineNumP
	self.colNum=colNumP
	self.classDefinition=classDefinitionP
	self.componentClause1=componentClause1P
	self.replaceableClause=replaceableClauseP
	self.hasEach=hasEachP
	self.isFinal=isFinalP

    def __str__(self):
	result='redeclare '
	if self.hasEach==True:
	    result=result+'each '
	if self.isFinal==True:
	    result=result+'final '
	if not self.classDefinition==None:
	    result=result+str(self.classDefinition)
	elif not self.componentClause1==None:
	    result=result+str(self.componentClause1)
	elif not self.replaceableClause==None:
	    result=result+str(self.replaceableClause)
	return result

    def accept(self, v):
        v.visitElementRedeclaration(self)

class ComponentClause1:
    def __init__(self, lineNumP, colNumP, typePrefixP, typeP, \
		componetDeclarationP, isFinalP=False, \
		isReplaceableP=False):
	self.lineNum=lineNumP
	self.colNum=colNumP
	self.typePrefix=typePrefixP
	self.type=typeP
	self.componetDeclaration=componetDeclarationP # DECLARATION
	self.isFinal=isFinalP
	self.isReplaceable=isReplaceableP

    def setFinal(self, value):
        self.isFinal=value

    def setReplaceable(self, value):
        self.isReplaceable=value

    def __str__(self):
	result=''
	if self.isReplaceable==True:
	    result=result+'replaceable '
	result=result+str(self.typePrefix)+str(self.type)+' '+ \
		str(self.componetDeclaration)
	return result
