# MuModelica Compiler Project
# Author: Steven Xu
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University

from AST.ASTEquations import *
from BackEnd.printers.EqExpPrinter import EqExpPrinter

class CanonicalTransformer:
    "canonical transformation"
    def __init__(self, flatClassDef):
        # the AST (ClassFile)
        #self.ast=astP
        # the target class
        #self.modelName=modelNameP
	#mainEntry=self.ast.getScope().localLookup(self.modelName)
	#self.mainModel=mainEntry.getDefinition()
	self.mainModel=flatClassDef
	self.printer=EqExpPrinter()

    def go(self):
	# rule 0: eliminate '(' in AST (ExpListExp of kind 'PAR')
	#self.rule0()
	# set 1
	# step 1: convert Div and Sub exps
	# rule 2: convert constants to real numbers
        self.rule2()
	# rule 3: rewrite constants or term, e.g, -c -> +(-c),
        # -E -> (-1.0)*E, where E is a term (convert SubtractExp to SumExp)
        self.rule3()
	# rule 4: rewrite reciprocals (DivExp)
        self.rule4()
	# step 2: convert + and * to n-nary operators (flatten)
	# rule 5: convert '*' to n-nary operator
        self.rule5()
	# rule 6: convert '+' to n-nary operator
        self.rule6()
	# step 3: shift RHS to LHS
	# rule 1: move RHS to LHS
        self.rule1()
	# step 4: flatten the LHS again
	self.rule5()
	self.rule6()
	# step 5: constant fold
	# rule 7.0: constant folding in PowerExp
        #self.rule7_0()
        # rule 7.1: constant folding in Sum and Mul
        self.rule7_1()
	# rule 7.2 remove superfluous: 0.0+E -> E, 1.0*E -> E, 0.0*E -> 0.0
        self.rule7_2()
	# remove superfluous in PowerExp: 0.0^c -> 0.0, 0.0^-c -> error
        # E^1.0 -> E, E^0.0 -> 1.0
        self.rule7_3()
	# rule 7.4: convert unary MulExp and SumExp, e.g, *(32)->32, +(a)->a
        self.rule7_4()

	# set 2
	strInitEqs1=[]
	strEqs1=[]
	t1=(strInitEqs1, strEqs1)
	for i in self.mainModel.getFlattenedInitEqs():
	    strInitEqs1.append(self.printer.dumpEq(i))
        for i in self.mainModel.getFlattenedEqs():
	    strEqs1.append(self.printer.dumpEq(i))
	while True:
	    # simplify power of products
	    self.rule12()
	    # flatten LHS
	    self.rule5()
            self.rule6()
	    # constant fold
	    #self.rule7_0()
	    self.rule7_1()
	    self.rule7_2()
	    self.rule7_3()
	    self.rule7_4()
	    # sort
	    self.rule8()
	    # simplify power of powers
	    self.rule11()
	    # simplify products of powers of the same base
	    self.rule10()
	    # constant fold
	    #self.rule7_0()
            self.rule7_1()
            self.rule7_2()
            self.rule7_3()
            self.rule7_4()
	    # sort
            self.rule8()
	    # creat string eqs
	    strInitEqs2=[]
            strEqs2=[]
	    t2=(strInitEqs2, strEqs2)
            for i in self.mainModel.getFlattenedInitEqs():
            	strInitEqs2.append(self.printer.dumpEq(i))
		#strInitEqs2.append(str(i))
            for i in self.mainModel.getFlattenedEqs():
            	strEqs2.append(self.printer.dumpEq(i))
		#strEqs2.append(str(i))
	    if not self.hasChangesInEqs(t1, t2):
		break
	    else:
		t1=t2

	# followed by:
	while True:
	    # distribute constant in MulExp
	    self.rule13()
	    # flatten LHS
            self.rule5()
            self.rule6()
	    # constant fold
            #self.rule7_0()
            self.rule7_1()
            self.rule7_2()
            self.rule7_3()
            self.rule7_4()
	    # sort
            self.rule8()
	    # collect like terms
	    self.rule9()
	    # constant fold
            #self.rule7_0()
            self.rule7_1()
            self.rule7_2()
            self.rule7_3()
            self.rule7_4()
	    # sort
            self.rule8()
	    # create string eqs
	    strInitEqs3=[]
            strEqs3=[]
	    t3=(strInitEqs3, strEqs3)
            for i in self.mainModel.getFlattenedInitEqs():
                strInitEqs3.append(self.printer.dumpEq(i))
		#strInitEqs3.append(str(i))
            for i in self.mainModel.getFlattenedEqs():
                strEqs3.append(self.printer.dumpEq(i))
		#strEqs3.append(str(i))
	    if not self.hasChangesInEqs(t2, t3):
                break
	    else:
                t2=t3


    # a method to test whether or not the equation trees have changed
    # by string comparison
    # t1, t2: tuple of list
    def hasChangesInEqs(self, t1, t2):
	initList1=t1[0]
	list1=t1[1]
	initList2=t2[0]
	list2=t2[1]
	if not (len(initList1)==len(initList2) and len(list1)==len(list2)):
	    raise 'two equation lists have inequal number of equations'
	for i in range(0, len(initList1)):
	    if not initList1[i]==initList2[i]:
		return True
	for i in range(0, len(list1)):
	    if not list1[i]==list2[i]:
		return True
	return False

    # rule 0: eliminate '(' in AST (ExpListExp of kind 'PAR')
    def rule0(self):
        for i in self.mainModel.getFlattenedInitEqs():
            self.eliminateParInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.eliminateParInEq(i)

    def eliminateParInEq(self, eq):
	if isinstance(eq, RegularEqStm):
	    eq.setLHS(self.eliminatePar(eq.getLHS()))
	    eq.setRHS(self.eliminatePar(eq.getRHS()))
	else:
	    raise 'Currently only handles regular equations in \
			canonical transformation!'

    # rule 0: eliminate '('
    def eliminatePar(self, exp):
	if isinstance(exp, ExpListExp):
	    operands=exp.getOperands()
	    if exp.getKind()=='PAR' and len(operands)==1:
		return self.eliminatePar(operands[0])
	elif isinstance(exp, SumExp) or \
	     isinstance(exp, SubtractExp) or \
	     isinstance(exp, MulExp) or \
	     isinstance(exp, DivExp) or \
	     isinstance(exp, UnaryMinusExp) or \
	     isinstance(exp, PowerExp):
	    operands=exp.getOperands()
	    index=0
	    for index in range(0, len(operands)):
		exp.operands[index]=self.eliminatePar(operands[index])
	    return exp
	else:
	    return exp

    # rule 1
    def rule1(self):
	for i in self.mainModel.getFlattenedInitEqs():
            self.moveRHSToLHS(i)
        for i in self.mainModel.getFlattenedEqs():
            self.moveRHSToLHS(i)

    # rule 1: move RHS to LHS
    def moveRHSToLHS(self, eq):
	if isinstance(eq, RegularEqStm):
	    LHS=eq.getLHS()
	    RHS=eq.getRHS()
	    if isinstance(RHS, IfElseExp):
		# the IfElseExp need to be expanded first before this 
		pass
	    elif isinstance(RHS, SumExp):
		operands=RHS.getOperands()
		newRHS=RealExp(-1, -1, 0.0)
		for i in range(0, len(operands)):
		    operands[i]=MulExp(-1, -1, [RealExp(-1, -1, -1.0), \
				operands[i]])
		newLHS=SumExp(-1, -1, [LHS, RHS])
		eq.setLHS(newLHS)
		eq.setRHS(newRHS)
	    else:
		newRHS=RealExp(-1, -1, 0.0)
                newTerm=MulExp(LHS.getLineNum(), LHS.getColNum(), \
                        [RealExp(-1, -1, -1.0), RHS])
                newLHS=SumExp(-1, -1, [LHS, newTerm])
                eq.setLHS(newLHS)
                eq.setRHS(newRHS)
	elif isinstance(eq, ConnectEqStm):
	    pass
	elif isinstance(eq, FunCallEqStm):
	    pass
	elif isinstance(eq, ConditionalEqStm):
	    for i in eq.getIfBody():
		self.moveRHSToLHS(i)
	    for i in eq.getElseifList():
		self.moveRHSToLHS(i)
	    for i in eq.getElseBody():
		self.moveRHSToLHS(i)
	elif isinstance(eq, ForEqStm):
	    for i in eq.getBody():
		self.moveRHSToLHS(i)
	elif isinstance(eq, WhenEqStm):
	    for i in eq.getBody():
		self.moveRHSToLHS(i)
	    for i in eq.getElsewhenList():
		self.moveRHSToLHS(i)

    # rule 2: convert constants to real numbers
    def rule2(self):
        for i in self.mainModel.getFlattenedInitEqs():
            self.constToReal(i)
	    #self.convertConstToRealInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.constToReal(i)
	    #self.convertConstToRealInEq(i)

    #def convertConstToRealInEq(self, eq):
	#if isinstance(eq, RegularEqStm):
        #    eq.setLHS(self.convertConstToReal(eq.getLHS()))
        #    eq.setRHS(self.convertConstToReal(eq.getRHS()))
        #else:
        #    raise 'Currently only handles regular equations in \
        #               canonical transformation!'

    # rule 2: convert constants and fractions to real numbers
    #def convertConstToReal(self, exp):
	#if isinstance(exp, IntegerExp):
        #    return RealExp(exp.getLineNum(), exp.getColNum(), \
	#		1.0*exp.getValue())
	#elif isinstance(exp, DivExp):
	#    #print 'DivExp: ', str(exp)
	#    operands=exp.getOperands()
	#    if (isinstance(exp.getOperands()[0], RealExp) or\
        #       isinstance(exp.getOperands()[0], IntegerExp)) and \
        #       (isinstance(exp.getOperands()[1], RealExp) or\
        #       isinstance(exp.getOperands()[1], IntegerExp)):
	#	return RealExp(exp.getLineNum(), exp.getColNum(), \
	#		1.0*exp.getOperands()[0].getValue()/ \
	#		(1.0*exp.getOperands()[1].getValue()))
	#    else:
	#	for i in range(0, len(exp.getOperands())):
	#	    exp.operands[i]=self.convertConstToReal(operands[i])
	#	if isinstance(operands[0], IdentExp) and \
	#	   isinstance(operands[1], IdentExp):
	#	    return exp
	#	else:
	#            return self.convertConstToReal(exp)
	#	#return exp
        #elif isinstance(exp, SumExp) or \
        #     isinstance(exp, SubtractExp) or \
        #     isinstance(exp, MulExp) or \
        #     isinstance(exp, UnaryMinusExp) or \
        #     isinstance(exp, PowerExp):
        #    operands=exp.getOperands()
        #    for index in range(0, len(operands)):
        #        exp.operands[index]=self.convertConstToReal(operands[index])
        #    return exp
        #else:
        #    return exp

    # rule 2: convert constants and fractions to real numbers
    def constToReal(self, eq):
	# iterate over all types of ASTEquations
	if isinstance(eq, RegularEqStm):
	    LHS=eq.getLHS()
	    if isinstance(LHS, IntegerExp):
		newLHS=RealExp(LHS.getLineNum(), LHS.getColNum(), \
			1.0*LHS.getValue())
		eq.setLHS(newLHS)
	    elif isinstance(LHS, DivExp):
		if (isinstance(LHS.getOperands()[0], RealExp) or \
		    isinstance(LHS.getOperands()[0], IntegerExp)) and \
		   (isinstance(LHS.getOperands()[1], RealExp) or \
		    isinstance(LHS.getOperands()[1], IntegerExp)):
		    newLHS=RealExp(LHS.getLineNum(), LHS.getColNum(), \
			1.0*LHS.getOperands()[0].getValue()/ \
			1.0*LHS.getOperands()[1].getValue())
		    eq.setLHS(newLHS)
		else:
		    self.convertToReal(LHS)
	    else:
	    	self.convertToReal(LHS)
	    #self.convertToReal(eq.getRHS())
	    RHS=eq.getRHS()
            if isinstance(RHS, IntegerExp):
                newRHS=RealExp(RHS.getLineNum(), RHS.getColNum(), \
                        1.0*RHS.getValue())
                eq.setRHS(newRHS)
            elif isinstance(RHS, DivExp):
                if (isinstance(RHS.getOperands()[0], RealExp) or \
                    isinstance(RHS.getOperands()[0], IntegerExp)) and \
                   (isinstance(RHS.getOperands()[1], RealExp) or \
                    isinstance(RHS.getOperands()[1], IntegerExp)):
                    newRHS=RealExp(RHS.getLineNum(), RHS.getColNum(), \
                        1.0*RHS.getOperands()[0].getValue()/ \
                        1.0*RHS.getOperands()[1].getValue())
                    eq.setRHS(newRHS)
                else:
                    self.convertToReal(RHS)
            else:
                self.convertToReal(RHS)
	elif isinstance(eq, ConnectEqStm):
	    pass
	elif isinstance(eq, FunCallEqStm):
	    pass
	elif isinstance(eq, ConditionalEqStm):
	    self.convertToReal(eq.getIfTest())
	    for i in eq.getIfBody():
	    	self.constToReal(i)
	    for i in eq.getElseifList():
		self.constToReal(i)
	    for i in eq.getElseBody():
		self.constToReal(i)
	elif isinstance(eq, ForEqStm):
	    for i in eq.getBody():
		self.constToReal(i)
	elif isinstance(eq, WhenEqStm):
	    for i in eq.getBody():
                self.constToReal(i)
	    for i in eq.getElsewhenList():
		self.constToReal(i)
	else:
	    pass

    # method for rule 2
    def convertToReal(self, exp):
	if isinstance(exp, OrExp):
	    for i in exp.getOperands():
		self.convertToReal(i)
	elif isinstance(exp, AndExp):
	    for i in exp.getOperands():
		self.convertToReal(i)
	elif isinstance(exp, NotExp):
	    for i in exp.getOperands():
		self.convertToReal(i)
	elif isinstance(exp, LessExp) or isinstance(exp, LessEQExp) or \
	     isinstance(exp, GreaterExp) or isinstance(exp, GreaterEQExp) or\
	     isinstance(exp, EQEQExp) or isinstance(exp, LessGtExp):
	    self.convertExp(exp)
	elif isinstance(exp, SumExp) or isinstance(exp, MulExp) or \
	     isinstance(exp, SubtractExp) or isinstance(exp, DivExp) or \
	     isinstance(exp, UnaryMinusExp) or isinstance(exp, PowerExp):
	    self.convertExp(exp)
        elif isinstance(exp, RangeExp):
	    self.convertExp(exp)
        elif isinstance(exp, IfElseExp):
	    self.convertToReal(exp.getIfTest())	
	    ifBody=exp.getIfBody()
	    if isinstance(ifBody, IntegerExp):
		newIfBody=RealExp(ifBody.getLineNum(), \
			ifBody.getColNum(), 1.0*ifBody.getValue())
		exp.setIfBody(newIfBody)
	    else:
		self.convertToReal(ifBody)
            for i in exp.getElseifList():
                self.convertToReal(i)
	    elseBody=exp.getElseBody()
            if isinstance(elseBody, IntegerExp):
                newElseBody=RealExp(elseBody.getLineNum(), \
                        elseBody.getColNum(), 1.0*elseBody.getValue())
                exp.setElseBody(newElseBody)
        elif isinstance(exp, FunCallExp):
	    pass
        elif isinstance(exp, FunArgsExp):
	    pass
        elif isinstance(exp, NamedArgExp):
	    pass
        elif isinstance(exp, ConnectRefExp):
	    pass
        elif isinstance(exp, SingleCompRefExp):
	    pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
	elif isinstance(exp, ExpListExp):
	    self.convertExp(exp)
	else:
	    pass

    # rule 2
    def convertExp(self, exp):
	operands=exp.getOperands()
        for i in range(0, len(operands)):
            if isinstance(operands[i], IntegerExp):
                operands[i]=RealExp(operands[i].getLineNum(), \
                    operands[i].getColNum(), 1.0*operands[i].getValue())
            elif isinstance(operands[i], DivExp):
                if (isinstance(operands[i].getOperands()[0], RealExp) or\
                    isinstance(operands[i].getOperands()[0], IntegerExp))\
                    and \
                    (isinstance(operands[i].getOperands()[1], RealExp) or\
                     isinstance(operands[i].getOperands()[1], IntegerExp)):
                    operands[i]=RealExp(operands[i].getLineNum(), \
                        operands[i].getColNum(), \
                        (1.0*operands[i].getOperands()[0].getValue())/ \
                        (1.0*operands[i].getOperands()[1].getValue()))
		else:
		    self.convertToReal(operands[i])
            else:
                self.convertToReal(operands[i])

    # rule 3: rewrite subtraction
    def rule3(self):
    	for i in self.mainModel.getFlattenedInitEqs():
            #self.rewriteSubtraction(i)
	    self.rewriteNegativeTermsInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            #self.rewriteSubtraction(i)
	    self.rewriteNegativeTermsInEq(i)

    def rewriteNegativeTermsInEq(self, eq):
	if isinstance(eq, RegularEqStm):
	    eq.setLHS(self.rewriteNegativeTerms(eq.getLHS()))
	    eq.setRHS(self.rewriteNegativeTerms(eq.getRHS()))
	else:
            raise 'Currently only handles regular equations in \
                        canonical transformation!'

    # rule 3: 
    def rewriteNegativeTerms(self, exp):
        if isinstance(exp, SubtractExp):
            operands=exp.getOperands()
	    newOperand1=operands[0]
	    newOperand2=MulExp(-1, -1, [RealExp(-1,-1,-1.0), operands[1]])
	    newExp=SumExp(-1, -1, [newOperand1, newOperand2])
            return self.rewriteNegativeTerms(newExp)
	elif isinstance(exp, UnaryMinusExp):
	    operands=exp.getOperands()
	    newOperand1=RealExp(-1, -1, -1.0)
	    newOperand2=operands[0]
	    newExp=MulExp(-1, -1, [newOperand1, newOperand2])
	    return self.rewriteNegativeTerms(newExp)
        elif isinstance(exp, SumExp) or \
             isinstance(exp, MulExp) or \
             isinstance(exp, DivExp) or \
             isinstance(exp, PowerExp):
            operands=exp.getOperands()
            index=0
            for index in range(0, len(operands)):
                exp.operands[index]=self.rewriteNegativeTerms(operands[index])
            return exp
        else:
            return exp

    # rule 3: rewrite subtraction
    def rewriteSubtraction(self, eq):
        # iterate over all types of ASTEquations
        if isinstance(eq, RegularEqStm):
	    LHS=eq.getLHS()
	    RHS=eq.getRHS()
	    if isinstance(LHS, SubtractExp):
		operand1=LHS.getOperands()[0]
		operand2=MulExp(-1, -1, [RealExp(-1,-1,-1.0), \
			LHS.getOperands()[1]])
		newLHS=SumExp(-1, -1, [operand1, operand2])
		eq.setLHS(newLHS)
	    elif isinstance(LHS, UnaryMinusExp):
		operand1=RealExp(-1, -1, -1.0)
		operand2=LHS.getOperands()[0]
		newLHS=MulExp(-1, -1, [operand1, operand2])
	        eq.setLHS(newLHS)
            self.rewriteExp(eq.getLHS())
	    if isinstance(RHS, SubtractExp):
                operand1=RHS.getOperands()[0]
		operand2=MulExp(-1, -1, [RealExp(-1,-1,-1.0), \
                        RHS.getOperands()[1]])
                newRHS=SumExp(-1, -1, [operand1, operand2])
		eq.setRHS(newRHS)
	    elif isinstance(LHS, UnaryMinusExp):
		operand1=RealExp(-1, -1, -1.0)
                operand2=LHS.getOperands()[0]
		newLHS=MulExp(-1, -1, [operand1, operand2])
                eq.setRHS(newRHS)
            self.rewriteExp(eq.getRHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            #self.convertToReal(eq.getIfTest())
            for i in eq.getIfBody():
		self.rewriteSubtraction(i)
            for i in eq.getElseifList():
		self.rewriteSubtraction(i)
            for i in eq.getElseBody():
		self.rewriteSubtraction(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
		self.rewriteSubtraction(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
		self.rewriteSubtraction(i)
            for i in eq.getElsewhenList():
		self.rewriteSubtraction(i)
        else:
            pass
	
    # for rule 3
    def rewriteExp(self, exp):
        # iterate over all types of ASTExp
        if isinstance(exp, IntegerExp):
	    pass
        elif isinstance(exp, OrExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, AndExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, NotExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, LessExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, LessEQExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, GreaterExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, GreaterEQExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, EQEQExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, LessGtExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, SumExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, MulExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, SubtractExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, DivExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, UnaryMinusExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, PowerExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, RangeExp):
            pass
        elif isinstance(exp, IfElseExp):
            self.convertSubtractToSum(exp.getIfTest())
	    self.convertSubtractToSum(exp.getIfBody())
            for i in exp.getElseifList():
		self.convertSubtractToSum(i)
	    self.convertSubtractToSum(exp.getElseBody())
        elif isinstance(exp, FunCallExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, FunArgsExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, NamedArgExp):
	    self.convertSubtractToSum(exp)
        elif isinstance(exp, ConnectRefExp):
            pass
        elif isinstance(exp, SingleCompRefExp):
            pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
        elif isinstance(exp, ExpListExp):
	    self.convertSubtractToSum(exp)
        else:
            pass

    # for rule 3
    def convertSubtractToSum(self, exp):
	index=0
	for i in exp.getOperands():
	    if isinstance(i, SubtractExp):
		operand1=i.getOperands()[0]
		#operand2=UnaryMinusExp(-1, -1, [i.getOperands()[1]])
		operand2=MulExp(-1, -1, [RealExp(-1,-1,-1.0), \
                        i.getOperands()[1]])
		newI=SumExp(-1, -1, [operand1, operand2])
		exp.operands[index]=newI
		self.rewriteExp(newI)
	    elif isinstance(i, UnaryMinusExp):
		operand1=RealExp(-1, -1, -1.0)
		operand2=i.getOperands()[0]
		newI=MulExp(-1, -1, [operand1, operand2])
		exp.operands[index]=newI
		self.rewriteExp(newI)
	    else:
		self.rewriteExp(i)
	    index=index+1

    # rule 4: rewrite reciprocals
    def rule4(self):
	for i in self.mainModel.getFlattenedInitEqs():
            self.rewriteReciprocal(i)
        for i in self.mainModel.getFlattenedEqs():
            self.rewriteReciprocal(i)

    # rule 4: rewrite reciprocals
    def rewriteReciprocal(self, eq):
        # iterate over all types of ASTEquations
        if isinstance(eq, RegularEqStm):
            LHS=eq.getLHS()
            RHS=eq.getRHS()
            if isinstance(LHS, DivExp):
                operand1=LHS.getOperands()[0]
                operand2=PowerExp(-1, -1, [LHS.getOperands()[1], \
				  RealExp(-1,-1, -1.0)])
                newLHS=MulExp(-1, -1, [operand1, operand2])
                eq.setLHS(newLHS)
            self.rewriteExpR(eq.getLHS())
	    if isinstance(RHS, DivExp):
                operand1=RHS.getOperands()[0]
                operand2=PowerExp(-1, -1, [RHS.getOperands()[1], \
                                  RealExp(-1,-1, -1.0)])
                newRHS=MulExp(-1, -1, [operand1, operand2])
                eq.setRHS(newRHS)
            self.rewriteExpR(eq.getRHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            #self.convertToReal(eq.getIfTest())
            for i in eq.getIfBody():
                self.rewriteReciprocal(i)
            for i in eq.getElseifList():
		self.rewriteReciprocal(i)
            for i in eq.getElseBody():
		self.rewriteReciprocal(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
		self.rewriteReciprocal(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
		self.rewriteReciprocal(i)
            for i in eq.getElsewhenList():
		self.rewriteReciprocal(i)
        else:
            pass

    # for rule 4
    def rewriteExpR(self, exp):
        # iterate over all types of ASTExp
        if isinstance(exp, IntegerExp):
            pass
        elif isinstance(exp, OrExp):
            self.convertDivToMul(exp)
        elif isinstance(exp, AndExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, NotExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, LessExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, LessEQExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, GreaterExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, GreaterEQExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, EQEQExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, LessGtExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, SumExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, MulExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, SubtractExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, DivExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, UnaryMinusExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, PowerExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, RangeExp):
            pass
        elif isinstance(exp, IfElseExp):
            self.convertDivToMul(exp.getIfTest())
	    self.convertDivToMul(exp.getIfBody())
            for i in exp.getElseifList():
		self.convertDivToMul(i)
	    self.convertDivToMul(exp.getElseBody())
        elif isinstance(exp, FunCallExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, FunArgsExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, NamedArgExp):
	    self.convertDivToMul(exp)
        elif isinstance(exp, ConnectRefExp):
            pass
        elif isinstance(exp, SingleCompRefExp):
            pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
        elif isinstance(exp, ExpListExp):
	    self.convertDivToMul(exp)
        else:
            pass

    # for rule 4
    def convertDivToMul(self, exp):
        index=0
        for i in exp.getOperands():
            if isinstance(i, DivExp):
                operand1=i.getOperands()[0]
                operand2=PowerExp(-1, -1, [i.getOperands()[1], \
				  RealExp(-1, -1, -1.0)])
                newI=MulExp(-1, -1, [operand1, operand2])
                exp.operands[index]=newI
                self.rewriteExpR(newI)
            else:
                self.rewriteExpR(i)
            index=index+1

    # rule 5
    def rule5(self):
	for i in self.mainModel.getFlattenedInitEqs():
            self.convertMulOp(i)
        for i in self.mainModel.getFlattenedEqs():
            self.convertMulOp(i)

    # for rule 5
    def convertMulOp(self, eq):
	# iterate over all types of ASTEquations
        if isinstance(eq, RegularEqStm):
	    self.convertMulExp(eq.getLHS())
	    self.convertMulExp(eq.getRHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            #self.convertToReal(eq.getIfTest())
            for i in eq.getIfBody():
                self.convertMulOp(i)
            for i in eq.getElseifList():
		self.convertMulOp(i)
            for i in eq.getElseBody():
		self.convertMulOp(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
		self.convertMulOp(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
		self.convertMulOp(i)
            for i in eq.getElsewhenList():
		self.convertMulOp(i)
        else:
            pass

    # for rule 5
    def convertMulExp(self, exp):
	if isinstance(exp, MulExp):
	    operands=exp.getOperands()
	    for i in operands:
		if not isinstance(i, MulExp):
		    pass
		else:
		    self.convertMulExp(i)
		    # need add new methods for this stm
		    exp.operands=exp.operands+i.getOperands()
		    #opStr=''
		    #for i in exp.operands:
			#opStr=opStr+str(i)+','
		    #print 'before: ', opStr
	    # delete all MulExps in operands list
	    index=len(exp.getOperands())-1
            while index>=0:
            	if isinstance(exp.operands[index], MulExp):
		    #print 'bingo: ', str(exp.operands[index])
               	    del exp.operands[index]
            	index=index-1
	    #opStr=''
            #for i in exp.operands:
            #    opStr=opStr+str(i)+','
            #print 'after: ', opStr
	    for i in exp.getOperands():
                self.convertMulExp(i)
	elif isinstance(exp, IfElseExp):
	    self.convertMulExp(exp.getIfTest())
            self.convertMulExp(exp.getIfBody())
            for i in exp.getElseifList():
                self.convertMulExp(i)
            self.convertMulExp(exp.getElseBody())
	elif isinstance(exp, IdentExp) or isinstance(exp, RealExp) or \
		isinstance(exp, IntegerExp) or isinstance(exp, BoolExp) \
		or isinstance(exp, StringExp) or \
		isinstance(exp, SingleCompRefExp) or \
		isinstance(exp, CompRefExp) or isinstance(exp, NameExp): 
	    return 
	else:
	    for i in exp.getOperands():
		self.convertMulExp(i)

    # rule 6
    def rule6(self):
	for i in self.mainModel.getFlattenedInitEqs():
            self.convertPlusOp(i)
        for i in self.mainModel.getFlattenedEqs():
            self.convertPlusOp(i)

    # for rule 6
    def convertPlusOp(self, eq):
        # iterate over all types of ASTEquations
        if isinstance(eq, RegularEqStm):
            self.convertSumExp(eq.getLHS())
            self.convertSumExp(eq.getRHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            #self.convertToReal(eq.getIfTest())
            for i in eq.getIfBody():
                self.convertPlusOp(i)
            for i in eq.getElseifList():
                self.convertPlusOp(i)
            for i in eq.getElseBody():
                self.convertPlusOp(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
                self.convertPlusOp(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
                self.convertPlusOp(i)
            for i in eq.getElsewhenList():
                self.convertPlusOp(i)
        else:
            pass

    # for rule 6
    def convertSumExp(self, exp):
        operands=exp.getOperands()
        if isinstance(exp, SumExp):
            for i in operands:
                if not isinstance(i, SumExp):
                    # what if in the case a*(b*c) where (b*c) is ExpListExp
                    pass
                else:
                    self.convertSumExp(i)
                    # need add new methods for this stm
                    exp.operands=exp.operands+i.getOperands()
            # delete all MulExps in operands list
            index=len(exp.getOperands())-1
            while index>=0:
                if isinstance(exp.operands[index], SumExp):
                    del exp.operands[index]
                index=index-1
            for i in exp.getOperands():
                self.convertSumExp(i)
	elif isinstance(exp, IfElseExp):
            self.convertSumExp(exp.getIfTest())
            self.convertSumExp(exp.getIfBody())
            for i in exp.getElseifList():
                self.convertSumExp(i)
            self.convertSumExp(exp.getElseBody())
        elif isinstance(exp, IdentExp) or isinstance(exp, RealExp) or \
                isinstance(exp, IntegerExp) or isinstance(exp, BoolExp) \
                or isinstance(exp, StringExp) or \
                isinstance(exp, SingleCompRefExp) or \
                isinstance(exp, CompRefExp) or isinstance(exp, NameExp):
            return
        else:
            for i in exp.getOperands():
                self.convertSumExp(i)

    # ----------------rule 7.0 is no longer used --------------------
    # rule 7.0:
    def rule7_0(self):
	for i in self.mainModel.getFlattenedInitEqs():
            self.evalConstPowerInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.evalConstPowerInEq(i)

    # rule 7.0: constant folding in PowerExp
    def evalConstPowerInEq(self, eq):
	# iterate over all types of ASTEquations
        if isinstance(eq, RegularEqStm):
            LHS=eq.getLHS()
            RHS=eq.getRHS()
            if isinstance(LHS, PowerExp):
		operands=LHS.getOperands()
            	if len(operands)==1:
                    return
            	elif (isinstance(operands[0], RealExp) or \
                    isinstance(operands[0], IntegerExp)) and \
                    (isinstance(operands[1], RealExp) or \
                    isinstance(operands[1], IntegerExp)):
                    power=pow(operands[0].getValue(), operands[1].getValue())
		    eq.setLHS(RealExp(-1,-1,power))
            	elif (isinstance(operands[0], RealExp) or \
                    isinstance(operands[0], IntegerExp)) and \
                    isinstance(operands[1], UnaryMinusExp):
                    opnd=operands[1].getOperands()[0]
                    if isinstance(opnd, RealExp) or \
                       isinstance(opnd, IntegerExp):
                        power=pow(operands[0].getValue(), opnd.getValue())
			eq.setLHS(RealExp(-1,-1,power))
            	elif (isinstance(operands[0], RealExp) or \
                    isinstance(operands[0], IntegerExp)) and \
                    isinstance(operands[1], UnaryMinusExp):
                    opnd=operands[1].getOperands()[0]
                    if isinstance(opnd, RealExp) or \
                       isinstance(opnd, IntegerExp):
                        power=pow(operands[0].getValue(), opnd.getValue())
		        eq.setLHS(RealExp(-1,-1,power))
	    else:
            	self.evalConstPowerInExp(eq.getLHS())
            	#self.rewriteExpR(eq.getRHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            #self.convertToReal(eq.getIfTest())
            for i in eq.getIfBody():
                self.evalConstPowerInEq(i)
            for i in eq.getElseifList():
                self.evalConstPowerInEq(i)
            for i in eq.getElseBody():
                self.evalConstPowerInEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
                self.evalConstPowerInEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
                self.evalConstPowerInEq(i)
            for i in eq.getElsewhenList():
                self.evalConstPowerInEq(i)
        else:
            pass

    # for rule 7.0
    def evalConstPowerInExp(self, exp):
	# iterate over all types of ASTExp
        if isinstance(exp, IntegerExp):
            pass
        elif isinstance(exp, OrExp):
            self.evalConstPower(exp)
        elif isinstance(exp, AndExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, NotExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, LessExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, LessEQExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, GreaterExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, GreaterEQExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, EQEQExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, LessGtExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, SumExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, MulExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, SubtractExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, DivExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, UnaryMinusExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, PowerExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, RangeExp):
            pass
	elif isinstance(exp, IfElseExp):
            self.evalConstPower(exp.getIfTest())
            self.evalConstPower(exp.getIfBody())
            for i in exp.getElseifList():
                self.evalConstPower(i)
            self.evalConstPower(exp.getElseBody())
        elif isinstance(exp, FunCallExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, FunArgsExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, NamedArgExp):
	    self.evalConstPower(exp)
        elif isinstance(exp, ConnectRefExp):
            pass
        elif isinstance(exp, SingleCompRefExp):
            pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
        elif isinstance(exp, ExpListExp):
	    self.evalConstPower(exp)
        else:
            pass

    # for rule 7.0
    def evalConstPower(self, exp):
        index=0
        for i in exp.getOperands():
            if isinstance(i, PowerExp):
		operands=i.getOperands()
                if len(operands)==1:
                    return
                elif (isinstance(operands[0], RealExp) or \
                    isinstance(operands[0], IntegerExp)) and \
                    (isinstance(operands[1], RealExp) or \
                    isinstance(operands[1], IntegerExp)):
                    power=pow(operands[0].getValue(), operands[1].getValue())
                    exp.operands[index]=RealExp(-1,-1,power)
                elif (isinstance(operands[0], RealExp) or \
                    isinstance(operands[0], IntegerExp)) and \
                    isinstance(operands[1], UnaryMinusExp):
                    opnd=operands[1].getOperands()[0]
                    if isinstance(opnd, RealExp) or \
                       isinstance(opnd, IntegerExp):
                        power=pow(operands[0].getValue(), opnd.getValue())
                        exp.operands[index]=RealExp(-1,-1,power)
                elif (isinstance(operands[0], RealExp) or \
                    isinstance(operands[0], IntegerExp)) and \
                    isinstance(operands[1], UnaryMinusExp):
                    opnd=operands[1].getOperands()[0]
                    if isinstance(opnd, RealExp) or \
                       isinstance(opnd, IntegerExp):
                        power=pow(operands[0].getValue(), opnd.getValue())
                        exp.operands[index]=RealExp(-1,-1,power)
            else:
                self.evalConstPowerInExp(i)
            index=index+1
    # ---------------rule 7.0 is no longer used (end)------------------

    # rule 7.1
    def rule7_1(self):
	for i in self.mainModel.getFlattenedInitEqs():
            self.foldConstantInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.foldConstantInEq(i)

    # for rule 7.1
    def foldConstantInEq(self, eq):
        # iterate over all types of ASTEquations
        if isinstance(eq, RegularEqStm):
            self.foldConstantInExp(eq.getLHS())
            self.foldConstantInExp(eq.getRHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            #self.convertToReal(eq.getIfTest())
            for i in eq.getIfBody():
                self.foldConstantInEq(i)
            for i in eq.getElseifList():
                self.foldConstantInEq(i)
            for i in eq.getElseBody():
                self.foldConstantInEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
                self.foldConstantInEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
                self.foldConstantInEq(i)
            for i in eq.getElsewhenList():
                self.foldConstantInEq(i)
        else:
            pass

    # for rule 7.1
    def foldConstantInExp(self, exp):
	if isinstance(exp, SumExp):
	    operands=exp.getOperands()
	    sum=0.0
	    for i in operands:
		if isinstance(i, RealExp) or isinstance(i, IntegerExp):
		    sum=sum+i.getValue()
		elif isinstance(i, UnaryMinusExp):
		    opnd=i.getOperands()[0]
		    if isinstance(opnd, RealExp) or \
		       isinstance(opnd, IntegerExp):	
			sum=sum-opnd.getValue()
		    else:
			self.foldConstantInExp(opnd)
		else:
		    self.foldConstantInExp(i)
	    # delete all constants in operands list
            index=len(operands)-1
	    deleted=False
            while index>=0:
                if isinstance(operands[index], RealExp) or \
			isinstance(operands[index], IntegerExp):
                    del operands[index]
		    deleted=True
		elif isinstance(operands[index], UnaryMinusExp):
		    opnd=operands[index].getOperands()[0]
		    if isinstance(opnd, RealExp) or \
                       isinstance(opnd, IntegerExp):
			del operands[index]
			deleted=True
                index=index-1
	    # insert sum
	    if deleted==True:
	    	exp.getOperands().append(RealExp(-1,-1,sum))
	elif isinstance(exp, MulExp):
	    operands=exp.getOperands()
	    product=1.0
	    for i in operands:
		if isinstance(i, RealExp) or isinstance(i, IntegerExp):
		    product=product*i.getValue()
		elif isinstance(i, UnaryMinusExp):
		    opnd=i.getOperands()[0]
		    if isinstance(opnd, RealExp) or \
                       isinstance(opnd, IntegerExp):
			product=product*(-1.0)*opnd.getValue()
		    else:
			self.foldConstantInExp(opnd)
		else:
		    self.foldConstantInExp(i)
	    # delete all constants in operands list
            index=len(operands)-1
	    deleted=False
            while index>=0:
                if isinstance(operands[index], RealExp) or \
                        isinstance(operands[index], IntegerExp):
                    del operands[index]
		    deleted=True
                elif isinstance(operands[index], UnaryMinusExp):
                    opnd=operands[index].getOperands()[0]
                    if isinstance(opnd, RealExp) or \
                       isinstance(opnd, IntegerExp):
                        del operands[index]
			deleted=True
                index=index-1
            # insert product
	    if deleted==True:
            	exp.getOperands().append(RealExp(-1,-1,product))
	elif isinstance(exp, PowerExp):
	    # a constant power is converted to 2.0^3.0 -> 8.0^1.0
	    operands=exp.getOperands()
	    if isinstance(operands[0], RealExp):
		if isinstance(operands[1], RealExp):
		    operands[0]=RealExp(-1,-1,pow(operands[0].getValue(), \
						  operands[1].getValue()))
		    operands[1]=RealExp(-1, -1, 1.0)
		# convert eg., 2^(2*3*a) -> 32^a
		elif isinstance(operands[1], MulExp):
		    # assume that constants have been folded in MulExp
		    index=0
		    for i in operands[1].getOperands():
			if isinstance(i, RealExp):
			    operands[0]=RealExp(-1, -1, \
				pow(operands[0].getValue(), i.getValue()))
			    del operands[1].operands[index]
			index=index+1
		    self.foldConstantInExp(operands[1])
		else:
		    self.foldConstantInExp(operands[1])
	    else:
		self.foldConstantInExp(operands[0])
		self.foldConstantInExp(operands[1])
	elif isinstance(exp, IdentExp) or isinstance(exp, RealExp) or \
                isinstance(exp, IntegerExp) or isinstance(exp, BoolExp) \
                or isinstance(exp, StringExp) or isinstance(exp, FunCallExp)\
                or isinstance(exp, SingleCompRefExp) or \
                isinstance(exp, CompRefExp) or isinstance(exp, NameExp):
            return
	else:
	    operands=exp.getOperands()
	    for i in operands:
		self.foldConstantInExp(i)

    # rule 7.2
    def rule7_2(self):
	for i in self.mainModel.getFlattenedInitEqs():
            self.removeZerosAndOnesInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.removeZerosAndOnesInEq(i)

    # rule 7.2
    def removeZerosAndOnesInEq(self, eq):
	# iterate over all types of ASTEquations
        if isinstance(eq, RegularEqStm):
            self.removeZerosAndOnesInExp(eq.getLHS())
            self.removeZerosAndOnesInExp(eq.getRHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            #self.convertToReal(eq.getIfTest())
            for i in eq.getIfBody():
		self.removeZerosAndOnesInEq(i)
            for i in eq.getElseifList():
		self.removeZerosAndOnesInEq(i)
            for i in eq.getElseBody():
		self.removeZerosAndOnesInEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
		self.removeZerosAndOnesInEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
		self.removeZerosAndOnesInEq(i)
            for i in eq.getElsewhenList():
		self.removeZerosAndOnesInEq(i)
        else:
            pass

    # rule 7.2
    def removeZerosAndOnesInExp(self, exp):
	# delete all zeros operands
	if isinstance(exp, SumExp):
            operands=exp.getOperands()
	    if len(operands)==1:
		return 
            # delete all zeros in operands list
            index=len(operands)-1
            while index>=0:
                if isinstance(operands[index], RealExp) or \
                        isinstance(operands[index], IntegerExp):
		    if operands[index].getValue()==0:
                    	del operands[index]
                #elif isinstance(operands[index], UnaryMinusExp):
                #    opnd=operands[index].getOperands()[0]
                #    if isinstance(opnd, RealExp) or \
                #       isinstance(opnd, IntegerExp):
                #        del operands[index]
		else:
		    self.removeZerosAndOnesInExp(operands[index])
                index=index-1
	elif isinstance(exp, MulExp):
	    # delete all ones (not -1.0) operands
            operands=exp.getOperands()
	    if len(operands)==1:
                return
            index=len(operands)-1
            while index>=0:
                if isinstance(operands[index], RealExp) or \
                        isinstance(operands[index], IntegerExp):
		    # 1.0*E -> E
		    if operands[index].getValue()==1.0:
                    	del operands[index]
		    # 0.0*E -> 0.0
		    elif operands[index].getValue()==0.0:
			exp.setOperands([RealExp(-1,-1,0.0)])
                #elif isinstance(operands[index], UnaryMinusExp):
                #    opnd=operands[index].getOperands()[0]
                #    if isinstance(opnd, RealExp) or \
                #       isinstance(opnd, IntegerExp):
                #        del operands[index]
		else:
		    self.removeZerosAndOnesInExp(operands[index])
                index=index-1
	elif isinstance(exp, IdentExp) or isinstance(exp, RealExp) or \
                isinstance(exp, IntegerExp) or isinstance(exp, BoolExp) \
                or isinstance(exp, StringExp) or isinstance(exp, FunCallExp)\
                or isinstance(exp, SingleCompRefExp) or \
                isinstance(exp, CompRefExp) or isinstance(exp, NameExp):
            return
        else:
            operands=exp.getOperands()
            for i in operands:
                self.removeZerosAndOnesInExp(i)

    # rule 7.3 remove superfluous in PowerExp
    def rule7_3(self):
        for i in self.mainModel.getFlattenedInitEqs():
            self.removeZerosAndOnes2InEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.removeZerosAndOnes2InEq(i)

    # rule 7.3
    def removeZerosAndOnes2InEq(self, eq):
        # iterate over all types of ASTEquations
        if isinstance(eq, RegularEqStm):
            self.removeZerosAndOnes2InExp(eq.getLHS())
            #self.removeZerosAndOnes2InExp(eq.getRHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            #self.removeZerosAndOnes2InExp(eq.getIfTest())
            for i in eq.getIfBody():
                self.removeZerosAndOnes2InEq(i)
            for i in eq.getElseifList():
                self.removeZerosAndOnes2InEq(i)
            for i in eq.getElseBody():
                self.removeZerosAndOnes2InEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
                self.removeZerosAndOnes2InEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
                self.removeZerosAndOnes2InEq(i)
            for i in eq.getElsewhenList():
                self.removeZerosAndOnes2InEq(i)
        else:
            pass
    
    # rule 7.3
    def removeZerosAndOnes2InExp(self, exp):
        if isinstance(exp, IdentExp) or isinstance(exp, RealExp) or \
                isinstance(exp, IntegerExp) or isinstance(exp, BoolExp) \
                or isinstance(exp, StringExp) or isinstance(exp, FunCallExp)\
                or isinstance(exp, SingleCompRefExp) or \
                isinstance(exp, CompRefExp) or isinstance(exp, NameExp):
            return
        else:
            self.removeZerosAndOnesInPowerExp(exp)

    # rule 7.3
    def removeZerosAndOnesInPowerExp(self, exp):
	index=0
        for i in exp.getOperands():
            operands=i.getOperands()
            if isinstance(i, PowerExp):
                if isinstance(operands[0], RealExp):
		    # this is to fold c^1.0 where c is constant
		    # which is produced by the prevous rule
		    if isinstance(operands[1], RealExp) and \
		       operands[1].getValue()==1.0:
			exp.operands[index]=RealExp(-1, -1, \
					    operands[0].getValue())
			continue
		    if operands[0].getValue()==0.0:
		    	if isinstance(operands[1], RealExp):
			    # 0.0^c -> 0.0 where c>0 is constant
			    if operands[1].getValue()>0.0:
			    	exp.operands[index]=RealExp(-1, -1, 0.0)
			    # 0.0^-c -> zero division error, 
			    # where c>0 is constant
			    elif operands[1].getValue()<0.0:
			    	raise 'zero division error!'
			    # what if 0.0^0.0
			    else:
			    	pass
		    elif operands[0].getValue()==1.0:
			# 1.0^E -> 1.0 where E is an exp
			exp.operands[index]=RealExp(-1, -1, 1.0)
		    else:
			self.removeZerosAndOnes2InExp(operands[1])
		elif isinstance(operands[1], RealExp):
		    if operands[1].getValue()==1.0:
		        # E^1.0 ->E where E is an exp
			exp.operands[index]=operands[0]
		    elif operands[1].getValue()==0.0:
		        # E^0.0 ->1 where E is an exp
		        exp.operands[index]=RealExp(-1, -1, 1)
		else:
                    self.removeZerosAndOnes2InExp(operands[0])
		    self.removeZerosAndOnes2InExp(operands[1])
            else:
                self.removeZerosAndOnes2InExp(i)
            index=index+1


    # rule 7.4 convert unary Sum and Mul exps
    def rule7_4(self):
	for i in self.mainModel.getFlattenedInitEqs():
            self.convertUnaryMulAndSumInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.convertUnaryMulAndSumInEq(i)

    # rule 7.4
    def convertUnaryMulAndSumInEq(self, eq):
	if isinstance(eq, RegularEqStm):
	    LHS=eq.getLHS()
	    operands=LHS.getOperands()
	    if isinstance(LHS, SumExp) or isinstance(LHS, MulExp):
		if len(operands)==1:
		    eq.setLHS(operands[0])
		else:
		    self.convertUnaryMulAndSumInExp(LHS)
	    else:
		self.convertUnaryMulAndSumInExp(LHS)
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            #self.convertToReal(eq.getIfTest())
            for i in eq.getIfBody():
		self.convertUnaryMulAndSumInEq(i)
            for i in eq.getElseifList():
		self.convertUnaryMulAndSumInEq(i)
            for i in eq.getElseBody():
		self.convertUnaryMulAndSumInEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
		self.convertUnaryMulAndSumInEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
		self.convertUnaryMulAndSumInEq(i)
            for i in eq.getElsewhenList():
		self.convertUnaryMulAndSumInEq(i)
        else:
            pass

    # rule 7.4
    def convertUnaryMulAndSumInExp(self, exp):
	if isinstance(exp, IntegerExp):
            pass
        elif isinstance(exp, OrExp):
            self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, AndExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, NotExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, LessExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, LessEQExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, GreaterExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, GreaterEQExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, EQEQExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, LessGtExp):
	    self.convertUnaryMulAndSum(exp)
	elif isinstance(exp, SumExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, MulExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, SubtractExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, DivExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, UnaryMinusExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, PowerExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, RangeExp):
            pass
	elif isinstance(exp, IfElseExp):
            self.convertUnaryMulAndSum(exp.getIfTest())
            self.convertUnaryMulAndSum(exp.getIfBody())
            for i in exp.getElseifList():
                self.convertUnaryMulAndSum(i)
            self.convertUnaryMulAndSum(exp.getElseBody())
        elif isinstance(exp, FunCallExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, FunArgsExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, NamedArgExp):
	    self.convertUnaryMulAndSum(exp)
        elif isinstance(exp, ConnectRefExp):
            pass
        elif isinstance(exp, SingleCompRefExp):
            pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
        elif isinstance(exp, ExpListExp):
	    self.convertUnaryMulAndSum(exp)
        else:
            pass

    # rule 7.4
    def convertUnaryMulAndSum(self, exp):
	index=0
        for i in exp.getOperands():
            operands=i.getOperands()
	    if isinstance(i, SumExp) or isinstance(i, MulExp):
                if len(operands)==1:
                    exp.operands[index]=i.getOperands()[0]
		    self.convertUnaryMulAndSumInExp(exp.operands[index])
		else:
		    self.convertUnaryMulAndSumInExp(i)
            else:
                self.convertUnaryMulAndSumInExp(i)
            index=index+1


    # rule 8: sort Sum and Mul exps
    def rule8(self):
        for i in self.mainModel.getFlattenedInitEqs():
            self.sortInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.sortInEq(i)

    # rule 8
    def sortInEq(self, eq):
        if isinstance(eq, RegularEqStm):
	    self.sortInExp(eq.getLHS())
	    #self.sortInExp(eq.getRHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            self.sortInExp(eq.getIfTest())
            for i in eq.getIfBody():
		self.sortInEq(i)
            for i in eq.getElseifList():
		self.sortInEq(i)
            for i in eq.getElseBody():
		self.sortInEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
		self.sortInEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
		self.sortInEq(i)
            for i in eq.getElsewhenList():
		self.sortInEq(i)
        else:
            pass

    # rule 8
    def sortInExp(self, exp):
        if isinstance(exp, OrExp) or \
           isinstance(exp, AndExp) or \
           isinstance(exp, NotExp) or \
           isinstance(exp, LessExp) or \
           isinstance(exp, LessEQExp) or \
           isinstance(exp, GreaterExp) or \
           isinstance(exp, GreaterEQExp)or \
           isinstance(exp, EQEQExp) or \
           isinstance(exp, LessGtExp):
	    for i in exp.getOperands():
		self.sortInExp(i)
        elif isinstance(exp, SumExp):
	    for i in exp.getOperands():
                self.sortInExp(i)
	    operands=exp.getOperands()
	    strOperands=[]
	    for i in operands:
		strOperands.append(str(i))
	    strOperands.sort()
	    index=0
	    newOperands=[]
	    foundPositions=[]
	    for j in strOperands:
		for k in range(0, len(operands)):
		    if j==str(operands[k]):
			if not k in foundPositions:
			    newOperands.append(operands[k])
			    foundPositions.append(k)
		index=index+1
	    exp.setOperands(newOperands)
	    #for i in exp.getOperands():
	#	self.sortInExp(i)
        elif isinstance(exp, MulExp):
	    for i in exp.getOperands():
                self.sortInExp(i)
	    operands=exp.getOperands()
            strOperands=[]
            for i in operands:
                strOperands.append(str(i))
            strOperands.sort()
            index=0
            newOperands=[]
            foundPositions=[]
            for j in strOperands:
                for k in range(0, len(operands)):
                    if j==str(operands[k]):
                        if not k in foundPositions:
                            newOperands.append(operands[k])
                            foundPositions.append(k)
                index=index+1
	    # move the constant to the end of a MulExp
	    # assume that constants have been folded (only one RealExp 
	    # in newOperands)
	    index=0
	    for i in newOperands:
		if isinstance(newOperands[index], RealExp):
		    newOperands.append(newOperands[index])
		    del newOperands[index]
		    break
		index=index+1
	    #if isinstance(newOperands[0], RealExp):
		#newOperands.append(newOperands[0])
		#newOperands=newOperands[1:len(newOperands)]
            exp.setOperands(newOperands)
            #for i in exp.getOperands():
            #    self.sortInExp(i)
        elif isinstance(exp, SubtractExp) or \
             isinstance(exp, DivExp) or \
             isinstance(exp, UnaryMinusExp) or \
             isinstance(exp, PowerExp) or \
             isinstance(exp, RangeExp):
            for i in exp.getOperands():
                self.sortInExp(i)
	elif isinstance(exp, IfElseExp):
            self.sortInExp(exp.getIfTest())
            self.sortInExp(exp.getIfBody())
            for i in exp.getElseifList():
                self.sortInExp(i)
            self.sortInExp(exp.getElseBody())
        elif isinstance(exp, FunCallExp) or \
             isinstance(exp, FunArgsExp) or \
             isinstance(exp, NamedArgExp) or \
	     isinstance(exp, ExpListExp):
            for i in exp.getOperands():
                self.sortInExp(i)
	elif isinstance(exp, ConnectRefExp):
	    pass
        elif isinstance(exp, SingleCompRefExp):
            pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
        else:
            pass

    # rule 9: collect like terms in SumExp
    def rule9(self):
        for i in self.mainModel.getFlattenedInitEqs():
            self.collectLikeTermsInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.collectLikeTermsInEq(i)

    # rule 9
    def collectLikeTermsInEq(self, eq):
	if isinstance(eq, RegularEqStm):
	    self.collectLikeTermsInExp(eq.getLHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            self.collectLikeTermsInExp(eq.getIfTest())
            for i in eq.getIfBody():
		self.collectLikeTermsInEq(i)
            for i in eq.getElseifList():
		self.collectLikeTermsInEq(i)
            for i in eq.getElseBody():
		self.collectLikeTermsInEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
		self.collectLikeTermsInEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
		self.collectLikeTermsInEq(i)
            for i in eq.getElsewhenList():
		self.collectLikeTermsInEq(i)
        else:
            pass

    # rule 9
    def collectLikeTermsInExp(self, exp):
	if isinstance(exp, SumExp):
	    operands=exp.getOperands()
	    newOperands=[]
	    #for i in operands:
		#self.collectLikeTermsInExp(i)
	    if len(operands)==1:
		return 
	    for i in range(0, len(operands)-1):
		newTerm=self.collectLikeTerms(operands[i], operands[i+1])
		if not newTerm==None:
		    if not i==len(operands)-2:
		    	operands[i+1]=newTerm
		    else:
		    	newOperands.append(newTerm)
		else:
		    newOperands.append(operands[i])
		    if i==len(operands)-2:
			newOperands.append(operands[len(operands)-1])
		    #del operands[i]
	    exp.setOperands(newOperands)
	    for i in exp.getOperands():
                self.collectLikeTermsInExp(i)
	elif isinstance(exp, OrExp) or \
           isinstance(exp, AndExp) or \
           isinstance(exp, NotExp) or \
           isinstance(exp, LessExp) or \
           isinstance(exp, LessEQExp) or \
           isinstance(exp, GreaterExp) or \
           isinstance(exp, GreaterEQExp)or \
           isinstance(exp, EQEQExp) or \
           isinstance(exp, LessGtExp):
            for i in exp.getOperands():
		self.collectLikeTermsInExp(i)
	elif isinstance(exp, SubtractExp) or \
	     isinstance(exp, MulExp) or \
             isinstance(exp, DivExp) or \
             isinstance(exp, UnaryMinusExp) or \
             isinstance(exp, PowerExp) or \
             isinstance(exp, RangeExp):
            for i in exp.getOperands():
		self.collectLikeTermsInExp(i)
	elif isinstance(exp, IfElseExp):
            self.collectLikeTermsInExp(exp.getIfTest())
            self.collectLikeTermsInExp(exp.getIfBody())
            for i in exp.getElseifList():
		self.collectLikeTermsInExp(i)
            self.collectLikeTermsInExp(exp.getElseBody())
        elif isinstance(exp, FunCallExp) or \
             isinstance(exp, FunArgsExp) or \
             isinstance(exp, NamedArgExp) or \
             isinstance(exp, ExpListExp):
            for i in exp.getOperands():
		self.collectLikeTermsInExp(i)
        elif isinstance(exp, ConnectRefExp):
            pass
        elif isinstance(exp, SingleCompRefExp):
            pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
        else:
            pass

    # rule 9
    def collectLikeTerms(self, term1, term2):
	#if not (isinstance(term1, MulExp) and isinstance(term2, MulExp)):
	#    return None
	# term1 or term2 can be IdentExp or CompRefExp, eg., a+2*a
	operands1=term1.getOperands()
	operands2=term2.getOperands()
	#if len(operands1)==1 and len(operands2)==1:
	if (isinstance(term1, IdentExp) or isinstance(term1, CompRefExp)) and\
	   (isinstance(term2, IdentExp) or isinstance(term2, CompRefExp)):
	    # eg., x+x
	    if str(term1)==str(term2):
		return MulExp(-1,-1,[RealExp(-1, -1, 2.0), term2])
	    else:
		return None
	elif isinstance(term1, PowerExp) and isinstance(term2, PowerExp):
	    # eg., x^2+x^2
	    if str(term1)==str(term2):
		return MulExp(-1,-1,[term2, RealExp(-1,-1,2.0)])
	    else:
		return None
	elif (isinstance(term1, IdentExp) or isinstance(term1, CompRefExp)) \
             and (isinstance(term2, MulExp) and len(operands2)==2):
	    # eg., x+x*2.0
	    if str(operands1[0])==str(operands2[0]):
		if isinstance(operands2[1], RealExp):
		    newOperand=MulExp(-1, -1, \
                        [RealExp(-1,-1,operands2[1].getValue()+1), \
                        operands2[0]])
		elif isinstance(operands2[1], IdentExp):
		    pass # check if operands2[0] refers to parameter
                elif isinstance(operands2[1], CompRefExp):
		    # check if operands2[0] refers to parameter
		    pass
		else:
		    newOperand=None
		return newOperand
	elif isinstance(term1, PowerExp) and isinstance(term2, MulExp):
	    # eg., a^2+a^2*3.0
	    # sorting will always place a^2 before a^2*3.0
	    if str(term1)==str(operands2[0]):
		if isinstance(operands2[1], RealExp):
		    return MulExp(-1, -1, [operands2[0], \
                        RealExp(-1,-1,operands2[1].getValue()+1)])
		# elif: case of parameter
		else:
		    return None
	    else:
		return None
	elif isinstance(term1, MulExp) and isinstance(term2, MulExp):
	    if len(operands1)==2 and len(operands2)==2:
		if str(operands1[0])==str(operands2[0]):
		    if isinstance(operands1[1], RealExp) and \
		       isinstance(operands2[1], RealExp):
			return MulExp(-1, -1, [operands2[0], RealExp(-1,-1, \
                          operands1[1].getValue()+operands2[1].getValue())])
	elif (len(operands1)==2 and len(operands2)==2) and \
		(isinstance(term1, MulExp) and isinstance(term2, MulExp)):
	    if str(operands1[1])==str(operands2[1]) and \
	       isinstance(operands1[0], RealExp) and \
	       isinstance(operands2[0], RealExp):
		return MulExp(-1, -1, [RealExp(-1, -1, \
			operands1[0].getValue()+operands2[0].getValue()), \
                        operands2[1]])
	# need to consider the case of 2*a*b^2+5*a*b^2
	else:
	    return None


    # rule 10: combine power of the same base, eg, x^p*x^q ->x^(p+q)
    def rule10(self):
        for i in self.mainModel.getFlattenedInitEqs():
            self.combineProductOfPowersInEq(i)
        for i in self.mainModel.getFlattenedEqs():
	    self.combineProductOfPowersInEq(i)

    # rule 10
    def combineProductOfPowersInEq(self, eq):
	if isinstance(eq, RegularEqStm):
	    self.combineProductOfPowersInExp(eq.getLHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            self.combineProductOfPowersInExp(eq.getIfTest())
            for i in eq.getIfBody():
		self.combineProductOfPowersInEq(i)
            for i in eq.getElseifList():
		self.combineProductOfPowersInEq(i)
            for i in eq.getElseBody():
		self.combineProductOfPowersInEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
		self.combineProductOfPowersInEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
		self.combineProductOfPowersInEq(i)
            for i in eq.getElsewhenList():
		self.combineProductOfPowersInEq(i)
        else:
            pass

 
    # rule 10
    def combineProductOfPowersInExp(self, exp):
	if isinstance(exp, MulExp):
	    operands=exp.getOperands()
	    newOperands=[]
	    #for i in operands:
		#self.combineProductOfPowersInExp(i)
	    if len(operands)==1:
		return
	    for i in range(0, len(operands)-1):
		newTerm=self.combineProductOfPowers(operands[i], \
			operands[i+1])
		if not newTerm==None:
                    if not i==len(operands)-2:
                        operands[i+1]=newTerm
                    else:
                        newOperands.append(newTerm)
		else:
		    newOperands.append(operands[i])
		    if i==len(operands)-2:
			newOperands.append(operands[len(operands)-1])
		    #del operands[i]
	    exp.setOperands(newOperands)
	    for i in exp.getOperands():
                self.combineProductOfPowersInExp(i)
	elif isinstance(exp, OrExp) or \
           isinstance(exp, AndExp) or \
           isinstance(exp, NotExp) or \
           isinstance(exp, LessExp) or \
           isinstance(exp, LessEQExp) or \
           isinstance(exp, GreaterExp) or \
           isinstance(exp, GreaterEQExp)or \
           isinstance(exp, EQEQExp) or \
           isinstance(exp, LessGtExp):
            for i in exp.getOperands():
		self.combineProductOfPowersInExp(i)
	elif isinstance(exp, SubtractExp) or \
	     isinstance(exp, SumExp) or \
             isinstance(exp, DivExp) or \
             isinstance(exp, UnaryMinusExp) or \
             isinstance(exp, PowerExp) or \
             isinstance(exp, RangeExp):
            for i in exp.getOperands():
		self.combineProductOfPowersInExp(i)
	elif isinstance(exp, IfElseExp):
            self.combineProductOfPowersInExp(exp.getIfTest())
            self.combineProductOfPowersInExp(exp.getIfBody())
            for i in exp.getElseifList():
		self.combineProductOfPowersInExp(i)
            self.combineProductOfPowersInExp(exp.getElseBody())
        elif isinstance(exp, FunCallExp) or \
             isinstance(exp, FunArgsExp) or \
             isinstance(exp, NamedArgExp) or \
             isinstance(exp, ExpListExp):
            for i in exp.getOperands():
                self.combineProductOfPowersInExp(i)
        elif isinstance(exp, ConnectRefExp):
            pass
        elif isinstance(exp, SingleCompRefExp):
            pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
        else:
            pass


    # rule 10
    def combineProductOfPowers(self, term1, term2):
	if not (((isinstance(term1, IdentExp) or \
		isinstance(term1, CompRefExp) or \
		isinstance(term1, PowerExp)) and \
		(isinstance(term2, IdentExp) or \
		isinstance(term2, CompRefExp) or \
		isinstance(term2, PowerExp)))):
	    return None
	operands1=term1.getOperands()
	operands2=term2.getOperands()
	if len(operands1)==1 and len(operands2)==1:
	    # eg., a*a
	    if str(operands1[0])==str(operands2[0]):
		return PowerExp(-1,-1,[operands2[0], RealExp(-1,-1,2.0)])
	elif len(operands1)==2 and len(operands2)==1:
	    # eq., a^2*a
	    if isinstance(term1, PowerExp) and \
	       str(operands1[0])==str(operands2[0]):
		newOperand=PowerExp(-1, -1, [operands2[0], \
			SumExp(-1,-1,[RealExp(-1,-1,1.0), operands1[1]])])
	    else:
	        newOperand=None
	    return newOperand
	elif len(operands1)==1 and len(operands2)==2:
	    # eq., a*a^2
            if isinstance(term2, PowerExp) and \
               str(operands1[0])==str(operands2[0]):
                newOperand=PowerExp(-1, -1, [operands2[0], \
                        SumExp(-1,-1,[RealExp(-1,-1,1.0), operands2[1]])])
            else:
                newOperand=None
            return newOperand
	elif len(operands1)==2 and len(operands2)==2:
	    if (isinstance(term1, PowerExp) and isinstance(term2, PowerExp))\
		and str(operands1[0])==str(operands2[0]):
		return PowerExp(-1, -1, [operands2[0], \
			SumExp(-1,-1, [operands1[1], operands2[1]])])
	else:
	    return None

    # rule 11: flatten the power of a power, eg, (x^a)^b ->x^(a*b)
    def rule11(self):
        for i in self.mainModel.getFlattenedInitEqs():
            self.flattenPowerOfPowerInEq(i)
        for i in self.mainModel.getFlattenedEqs():
	    self.flattenPowerOfPowerInEq(i)

    # rule 11
    def flattenPowerOfPowerInEq(self, eq):
        if isinstance(eq, RegularEqStm):
            self.flattenPowerOfPowerInExp(eq.getLHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            self.flattenPowerOfPowerInExp(eq.getIfTest())
            for i in eq.getIfBody():
		self.flattenPowerOfPowerInEq(i)
            for i in eq.getElseifList():
		self.flattenPowerOfPowerInEq(i)
            for i in eq.getElseBody():
		self.flattenPowerOfPowerInEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
		self.flattenPowerOfPowerInEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
		self.flattenPowerOfPowerInEq(i)
            for i in eq.getElsewhenList():
		self.flattenPowerOfPowerInEq(i)
        else:
            pass

    # rule 11
    def flattenPowerOfPowerInExp(self, exp):
        if isinstance(exp, PowerExp):
            operands=exp.getOperands()
	    if isinstance(exp.getOperands()[0], PowerExp):
		newOperand1=operands[0].getOperands()[0]
		newOperand2=MulExp(-1,-1,[operands[0].getOperands()[1], \
			operands[1]])
		newOperands=[newOperand1, newOperand2]
		exp.setOperands(newOperands)
		for i in exp.getOperands():
		    self.flattenPowerOfPowerInExp(i)
	    else:
		for i in operands:
		    self.flattenPowerOfPowerInExp(i)
        elif isinstance(exp, OrExp) or \
           isinstance(exp, AndExp) or \
           isinstance(exp, NotExp) or \
           isinstance(exp, LessExp) or \
           isinstance(exp, LessEQExp) or \
           isinstance(exp, GreaterExp) or \
           isinstance(exp, GreaterEQExp)or \
           isinstance(exp, EQEQExp) or \
           isinstance(exp, LessGtExp):
            for i in exp.getOperands():
		self.flattenPowerOfPowerInExp(i)
	elif isinstance(exp, SubtractExp) or \
             isinstance(exp, SumExp) or \
	     isinstance(exp, MulExp) or \
             isinstance(exp, DivExp) or \
             isinstance(exp, UnaryMinusExp) or \
             isinstance(exp, RangeExp):
            for i in exp.getOperands():
		self.flattenPowerOfPowerInExp(i)
        elif isinstance(exp, IfElseExp):
            self.flattenPowerOfPowerInExp(exp.getIfTest())
            self.flattenPowerOfPowerInExp(exp.getIfBody())
            for i in exp.getElseifList():
		self.flattenPowerOfPowerInExp(i)
            self.flattenPowerOfPowerInExp(exp.getElseBody())
        elif isinstance(exp, FunCallExp) or \
             isinstance(exp, FunArgsExp) or \
             isinstance(exp, NamedArgExp) or \
             isinstance(exp, ExpListExp):
            for i in exp.getOperands():
		self.flattenPowerOfPowerInExp(i)
        elif isinstance(exp, ConnectRefExp):
            pass
        elif isinstance(exp, SingleCompRefExp):
            pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
        else:
            pass


    # rule 12: flatten the power of a product, eg, (x*y)^b ->x^b*y^b
    def rule12(self):
        for i in self.mainModel.getFlattenedInitEqs():
            self.flattenPowerOfProductInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.flattenPowerOfProductInEq(i)

    # rule 12
    def flattenPowerOfProductInEq(self, eq):
        if isinstance(eq, RegularEqStm):
	    lhs=eq.getLHS()
	    if isinstance(lhs, PowerExp):
		operands=lhs.getOperands()
		if isinstance(operands[0], MulExp):
		    newOperands=[]
		    for i in operands[0].getOperands():
			newOperand=PowerExp(-1,-1,[i, operands[1]])
			newOperands.append(newOperand)
		    newExp=MulExp(-1,-1,newOperands)
		    eq.setLHS(newExp)
		    self.flattenPowerOfProductInExp(eq.getLHS())
		else:
		    self.flattenPowerOfProductInExp(lhs)
	    else:
		self.flattenPowerOfProductInExp(lhs)
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            self.flattenPowerOfProductInExp(eq.getIfTest())
            for i in eq.getIfBody():
                self.flattenPowerOfProductInEq(i)
            for i in eq.getElseifList():
                self.flattenPowerOfProductInEq(i)
            for i in eq.getElseBody():
                self.flattenPowerOfProductInEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
                self.flattenPowerOfProductInEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
                self.flattenPowerOfProductInEq(i)
            for i in eq.getElsewhenList():
                self.flattenPowerOfProductInEq(i)
        else:
            pass

    # rul 12
    def flattenPowerOfProductInExp(self, exp):
        if isinstance(exp, OrExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, AndExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, NotExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, LessExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, LessEQExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, GreaterExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, GreaterEQExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, EQEQExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, LessGtExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, SumExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, MulExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, SubtractExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, DivExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, UnaryMinusExp):
	    self.flattenPowerOfProduct(exp)
        elif isinstance(exp, PowerExp):
	    self.flattenPowerOfProduct(exp)
	elif isinstance(exp, RangeExp):
            pass
        elif isinstance(exp, IfElseExp):
            self.flattenPowerOfProduct(exp.getIfTest())
            self.flattenPowerOfProduct(exp.getIfBody())
            for i in exp.getElseifList():
                self.flattenPowerOfProduct(i)
            self.flattenPowerOfProduct(exp.getElseBody())
        elif isinstance(exp, FunCallExp):
            self.flattenPowerOfProduct(exp)
        elif isinstance(exp, FunArgsExp):
            self.flattenPowerOfProduct(exp)
        elif isinstance(exp, NamedArgExp):
            self.flattenPowerOfProduct(exp)
        elif isinstance(exp, ConnectRefExp):
            pass
        elif isinstance(exp, SingleCompRefExp):
            pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
        elif isinstance(exp, ExpListExp):
            self.flattenPowerOfProduct(exp)
        else:
            pass

    # rule 12
    def flattenPowerOfProduct(self, exp):
        index=0
        for i in exp.getOperands():
	    if isinstance(i, PowerExp):
		operands=i.getOperands()
                if isinstance(operands[0], MulExp):
                    newOperands=[]
                    for j in operands[0].getOperands():
                        newOperand=PowerExp(-1,-1,[j, operands[1]])
                        newOperands.append(newOperand)
		    # create a new method for this: setOperandAt(index, op)
                    exp.operands[index]=MulExp(-1,-1,newOperands)
                    self.flattenPowerOfProductInExp(exp.operands[index])
		else:
		    self.flattenPowerOfProductInExp(i)
	    else:
		self.flattenPowerOfProductInExp(i)
            index=index+1


    # rule 13: distribute MulExp, eg., c*(t1+t2+...+tn) -> c*t1+c*t2+...c*tn
    def rule13(self):
        for i in self.mainModel.getFlattenedInitEqs():
            self.distributeMulExpInEq(i)
        for i in self.mainModel.getFlattenedEqs():
            self.distributeMulExpInEq(i)

    # rule 13
    def distributeMulExpInEq(self, eq):
	if isinstance(eq, RegularEqStm):
            self.distributeMulExpInExp(eq.getLHS())
        elif isinstance(eq, ConnectEqStm):
            pass
        elif isinstance(eq, FunCallEqStm):
            pass
        elif isinstance(eq, ConditionalEqStm):
            self.distributeMulExpInExp(eq.getIfTest())
            for i in eq.getIfBody():
                self.distributeMulExpInEq(i)
            for i in eq.getElseifList():
                self.distributeMulExpInEq(i)
            for i in eq.getElseBody():
                self.distributeMulExpInEq(i)
        elif isinstance(eq, ForEqStm):
            for i in eq.getBody():
                self.distributeMulExpInEq(i)
        elif isinstance(eq, WhenEqStm):
            for i in eq.getBody():
                self.distributeMulExpInEq(i)
            for i in eq.getElsewhenList():
                self.distributeMulExpInEq(i)
        else:
            pass

    # rul 13
    def distributeMulExpInExp(self, exp):
	if isinstance(exp, MulExp):
            operands=exp.getOperands()
	    numOfOperands=len(operands)-1
	    lastOperand=operands[numOfOperands]
            #newOperands=[]
            if len(operands)==1 or \
		(not isinstance(lastOperand, RealExp)):
                return
	    index=numOfOperands-1
	    while index>=0:
                if isinstance(operands[index], SumExp):
		    newOperands=[]
        	    for i in operands[index].getOperands():
            		newOperand=MulExp(-1, -1, [i, lastOperand])
            		newOperands.append(newOperand)
		    newTerm=SumExp(-1, -1, newOperands)
		    operands[index]=newTerm
		    del operands[numOfOperands]
		    break
                index=index-1
            for i in exp.getOperands():
                self.distributeMulExpInExp(i)
        elif isinstance(exp, OrExp) or \
           isinstance(exp, AndExp) or \
           isinstance(exp, NotExp) or \
           isinstance(exp, LessExp) or \
           isinstance(exp, LessEQExp) or \
           isinstance(exp, GreaterExp) or \
           isinstance(exp, GreaterEQExp)or \
           isinstance(exp, EQEQExp) or \
           isinstance(exp, LessGtExp):
            for i in exp.getOperands():
                self.distributeMulExpInExp(i)
	elif isinstance(exp, SubtractExp) or \
             isinstance(exp, SumExp) or \
             isinstance(exp, DivExp) or \
             isinstance(exp, UnaryMinusExp) or \
             isinstance(exp, PowerExp) or \
             isinstance(exp, RangeExp):
            for i in exp.getOperands():
                self.distributeMulExpInExp(i)
        elif isinstance(exp, IfElseExp):
            self.distributeMulExpInExp(exp.getIfTest())
            self.distributeMulExpInExp(exp.getIfBody())
            for i in exp.getElseifList():
                self.distributeMulExpInExp(i)
            self.distributeMulExpInExp(exp.getElseBody())
        elif isinstance(exp, FunCallExp) or \
             isinstance(exp, FunArgsExp) or \
             isinstance(exp, NamedArgExp) or \
             isinstance(exp, ExpListExp):
            for i in exp.getOperands():
                self.distributeMulExpInExp(i)
        elif isinstance(exp, ConnectRefExp):
            pass
        elif isinstance(exp, SingleCompRefExp):
            pass
        elif isinstance(exp, CompRefExp):
            pass
        elif isinstance(exp, ArrSubExp):
            pass
        else:
            pass

    # rule 13
    # sum: ASTExp::SumExp
    # const: ASTExp::RealExp
    def distributeMulExp(self, sum, const):
	newOperands=[]
	for i in sum.getOperands():
	    newOperand=MulExp(-1, -1, [i, const])
	    newOperands.append(newOperand)
	return SumExp(-1, -1, newOperands)

