# MuModelica Compiler Project
# Author: Steven Xu
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University

from AST.ASTExp import *
from BackEnd.printers.EqExpPrinter import EqExpPrinter

class EquationRewriter:
    def __init__(self): # mainModelP, graphP, strongComponentsP):
        #self.mainModel=mainModelP
	#self.graph=graphP # FlowGraph
	#self.strongComponents=strongComponentsP
	#self.printer=PrettyPrinter3()
	pass

    # strComp: [] of strong components from loop detection
    def go(self, strComp):
	#eqPrinter=PrettyPrinter3()
	index=len(strComp)-1
	while index >=0:
	    comp=strComp[index]
	    # algebraic loops: will not be rewritten in causal form
	    if len(comp)>1:
		pass
	    # equations not involved in any algebraic loop are 
	    # rewritten in explicit form
	    else:
		varNode=comp[0].getVarNode()
		eq=comp[0].getEqNode().getEq()
		# if the variable occurs in the LHS more than twice, then
                # it is not able to rewrite the equation into causal form
                if self.occurrenceInExp(varNode.getDefinition(), \
			eq.getLHS())>1:
                    print 'Not able to rewrite equation into causal form: '+\
                        str(varNode)+' in '+str(eq)
		    # should call an DAE solver here
                    #continue
		else:
		    self.rewrite(varNode, eq)
	    index=index-1
	return strComp

    # v: AST::Exp (IdentExp, CompRefExp, FunCallExp)
    # varNode: VarNode
    def rewrite(self, varNode, eq):
	#print "rewriting: ", str(eq)
	lhs=eq.getLHS()
	rhs=eq.getRHS()
	v=self.lookupVar(varNode, lhs)
	if lhs==v:
	    return
	elif isinstance(lhs, SumExp):
	    newOps=[rhs]
	    for i in lhs.getOperands():
		if self.lookupVar(varNode, i)==None:
		    newOp=MulExp(-1,-1,[RealExp(-1,-1,-1.0), i])
		    newOps.append(newOp)
		else:
		    eq.setLHS(i)
	    newRHS=SumExp(-1,-1,newOps)
	    eq.setRHS(newRHS)
	    self.rewrite(varNode, eq)
	elif isinstance(lhs, MulExp):
	    newOps=[rhs]
	    for i in lhs.getOperands():
		if self.lookupVar(varNode, i)==None:
		    newOp=PowerExp(-1,-1,[i, RealExp(-1,-1,-1.0)])
		    newOps.append(newOp)
		else:
		    eq.setLHS(i)
	    newRHS=MulExp(-1,-1,newOps)
            eq.setRHS(newRHS)
	    self.rewrite(varNode, eq)
	elif isinstance(lhs, PowerExp):
	    raise 'rewritting PowerExp in equations not yet complete ...'
	else:
	    print 'Not able to rewrite equation into causal form: ', \
                str(varNode), str(eq)
	    raise ''

    # v: DeclarationEntry
    def occurrenceInExp(self, v, exp):
	occurrence=0
	# assume that all CompRefExp have been transformed to IdentExp
	if isinstance(exp, IdentExp):
	    if exp.getValue()==v:
		occurrence=occurrence+1
		return occurrence
        elif isinstance(exp, SumExp) or \
	     isinstance(exp, MulExp) or \
             isinstance(exp, UnaryMinusExp) or \
             isinstance(exp, PowerExp):
            for i in exp.getOperands():
		occurrence=occurrence+self.occurrenceInExp(v, i)
	# need loop over all cases 
	return occurrence

    # look up v in exp
    # return the IdentExp or FunCallExp that contains v
    # v: varNode
    def lookupVar(self, v, exp):
	found=None
	# assume that all CompRefExp have been transformed to IdentExp
	if isinstance(exp, IdentExp):
	    if v.integralCausality==False:
		if exp.getValue()==v.getDefinition():
		    return exp
	elif isinstance(exp, FunCallExp):
	    operands=exp.getOperands()
	    if str(operands[0])=='der':
		if v.integralCausality==True:
		    # this only handles the 'der' operator currently
		    args=operands[1].getOperands()
		    if len(args)!=1:
		    	raise 'der operator only has 1 argument!'
		    else:
		    	if isinstance(args[0], IdentExp):
		    	    if args[0].getValue()==v.getDefinition():
			    	return exp
	    else:
		args=operands[1].getOperands()
		for i in args:
		    found=self.lookupVar(v, i)
		    if found!=None:
			return found
		#raise 'Not able to rewrite equations with functions ...'
	elif isinstance(exp, SumExp) or \
             isinstance(exp, MulExp) or \
             isinstance(exp, UnaryMinusExp) or \
             isinstance(exp, PowerExp):
            for i in exp.getOperands():
                found=self.lookupVar(v, i)
                if found!=None:
                    return found
	return found

    # dump rewritten equations (in explicit form if possible)
    def dump(self, strComp):
	msg=""
	printer=EqExpPrinter()
	algLoopNum=0
	index=len(strComp)-1
        while index >=0:
            comp=strComp[index]
            # algebraic loops: will not be rewritten in causal form
            if len(comp)>1:
                algLoopNum=algLoopNum+1
                for dNode in comp:
                    #print 'Algebraic Loop '+str(algLoopNum)+': ', \
                    #    str(dNode.getEqNode().getEq())
		    msg=msg+'Algebraic Loop '+str(algLoopNum)+': '+ \
			str(dNode.getEqNode().getEq())+"\n"
            else:
                eq=comp[0].getEqNode().getEq()
                #print printer.dumpEq(eq)
		msg=msg+printer.dumpEq(eq)+"\n"
            index=index-1
	return msg

