# MuModelica Compiler Project
# Author: Steven Xu
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University

from BackEnd.causality.Node import Node

class Graph:
    def __init__(self):
	raise 'abstact class Graph not instantiable ...'

class FlowGraph(Graph):

    source=None
    sink=None
    eqNodes=None
    varNodes=None
    isSinkInL=False

    def __init__(self):
	#self.source=Node('s')
	#self.sink=Node('t')
	self.eqNodes={}
	self.varNodes={}

    def getSource(self):
	return self.source

    def getSink(self):
	return self.sink

    # add an EqNode
    def addEqNode(self, id, eqNode):
	self.eqNodes[id]=eqNode

    # add an VarNode
    def addVarNode(self, id, varNode):
        self.varNodes[id]=varNode

    def deleteVarNode(self, idP):
	del self.varNodes[idP]

    def getEqNodes(self):
	result=[]
        keys=self.eqNodes.keys()
        for i in keys:
            eqNode=self.eqNodes[i]
            result.append(eqNode)
        return result

    def getVarNodes(self):
	result=[]
        keys=self.varNodes.keys()
        for i in keys:
            varNode=self.varNodes[i]
            result.append(varNode)
        return result

    def getVarNodesWithDerForm(self):
	result=[]
	keys=self.varNodes.keys()
	for i in keys:
	    varNode=self.varNodes[i]
	    if varNode.hasDerivativeForm==True:
		result.append(varNode)
	return result

    def getAllNodes(self):
	return [self.source]+self.getEqNodes()+self.getVarNodes() \
		+[self.sink]

    # reset the graph in such a way that all its nodes and edges 
    # are set to have default value
    def reset(self):
	for e in self.source.getAllOutgoingEdgesInG():
	    e.reset()
	for node in self.getEqNodes():
	    for e in node.getAllOutgoingEdgesInG():
		e.reset()
	for node in self.getVarNodes():
	    if node.hasDerivativeForm:
	        node.integralCausality=True
	    else:
		node.integralCausality=False
	    for e in node.getAllOutgoingEdgesInG():	
		e.reset()
	for e in self.sink.getAllOutgoingEdgesInG():
            e.reset()

    def varNodeLookup(self, id):
	if id in self.varNodes.keys():
	    return self.varNodes[id]
	return None

    def __str__(self):
	result='source: '+str(self.source)+'\n  Outgoing Edges: '
	for i in self.source.getAllOutgoingEdgesInG():
	    result=result+' '+str(i.destination.getId())+','
	result=result+'\nEqNodes: '
	for i in self.getEqNodes():
	    result=result+'\n  '+i.getId()+': '+str(i)+'\n'
	    result=result+'     Outgoing Edges: '
	    for j in i.getAllOutgoingEdgesInG():
		result=result+' '+str(j.destination.getId())+','
	result=result+'\nVarNodes: '
	for i in self.getVarNodes():
	    result=result+'\n  '+i.getId()+': '+str(i)+'\n'
	    result=result+'     Outgoing Edges: '
	    for j in i.getAllOutgoingEdgesInG():
		result=result+' '+str(j.destination.getId())+','
	return result


class DependencyGraph(Graph):

    nodes=None

    def __init__(self):
	self.nodes={}

    def addNode(self, idP, nodeP):
	if idP in self.nodes.keys():
	    return
	else:
            self.nodes[idP]=nodeP

    def getNode(self, idP):
	if idP in self.nodes.keys():
	    return self.nodes[idP]
	return None

    def getAllNodes(self):
        result=[]
        keys=self.nodes.keys()
        for i in keys:
            node=self.nodes[i]
            result.append(node)
        return result

    # return the highest numbered unvisited node
    def getHighestOrderNumberedNode(self):
	keys=self.nodes.keys()
	currentHighest=0
	result=None
	for i in keys:
	    node=self.nodes[i]
	    if not node.isVisited():
		if node.getOrderNumber()>currentHighest:
		    currentHighest=node.getOrderNumber()
		    result=self.nodes[i]
	return result

    # return True if all nodes are visited
    def isAllVisited(self):
	result=True
	keys=self.nodes.keys()
	for key in keys:
	    node=self.nodes[key]
	    if not node.isVisited():
		result=False
		return result
	return result
