# MuModelica Compiler Project
# Author: Steven Xu
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University

from AST.ASTExp import *
from BackEnd.causality.Graph import DependencyGraph
#from BackEnd.causality.Edge import DependencyEdge
from BackEnd.causality.Node import DNode
from FrontEnd.printers.EqExpPrinter import EqExpPrinter

# sort equations into correct computation order
class Sorter:

    dfsCounter=1

    def __init__(self):
	self.msg=""

    def go(self, G):
	D1=self.buildDependencyGraph(G)
	D2=self.topSort(D1)
	#print '--------------- Sorting ------------------'
	self.msg=self.msg+'--------------- Sorting ------------------\n'
	self.msg=self.msg+self.dumpSortingResult(D2)+"\n"
	#print '----------- Algebraic Loops --------------'
	self.msg=self.msg+'----------- Algebraic Loops --------------\n'
	strComps=self.strongComponents(D2)
	#print self.dumpStrongComponents(strComps)
	self.msg=self.msg+self.dumpAlgebraicLoops(strComps)+"\n"
	return strComps

    # G: FlowGraph
    def buildDependencyGraph(self, G):
	D=DependencyGraph()
	# create dependency nodes
	for eqNode in G.getEqNodes():
	    # it has been validated in CausalityAssigner that 
	    # each eq is used to compute only one var 
	    # get the varNode that will appear on the LHS of the eq
	    varNode=eqNode.getAllSaturatedEdgesInG()[0].getDestination()
	    dNode=DNode(varNode.getId(), varNode, eqNode)
	    D.addNode(dNode.getId(), dNode)
	# add dependencies (edges)
	for d in D.getAllNodes():
	    eqNode=d.getEqNode()
	    for e in eqNode.getAllUnsaturatedEdgesInG():
		destination=e.getDestination()
		id=destination.getId()
		if id!='0':
		    dNode=D.getNode(id)
		    #D.addNode(id, destination)
		    #var.addDependencyEdge(id, DependencyEdge(destination))
		    d.addDependency(id, dNode) 
	return D

    # perform a topological sort on a depenedncy graph
    def topSort(self, D):
	for dNode in D.getAllNodes():
	    dNode.setVisited(False)
	for dNode in D.getAllNodes():
	    if not dNode.isVisited():
		self.dfsLabelling(dNode)
	return D

    def dfsLabelling(self, node):
	if not node.isVisited():
	    node.setVisited(True)
	    for dNode in node.getAllDependencies():
		self.dfsLabelling(dNode)
	    node.setOrderNumber(self.dfsCounter)
	    self.dfsCounter=self.dfsCounter+1

    # detect algebraic loops. D: DependencyGraph, topsorted with numbering
    def strongComponents(self, D):
	# produce a new graph with all edges rversed
	R=self.reverseEdges(D)
	# start with an empty list of strong components
	strongComponents=[]
	# mark all nodes as unvisited
	for node in R.getAllNodes():
	    node.setVisited(False)
	# As strong components are discovered and are added to the stong 
	# component list, they will be removed from R.
	# The algorithm terminates when R is reduced to empty
	while not R.isAllVisited():
	    startNode=R.getHighestOrderNumberedNode()
	    component=self.dfsCollect(startNode)
	    strongComponents.append(component)
	    # removing the identified stong component is done by
	    # labelling the nodes as visited
	return strongComponents

    def reverseEdges(self, D):
	for node in D.getAllNodes():
	    for dNode in node.getAllDependencies():
		dNode.addReverseDependency(node.getId(), node)
	return D

    def dfsCollect(self, node):
	component=[]
        if not node.isVisited():
            node.setVisited(True)
	    component.append(node)
            for neighbor in node.getAllReverseDependencies():
                component=component+self.dfsCollect(neighbor)
	return component

    def sortStrongComponents(self, strComp):
	pass

    def dumpDependencyGraph(self, D):
	result=''
	for i in D.getAllNodes():
	    result=result+'\n  '+i.getId()+'('+str(i.getOrderNumber())\
			+')'+': '
	    for j in i.getAllDependencies():
		result=result+j.getId()+', '
	return result

    # D: DependencyGraph
    def dumpSortingResult(self, D):
	printer=EqExpPrinter()
	orderList=[]
	for i in range(0, len(D.getAllNodes())):
	    orderList.append(None)
	#order=1
	for i in D.getAllNodes():
	    orderList[i.getOrderNumber()-1]=i
	result=''
	for i in orderList:
	    result=result+'\n  '+'('+str(i.getOrderNumber()) +') ' \
		+ printer.dumpEq(i.getEqNode().getEq())
		#+str(i) #+': '
	    #for j in i.getAllDependencies():
		#result=result+j.getId()+', '
	return result

    # list: [] of strong components
    def dumpStrongComponents(self, list):
	result=''
        for i in list:
	    result=result+'  ['
            for j in i:
		result=result+str(j.getVarNode())+', '
	    result=result[0:len(result)-2]+']\n'
	return result

    # list: [] of strong components
    def dumpAlgebraicLoops(self, list):
	printer=EqExpPrinter()
	loopIndex=1
	result=''
	for i in list:
	    if len(i)<=1:
		# equations not involved in a loop are not dumped
		continue
	    else:
		result=result+'Algebraic Loop: '+str(loopIndex)+'\n'
		for j in i:
		    result=result+'  '+j.getEqNode().getId()+': '+\
			printer.dumpEq(j.getEqNode().getEq())+'\n'
		loopIndex=loopIndex+1
	return result
