# MuModelica Compiler Project
# Author: Steven Xu
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University


from AST.ASTExp import *
from AST.ASTEquations import *
from AST.ASTScope import DeclarationEntry, SymbolTable

# ---------------------------------------------------------------------
class OctaveGenerator:
    " Generate Octave code."
    def __init__(self):
	pass

    # flatClassDef: the flate Modelica class
    # (ASTDefinition::RegularClassDefinition)
    # strComps: [] of strComponents 
    def go(self, flatClassDef, strComps, fileName):
	# prompt for input o time parameters
	code='time_init=input("Please enter initial time: ");\n'
	code=code+'time_end=input("Please enter end time: ");\n'
	code=code+'time_step=input("Please enter time step: ");\n'
	code=code+'num_of_intervals=(time_end - time_init)/time_step;\n'
	code=code+"time = linspace(time_init, time_end, num_of_intervals)';\n"
	# constants
	code=code+'# Constants \n'
	for decl in flatClassDef.getFlatScope().getConstantDecl():
	    varName=str(decl.getName())
	    value=str(decl.getEnv()['value'])
	    code=code+'global '+varName+'_last='+value+'\n'
	# paramenters 
	code=code+'# Parameters \n'
	for decl in flatClassDef.getFlatScope().getParameterDecl():
	    varName=str(decl.getName())
	    value=str(decl.getEnv()['value'])
	    code=code+'global '+varName+'_last='+value+'\n'
	    code=code+varName+'_last=input("enter the value of parameter '+\
		varName+'('+value+'): ");\n'
	# generate global vars declarations
	code=code+'# Variables \n'
	varList=[]
	for i in strComps:
	    #if len(i)>1:
		# loops
		#pass
	    #else:
	    for dNode in i:
		varNode=dNode.getVarNode()
		varName=str(varNode.getDefinition().getName())
		varList.append(varName)
		decl='global '+varName+'=zeros(num_of_intervals, 1)'
		code=code+decl+';\n'
		# prompt for input of initial values
		#code=code+varName+'(1)=input("Please enter initial value of '\
                #        +varName+': ");\n'
		lastValue=varName+'_last='+varName+'(1)'
		code=code+'global '+lastValue+';\n'
		if varNode.hasDerivativeForm:
		    derVarName='der_'+varName
		    varList.append(derVarName)
		    decl='global '+derVarName+'=zeros(num_of_intervals, 1)'
		    code=code+decl+';\n'
		    # prompt for input of initial values
		    #prompt=varName+'(1)=input("Please enter initial value of '\
			#+derVarName+': ")'
		    #code=code+prompt+';\n'
		    lastValue=derVarName+'_last='+derVarName+'(1)'
		    code=code+lastValue+';\n'
	# generate functions to calculate defferential equations and/or 
	# algebraic loops
	loopNum=1
	for compList in strComps:
	    # algebraic-loop
	    if len(compList)>1:
		# map vars to indexes of function parameter (vector)
		index=1
        	map={}
        	for dNode in compList:
            	    varNode=dNode.getVarNode()
            	    if varNode.hasDerivativeForm and varNode.integralCausality:
                	varName='der_'+dNode.getId()
            	    else:
                	varName=dNode.getId()
            	    map[varName]=index
            	    index=index+1
		funCode=self.genNonLinearEqFunction(compList, map, loopNum)
		code=code+'\n'+funCode
		loopNum=loopNum+1
	    # integration
	    else:
		dNode=compList[0]
		varNode=dNode.getVarNode()
		eqNode=dNode.getEqNode()
		if varNode.hasDerivativeForm and varNode.integralCausality:
		    funCode=self.generateDOTFunnction(dNode)
		    code=code+'\n'+funCode
	# generate the for-loop
	code=code+'\nfor i=2:num_of_intervals\n'
	index=len(strComps)-1
	eqNum=1
        while index >=0:
	    code=code+'  # Equation '+str(eqNum)+'\n'
	    compList=strComps[index]
	    loopNum=1
	    if len(compList)>1:
		# algebraic loops
		# all initial guess value for solving algebraic loops are 0s
		guess='init_guess=zeros('+str(len(compList))+', 1);\n'
		code=code+'  '+guess
		# call fsolve to solve the loop
		code=code+'  [loop'+str(loopNum)+\
		  ', info]=fsolve("f_loop'+str(loopNum)+'", init_guess);\n'
		# assign the values to vars and update the _last value
		j=1
		for dNode in compList:
		    varNode=dNode.getVarNode()
                    if varNode.hasDerivativeForm and varNode.integralCausality:
                        varName='der_'+dNode.getId()
		    else:
                        varName=dNode.getId()
		    code=code+'  '+varName+'(i)=loop'+str(loopNum)+\
			'('+str(j)+');\n'
		    # update the var_last value
		    code=code+'  '+varName+'_last='+varName+'(i);\n'
		    j=j+1
		loopNum=loopNum+1
	    else:
		varNode=compList[0].getVarNode()
		varName=str(varNode.getDefinition().getName())
		eq=compList[0].getEqNode().getEq()
		lhs=eq.getLHS()
		if isinstance(lhs, FunCallExp):
		    if str(lhs.getOperands()[0])=='der':
			strLhs='der_'+varName
		else:
		    strLhs=varName
		strEq=strLhs+'(i)='+self.dumpExp(eq.getRHS())
		code=code+'  '+strEq+';\n'
		# update the var_last value
		code=code+'  '+strLhs+'_last='+strLhs+'(i);\n'
		# call the 'lsode' solver to calculate the integration
		if varNode.hasDerivativeForm and varNode.integralCausality:
		    code=code+'  time_i='+\
		     "linspace(time(i-1), time(i), 10)'"\
		     +';\n'
		    #varName=str(eq.getLHS().getOperands()[1])
		    code=code+'  '+varName+'i0=['+varName+'_last];\n'
		    code=code+'  '+varName+'_i=lsode("f_'+varName+'", '+\
			varName+'i0, time_i);\n'
		    code=code+'  '+varName+'(i)='+varName+'_i(10);\n'
		    # update the var_last value
		    code=code+'  '+varName+'_last='+varName+'(i);\n'
		# calcaulate the derivative in the case of derivative 
		# causality
		if varNode.hasDerivativeForm and \
		   (not varNode.integralCausality):
		    derEq='der_'+varName+'(i)=('+varName+'(i)-'+varName+\
			'(i-1))/time_step'
		    code=code+'  '+derEq+';\n'
		    # update the var_last value
		    code=code+'  '+'der_'+varName+'_last='+'der_'+\
			varName+'(i);\n'
	    index=index-1
	    eqNum=eqNum+1
	# update last values
	#code=code+'  '+'# update latest values'
	#for varName in varList:
	#    code=code+'  '+varName+'_last='+varName+'(i);\n'
	code=code+'endfor\n\n'
	# print output values
	#for i in varList:
	#    code=code+i+'\n'
	octaveFile=open(fileName, 'w')
	octaveFile.write(code)

    # generate a function for defferential equation
    def generateDOTFunnction(self, dNode):
	# the generated function name is 'f'+varName
	varName=dNode.getId()
	result='function '+varName+'_dot = f_'+varName+'('+ \
		varName+'i'+', '+'ti)\n'
	# code of global variables
	rhs=dNode.getEqNode().getEq().getRHS()
	result=result+self.generateDOTGlobalVars(rhs)
	# code of function body
	eqLHS=varName+'_dot(1)='
	stateVar=dNode.getVarNode().getDefinition()
	eqRHS=self.generateExpInDOTFun(stateVar, rhs)
	result=result+'  '+eqLHS+eqRHS+';\n'
	result=result+'endfunction\n'
	return result

    # generate global declarations in DOT functions
    def generateDOTGlobalVars(self, exp):
	result=''
	if isinstance(exp, IdentExp):
	    result=result+'  global '+str(exp)+'_last;\n'
	elif isinstance(exp, SumExp) or \
	     isinstance(exp, MulExp) or \
             isinstance(exp, UnaryMinusExp) or \
             isinstance(exp, PowerExp):
	    for i in exp.getOperands():
		result=result+self.generateDOTGlobalVars(i)
	# need loop over all cases
	return result

    # generate the RHS of the equation in DOT function
    def generateExpInDOTFun(self, stateVar, exp):
	#print 'in: ', str(exp)
	if isinstance(exp, IdentExp):
	    if exp.getValue()==stateVar:
		return str(exp)+'i(1)'
	    else:
	    	return str(exp)+'_last'
        elif isinstance(exp, BoolExp) or \
           isinstance(exp, StringExp) or \
	   isinstance(exp, NameExp):
	    return str(exp)
	elif isinstance(exp, IntegerExp) or isinstance(exp, RealExp):
	    if exp.getValue()<0:
		return '('+str(exp)+')'
	    else:
		return str(exp)
	elif isinstance(exp, OrExp):
	    result=''
	    for i in exp.getOperands():
		insertPar=self.hasLowerPrecedence(i, exp)
		if insertPar:
		    result=result+'('+self.generateExpInDOTFun(stateVar, i)\
			+')'+' or'
		else:
		    result=result+self.generateExpInDOTFun(stateVar, i)+' or'
	    return result[0:len(result)-3]
	elif isinstance(exp, AndExp):
	    result=''
	    for i in exp.getOperands():
		insertPar=self.hasLowerPrecedence(i, exp)
		if insertPar:
		    result=result+'('+self.generateExpInDOTFun(stateVar, i)\
			+')'+' and'
		else:
		    result=result+self.generateExpInDOTFun(stateVar, i)+' and'
	    return result[0:len(result)-4]
	elif isinstance(exp, NotExp):
            insertPar=self.hasLowerPrecedence(exp.getOperands()[0], exp)
	    if insertPar:
	        return 'not ('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+')'
	    else:
		return 'not '+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])
	elif isinstance(exp, LessExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
	    if insertPar1:
	        result='('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+') < '
	    else:
		result=self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+' < '
	    insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
	    if insertPar2:
		result=result+'('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])+')'
	    else:
		result=result+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])
	    return result
	elif isinstance(exp, LessEQExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+') <= '
            else:
                result=self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+' <= '
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])+')'
            else:
                result=result+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])
            return result
	elif isinstance(exp, GreaterExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+') > '
            else:
                result=self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+' > '
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])+')'
            else:
                result=result+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])
            return result
	elif isinstance(exp, GreaterEQExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+') >= '
            else:
                result=self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+' >= '
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])+')'
            else:
                result=result+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])
            return result
	elif isinstance(exp, EQEQExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+') == '
            else:
                result=self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+' == '
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])+')'
            else:
                result=result+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])
            return result
	elif isinstance(exp, LessGtExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+') <> '
            else:
                result=self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+' <> '
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])+')'
            else:
                result=result+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])
            return result
	elif isinstance(exp, SumExp):
	    result=''
	    for i in exp.getOperands():
		insertPar=self.hasLowerPrecedence(i, exp)
		if insertPar:
		    result=result+'('+self.generateExpInDOTFun(stateVar, i)\
			+')'+'+'
		else:
		    result=result+self.generateExpInDOTFun(stateVar, i)+'+'
	    return result[0:len(result)-1]
	elif isinstance(exp, MulExp):
            result=''
            for i in exp.getOperands():
		insertPar=self.hasLowerPrecedence(i, exp)
		if insertPar:
		    result=result+'('+self.generateExpInDOTFun(stateVar, i)\
			+')'+'*'
		else:
                    result=result+self.generateExpInDOTFun(stateVar, i)+'*'
            return result[0:len(result)-1]
	elif isinstance(exp, SubtractExp):
	    raise 'SubtractExp should not exist in canonical form!'
	elif isinstance(exp, DivExp):
	    raise 'DivExp should not exist in canonical form!'
	elif isinstance(exp, UnaryMinusExp):
	    # need check precedence to place the ( here
	    insertPar=self.hasLowerPrecedence(exp.getOperands()[0], exp)
	    if insertPar:
	        return '(-('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+'))'
	    else:
		return '(-'+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+')'
	elif isinstance(exp, PowerExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+')^'
            else:
                result=self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[0])+'^'
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])+')'
            else:
                result=result+self.generateExpInDOTFun(stateVar, \
			exp.getOperands()[1])
            return result
	    #return self.generateExpInDOTFun(exp.getOperands()[0])+'^'+ \
            #    self.generateExpInDOTFun(exp.getOperands()[1])
	elif isinstance(exp, RangeExp):
	    if len(exp.getOperands())==0:
		return ':'
	    elif len(exp.getOperands())==2:
		return self.generateExpInDOTFun(stateVar, \
		    exp.getOperands()[0])+':'+ \
                    self.generateExpInDOTFun(stateVar, exp.getOperands()[1])
	    elif len(exp.getOperands())==3:
		return self.generateExpInDOTFun(stateVar, exp.getOperands()[0])\
		    +':'+self.generateExpInDOTFun(stateVar, \
		    exp.getOperands()[1])+':'+\
		    self.generateExpInDOTFun(stateVar, exp.getOperands()[2])
	elif isinstance(exp, IfElseExp):
	    raise 'IfElseExp should not exist in canonical form!'
	elif isinstance(exp, FunCallExp):
	    operands=exp.getOperands()
	    return str(operands[0]) + '(' \
		+self.generateExpInDOTFun(stateVar, operands[1]) + ')'
	elif isinstance(exp, FunArgsExp):
	    result=''
	    for i in exp.getOperands():
		result=result+self.generateExpInDOTFun(stateVar, i)+', '
	    return result[0:len(result)-2]
	elif isinstance(exp, ForFunArgExp):
	    operands=exp.getOperands()
	    result=self.dumExp(operands[0])+' for '
	    for i in range(1, len(operands)):
            	result=result+self.generateExpInDOTFun(stateVar, \
			operands[i])+', '
            return result[0:len(result)-2]
	elif isinstance(exp, NamedArgExp):
	    operands=exp.getOperands()
	    return self.generateExpInDOTFun(stateVar, operands[0])+'='\
		+self.generateExpInDOTFun(stateVar, operands[1])
	elif isinstance(exp, ConnectRefExp):
	    raise 'ConnectRefExp should not exist in canonical form!'
	elif isinstance(exp, SingleCompRefExp):
	    operands=exp.getOperands()
	    if len(operands)==1:
            	result=self.generateExpInDOTFun(stateVar, operands[0])
            if len(operands)==2:
            	result=self.generateExpInDOTFun(stateVar, operands[0])+ \
			self.generateExpInDOTFun(stateVar, operands[1])
            return result
	elif isinstance(exp, CompRefExp):
	    operands=exp.getOperands()
	    result=''
            for i in operands:
            	result=result+self.generateExpInDOTFun(stateVar, i)+'.'
            return result[0:len(result)-1]
	elif isinstance(exp, ArrSubExp):
	    raise 'ArrSubExp should not exist in canonical form!'
	elif isinstance(exp, CommentExp):
	    pass
	elif isinstance(exp, AnnotationExp):
	    pass
	elif isinstance(exp, IdentInExpExp):
	    raise 'IdentInExpExp should not exist in canonical form!'
	elif isinstance(exp, ForIndicesExp):
	    raise 'ForIndicesExp should not exist in canonical form!'
	elif isinstance(exp, ExpListExp):
	    kind=exp.getKind()
	    operands=exp.getOperands()
	    if kind=='PAR':
            	begin='('
            	end=')'
            elif kind=='BRACK':
            	begin='['
            	end=']'
            elif kind=='BRACE':
            	begin='{'
            	end='}'
            result=begin
            if kind=='BRACK':
            	for i in operands:
                    result=result+self.generateExpInDOTFun(stateVar, i)+'; '
            	result=result[0:len(result)-2]
            else:
            	for i in operands:
                    result=result+self.generateExpInDOTFun(stateVar, i)+','
            	result=result[0:len(result)-1]
            return result+end
	elif isinstance(exp, ConversionExp):
	    return 'Real('+self.generateExpInDOTFun(stateVar, \
		exp.getOperands()[0])+')'
	else:
	    return str(exp)

    # generate function for to solve non-linear equations 
    # (algebraic loops)
    # list: [] of DNode
    def genNonLinearEqFunction(self, list, map, loopNum):
	# map vars to indexes of function parameter (vector)
	#i=1
	#map={}
	#for dNode in list:
	#    varNode=dNode.getVarNode()
	#    if varNode.hasDerivativeForm and varNode.integralCausality:
	#	varName='der_'+dNode.getId()
	#    else:
	#        varName=dNode.getId()
	#    map[varName]=i
	#    i=i+1
	# the generated function name is 'f'+loopNum
        result='function loop'+str(loopNum)+' = f_loop'+str(loopNum)+'(x)\n'
	# code of global variables
	for dNode in list:
            lhs=dNode.getEqNode().getEq().getLHS()
            result=result+self.genGlobalVarsInNonLinearFun(lhs, map)
        # code of function body
	i=1
	for dNode in list:
	    eq=dNode.getEqNode().getEq()
	    strLhs='loop'+str(loopNum)+'('+str(i)+')'
	    strRhs=self.genEqRHS2(eq.getLHS(), map)
	    result=result+'  '+strLhs+'='+strRhs+';\n'
	    i=i+1
        result=result+'endfunction\n'
        return result

    # generate global declarations in non-linear functions
    # map: {}
    def genGlobalVarsInNonLinearFun(self, exp, map):
	# need to avoid duplicate generation of the same var
        result=''
        if isinstance(exp, IdentExp):
	    if not str(exp) in map.keys():
            	result=result+'  global '+str(exp)+'_last;\n'
        elif isinstance(exp, SumExp) or \
             isinstance(exp, MulExp) or \
             isinstance(exp, UnaryMinusExp) or \
             isinstance(exp, PowerExp):
            for i in exp.getOperands():
                result=result+self.genGlobalVarsInNonLinearFun(i, map)
        # need loop over all cases
        return result

    # generate the RHS of an equation in function for solving 
    # non-linear equations (loops)
    # exp: AST::Exp
    # map: dictionary {}
    def genEqRHS2(self, exp, map):
	if isinstance(exp, IdentExp):
            if str(exp) in map.keys():
                return 'x('+str(map[str(exp)])+')'
            else:
                return str(exp)+'_last'
        elif isinstance(exp, BoolExp) or \
           isinstance(exp, StringExp) or \
           isinstance(exp, NameExp):
            return str(exp)
        elif isinstance(exp, IntegerExp) or isinstance(exp, RealExp):
            if exp.getValue()<0:
                return '('+str(exp)+')'
            else:
                return str(exp)
	elif isinstance(exp, SumExp):
            result=''
            for i in exp.getOperands():
                insertPar=self.hasLowerPrecedence(i, exp)
                if insertPar:
                    result=result+'('+self.genEqRHS2(i, map)+')'+'+'
                else:
                    result=result+self.genEqRHS2(i, map)+'+'
            return result[0:len(result)-1]
        elif isinstance(exp, MulExp):
            result=''
            for i in exp.getOperands():
                insertPar=self.hasLowerPrecedence(i, exp)
                if insertPar:
                    result=result+'('+self.genEqRHS2(i, map)+')'+'*'
                else:
                    result=result+self.genEqRHS2(i, map)+'*'
            return result[0:len(result)-1]
        elif isinstance(exp, SubtractExp):
            raise 'SubtractExp should not exist in canonical form!'
        elif isinstance(exp, DivExp):
            raise 'DivExp should not exist in canonical form!'
        elif isinstance(exp, UnaryMinusExp):
            # need check precedence to place the ( here
            insertPar=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar:
                return '(-('+self.genEqRHS2(exp.getOperands()[0], map)+'))'
            else:
                return '(-'+self.genEqRHS2(exp.getOperands()[0], map)+')'
	elif isinstance(exp, PowerExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.genEqRHS2(exp.getOperands()[0], map)+')^'
            else:
                result=self.genEqRHS2(exp.getOperands()[0], map)+'^'
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.genEqRHS2(exp.getOperands()[1], map)+')'
            else:
                result=result+self.genEqRHS2(exp.getOperands()[1], map)
            return result
	elif isinstance(exp, FunCallExp):
            operands=exp.getOperands()
            funName=str(operands[0])
            if funName=='der':
                return str(operands[0]) + '_' \
                        +self.genEqRHS2(operands[1], map)
            else:
                return str(operands[0]) + '(' \
                        +self.genEqRHS2(operands[1], map) + ')'
        elif isinstance(exp, FunArgsExp):
            result=''
            for i in exp.getOperands():
                result=result+self.genEqRHS2(i, map)+', '
            return result[0:len(result)-2]
        elif isinstance(exp, ForFunArgExp):
            operands=exp.getOperands()
            result=self.dumExp(operands[0])+' for '
            for i in range(1, len(operands)):
                result=result+self.genEqRHS2(operands[i], map)+', '
            return result[0:len(result)-2]
        elif isinstance(exp, NamedArgExp):
            operands=exp.getOperands()
            return self.genEqRHS2(operands[0], map)+'='+\
		self.genEqRHS2(operands[1], map)
	else:
            return str(exp)

    # geneate exps (in equations) within the for-loop
    def dumpExp(self, exp):
	#print 'in: ', str(exp)
	if isinstance(exp, IdentExp):
	    if str(exp)=='time':
		return str(exp)+'(i)'
	    else:
	    	return str(exp)+'_last'
	    	#if exp.getValue().isParameter() or exp.getValue().isConstant():
		#    return str(exp)+'_last'
		#else:
	    	#    return str(exp)+'(i)'
        elif isinstance(exp, BoolExp) or \
           isinstance(exp, StringExp) or \
	   isinstance(exp, NameExp):
	    return str(exp)
	elif isinstance(exp, IntegerExp) or isinstance(exp, RealExp):
	    if exp.getValue()<0:
		return '('+str(exp)+')'
	    else:
		return str(exp)
	elif isinstance(exp, OrExp):
	    result=''
	    for i in exp.getOperands():
		insertPar=self.hasLowerPrecedence(i, exp)
		if insertPar:
		    result=result+'('+self.dumpExp(i)+')'+' or'
		else:
		    result=result+self.dumpExp(i)+' or'
	    return result[0:len(result)-3]
	elif isinstance(exp, AndExp):
	    result=''
	    for i in exp.getOperands():
		insertPar=self.hasLowerPrecedence(i, exp)
		if insertPar:
		    result=result+'('+self.dumpExp(i)+')'+' and'
		else:
		    result=result+self.dumpExp(i)+' and'
	    return result[0:len(result)-4]
	elif isinstance(exp, NotExp):
            insertPar=self.hasLowerPrecedence(exp.getOperands()[0], exp)
	    if insertPar:
	        return 'not ('+self.dumpExp(exp.getOperands()[0])+')'
	    else:
		return 'not '+self.dumpExp(exp.getOperands()[0])
	elif isinstance(exp, LessExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
	    if insertPar1:
	        result='('+self.dumpExp(exp.getOperands()[0])+') < '
	    else:
		result=self.dumpExp(exp.getOperands()[0])+' < '
	    insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
	    if insertPar2:
		result=result+'('+self.dumpExp(exp.getOperands()[1])+')'
	    else:
		result=result+self.dumpExp(exp.getOperands()[1])
	    return result
	elif isinstance(exp, LessEQExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.dumpExp(exp.getOperands()[0])+') <= '
            else:
                result=self.dumpExp(exp.getOperands()[0])+' <= '
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.dumpExp(exp.getOperands()[1])+')'
            else:
                result=result+self.dumpExp(exp.getOperands()[1])
            return result
	elif isinstance(exp, GreaterExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.dumpExp(exp.getOperands()[0])+') > '
            else:
                result=self.dumpExp(exp.getOperands()[0])+' > '
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.dumpExp(exp.getOperands()[1])+')'
            else:
                result=result+self.dumpExp(exp.getOperands()[1])
            return result
	elif isinstance(exp, GreaterEQExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.dumpExp(exp.getOperands()[0])+') >= '
            else:
                result=self.dumpExp(exp.getOperands()[0])+' >= '
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.dumpExp(exp.getOperands()[1])+')'
            else:
                result=result+self.dumpExp(exp.getOperands()[1])
            return result
	elif isinstance(exp, EQEQExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.dumpExp(exp.getOperands()[0])+') == '
            else:
                result=self.dumpExp(exp.getOperands()[0])+' == '
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.dumpExp(exp.getOperands()[1])+')'
            else:
                result=result+self.dumpExp(exp.getOperands()[1])
            return result
	elif isinstance(exp, LessGtExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.dumpExp(exp.getOperands()[0])+') <> '
            else:
                result=self.dumpExp(exp.getOperands()[0])+' <> '
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.dumpExp(exp.getOperands()[1])+')'
            else:
                result=result+self.dumpExp(exp.getOperands()[1])
            return result
	elif isinstance(exp, SumExp):
	    result=''
	    for i in exp.getOperands():
		insertPar=self.hasLowerPrecedence(i, exp)
		if insertPar:
		    result=result+'('+self.dumpExp(i)+')'+'+'
		else:
		    result=result+self.dumpExp(i)+'+'
	    return result[0:len(result)-1]
	elif isinstance(exp, MulExp):
            result=''
            for i in exp.getOperands():
		insertPar=self.hasLowerPrecedence(i, exp)
		if insertPar:
		    result=result+'('+self.dumpExp(i)+')'+'*'
		else:
                    result=result+self.dumpExp(i)+'*'
            return result[0:len(result)-1]
	elif isinstance(exp, SubtractExp):
	    raise 'SubtractExp should not exist in canonical form!'
	elif isinstance(exp, DivExp):
	    raise 'DivExp should not exist in canonical form!'
	elif isinstance(exp, UnaryMinusExp):
	    # need check precedence to place the ( here
	    insertPar=self.hasLowerPrecedence(exp.getOperands()[0], exp)
	    if insertPar:
	        return '(-('+self.dumpExp(exp.getOperands()[0])+'))'
	    else:
		return '(-'+self.dumpExp(exp.getOperands()[0])+')'
	elif isinstance(exp, PowerExp):
            insertPar1=self.hasLowerPrecedence(exp.getOperands()[0], exp)
            if insertPar1:
                result='('+self.dumpExp(exp.getOperands()[0])+')^'
            else:
                result=self.dumpExp(exp.getOperands()[0])+'^'
            insertPar2=self.hasLowerPrecedence(exp.getOperands()[1], exp)
            if insertPar2:
                result=result+'('+self.dumpExp(exp.getOperands()[1])+')'
            else:
                result=result+self.dumpExp(exp.getOperands()[1])
            return result
	    #return self.dumpExp(exp.getOperands()[0])+'^'+ \
            #    self.dumpExp(exp.getOperands()[1])
	elif isinstance(exp, RangeExp):
	    if len(exp.getOperands())==0:
		return ':'
	    elif len(exp.getOperands())==2:
		return self.dumpExp(exp.getOperands()[0])+':'+ \
                	self.dumpExp(exp.getOperands()[1])
	    elif len(exp.getOperands())==3:
		return self.dumpExp(exp.getOperands()[0])+':'+ \
			self.dumpExp(exp.getOperands()[1])+':'+ \
			self.dumpExp(exp.getOperands()[2])
	elif isinstance(exp, IfElseExp):
	    raise 'IfElseExp should not exist in canonical form!'
	elif isinstance(exp, FunCallExp):
	    operands=exp.getOperands()
	    funName=str(operands[0])
	    if funName=='der':
	    	return str(operands[0]) + '_' \
			+self.dumpExp(operands[1])
	    else:
		return str(operands[0]) + '(' \
                        +self.dumpExp(operands[1]) + ')'
	elif isinstance(exp, FunArgsExp):
	    result=''
	    for i in exp.getOperands():
		result=result+self.dumpExp(i)+', '
	    return result[0:len(result)-2]
	elif isinstance(exp, ForFunArgExp):
	    operands=exp.getOperands()
	    result=self.dumExp(operands[0])+' for '
	    for i in range(1, len(operands)):
            	result=result+self.dumpExp(operands[i])+', '
            return result[0:len(result)-2]
	elif isinstance(exp, NamedArgExp):
	    operands=exp.getOperands()
	    return self.dumpExp(operands[0])+'='+self.dumpExp(operands[1])
	elif isinstance(exp, ConnectRefExp):
	    raise 'ConnectRefExp should not exist in canonical form!'
	elif isinstance(exp, SingleCompRefExp):
	    operands=exp.getOperands()
	    if len(operands)==1:
            	result=self.dumpExp(operands[0])
            if len(operands)==2:
            	result=self.dumpExp(operands[0])+self.dumpExp(operands[1])
            return result
	elif isinstance(exp, CompRefExp):
	    operands=exp.getOperands()
	    result=''
            for i in operands:
            	result=result+self.dumpExp(i)+'.'
            return result[0:len(result)-1]
	elif isinstance(exp, ArrSubExp):
	    raise 'ArrSubExp should not exist in canonical form!'
	elif isinstance(exp, CommentExp):
	    pass
	elif isinstance(exp, AnnotationExp):
	    pass
	elif isinstance(exp, IdentInExpExp):
	    raise 'IdentInExpExp should not exist in canonical form!'
	elif isinstance(exp, ForIndicesExp):
	    raise 'ForIndicesExp should not exist in canonical form!'
	elif isinstance(exp, ExpListExp):
	    kind=exp.getKind()
	    operands=exp.getOperands()
	    if kind=='PAR':
            	begin='('
            	end=')'
            elif kind=='BRACK':
            	begin='['
            	end=']'
            elif kind=='BRACE':
            	begin='{'
            	end='}'
            result=begin
            if kind=='BRACK':
            	for i in operands:
                    result=result+self.dumpExp(i)+'; '
            	result=result[0:len(result)-2]
            else:
            	for i in operands:
                    result=result+self.dumpExp(i)+','
            	result=result[0:len(result)-1]
            return result+end
	elif isinstance(exp, ConversionExp):
	    return 'Real('+self.dumpExp(exp.getOperands()[0])+')'
	else:
	    return str(exp)

    # compare the precedence of the type of 2 exps
    # if the precedence of the first arg is lower than the second one,
    # return True
    def hasLowerPrecedence(self, exp1, exp2):
	# assign precedence to exp1
	if isinstance(exp1, IfElseExp):
	    p1=1
	elif isinstance(exp1, OrExp):
	    p1=2
	elif isinstance(exp1, AndExp):
	    p1=3
	elif isinstance(exp1, NotExp):
	    p1=4
	elif isinstance(exp1, LessExp) or isinstance(exp1, LessEQExp) or \
	     isinstance(exp1, GreaterExp) or isinstance(exp1, GreaterEQExp) \
	     or isinstance(exp1, EQEQExp) or isinstance(exp1, LessGtExp):
	    p1=5
	#elif (isinstance(exp1, RealExp) or isinstance(exp1, IntegerExp)) \
        #        and exp1.getValue()<0:
        #    p1=6
	elif isinstance(exp1, SumExp):
	    p1=8
	elif isinstance(exp1, SubtractExp):
	    p1=9
	elif isinstance(exp1, MulExp):
	    p1=10
	elif isinstance(exp1, DivExp):
	    p1=11
	elif isinstance(exp1, PowerExp):
	    p1=12
	elif isinstance(exp1, UnaryMinusExp):
            p1=13
	else: 
	    p1=100 # dont care, a higher preceednce is assigned
	# assign precedence to exp2
	if isinstance(exp2, IfElseExp):
            p2=1
        elif isinstance(exp2, OrExp):
            p2=2
        elif isinstance(exp2, AndExp):
            p2=3
        elif isinstance(exp2, NotExp):
            p2=4
        elif isinstance(exp2, LessExp) or isinstance(exp2, LessEQExp) or \
             isinstance(exp2, GreaterExp) or isinstance(exp2, GreaterEQExp) \
             or isinstance(exp2, EQEQExp) or isinstance(exp2, LessGtExp):
            p2=5
	#elif (isinstance(exp2, RealExp) or isinstance(exp2, IntegerExp)) \
        #        and exp2.getValue()<0:
        #    p2=6
        elif isinstance(exp2, SumExp):
            p2=8
        elif isinstance(exp2, SubtractExp):
            p2=9
        elif isinstance(exp2, MulExp):
            p2=10
        elif isinstance(exp2, DivExp):
            p2=11
        elif isinstance(exp2, PowerExp):
            p2=12
	elif isinstance(exp2, UnaryMinusExp):
            p2=13
        else:
	    p2=100 #dont care, a higher preceednce is assigned
	# compare
	if p1 < p2:
	    return True
	return False

