# MuModelica Compiler Project
# Author: Steven Xu
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University

# This is the visitor module


import copy
from AST.ASTScope import ClassInstance
from AST.ASTComponent import ClassModification
from AST.ASTComponent import EqualsModification, AssignModification
from AST.ASTComponent import ElementRedeclaration, ElementModification
from FrontEnd.TypeChecker1 import TypeChecker
from AST.ASTExp import IdentExp, SingleCompRefExp, CompRefExp
from FrontEnd.miscellaneous.CopyMaker import CopyMaker
#from AST.ASTScope import DefinitionEntry
#from AST.ASTScope import DeclarationEntry, SymbolTable
#from Visitors import AbstractVisitor

# ---------------------------------------------------------------------
class Instantiater:
    "create instances and resolve modifications"
    def __init__(self, astP):
	# the AST
	self.ast=astP
	# the target class
        #self.modelName=modelNameP
	self.tc=TypeChecker()
	self.copier=CopyMaker()

    def go(self):
	# get the main class
        #classDef=self.ast.getScope().localLookup(self.modelName).getDefinition()
	mainModel=self.ast.classFiles[0].classDefinitions[0]
        self.instantiateClass(mainModel)

    # classDef: AST::ClassDefinition
    def instantiateClass(self, classDef):
	for i in classDef.getScope().getDeclarations():
	    if self.isPredefinedType(i) or self.isBuiltinType(i):
		pass
	    else:
	    	self.instantiateComp(i)

    # instantiate a composite component
    # decl: ASTScope::DeclarationEntry
    def instantiateComp(self, decl):
	if not decl.hasModif():
	    if self.isPredefinedType(decl) or self.isBuiltinType(decl):
		pass
	    else:
		for i in decl.getDefinition().getScope().getDeclarations():
		    self.instantiateComp(i)
	else: # has modif
	    modifList=decl.getModif()
	    #if not self.isBuiltinType(decl):
	    inst=self.createInstance(decl)
	    decl.setInstance(inst)
	    # merge modif to instance here
	    for modif in modifList:
	    	self.mergeModif(decl, modif)
	    if self.isPredefinedType(decl) or self.isBuiltinType(decl):
		pass
	    else:
	        for i in inst.getDeclarations():
		    self.instantiateComp(i)

    # decl: ASTScope::DeclarationEntry
    # classInst: ASTScope::ClassInstance
    # modif: ASTComponent::Modification
    def mergeModif(self, declEntry, modif):
	classInst=declEntry.getInstance()
        # ClassModification
        if isinstance(modif, ClassModification):
            for arg in modif.getArgumentList():
                if isinstance(arg, ElementRedeclaration):
                    raise 'Error: \"redeclare\" not yet supported!'
                elif isinstance(arg, ElementModification):
		    compRef=arg.getCompRef()
                    firstElemName=str(compRef.getOperands()[0].getOperands()[0])
		    matchElemFound=False
		    for decl in classInst.getDeclarations():
                        if not str(decl.getName())==firstElemName:
                        # "continue" here can not find out illegal
                        # use of names. Have to loop through all
                        # ElementModification to find out the right one
                        # But this should be okay if all names in
                        # modifications have been looked up.
                            continue
			matchElemFound=True
                        # append modif
                        if len(compRef.getOperands())==1:
                            if isinstance(arg.getModification(), \
                                        ClassModification):
                                decl.addModif(arg.getModification())
                            elif isinstance(arg.getModification(), \
                                        EqualsModification):
                                decl.addModif(arg.getModification())
                            else:
                                raise 'Error: \
					AssignModification not suppoted!'
                        # example: A.B.C=10
                        # B.C=10 is appended to the modif list of A
			else:
                            compRef2=copy.deepcopy(compRef)
                            del compRef2.operands[0]
                            # make a copy of the modif using shallow copy
                            newEM=copy.copy(arg)
                            newEM.setCompRef(compRef2)
                            newCM=ClassModification(-1,-1,[newEM],None)
                            # append the modif
                            decl.addModif(newCM)
		    if not matchElemFound:
			raise "illegal class modification!"
	    # handle the EqualsModification part in ClassModification
	    equalsModifPart=modif.getEqualsModification()
	    if not equalsModifPart==None:
	        self.mergeModif(declEntry, equalsModifPart)
        elif isinstance(modif, EqualsModification):
	    exp=modif.getExp()
	    # exp need to be looked up first and set refered to ST entry
	    if isinstance(exp, IdentExp):
		# eg., Model m1 = m2;
		rhsSTentry=exp.getValue()
		if self.tc.isTypeEquivalent(declEntry, rhsSTentry):
		    if self.isPredefinedType(rhsSTentry):
			return
		    rhsClassDef=rhsSTentry.getDefinition()
		    for lhsDecl in classInst.getPublicDeclarations():
			rhsDecl=rhsClassDef.getScope().localLookup(\
				str(lhsDecl.getName()))
			# need create CompRefExp here
			equalsModifExp=CompRefExp(-1, -1, \
					[SingleCompRefExp(-1, -1, \
					 [IdentExp(-1, -1, rhsSTentry)]), \
					SingleCompRefExp(-1, -1, \
					 [IdentExp(-1, -1, rhsDecl)])])
			equalsModif=EqualsModification(-1, -1, equalsModifExp)
			lhsDecl.addModif(equalsModif)
            elif isinstance(exp, CompRefExp):
		# need to be completed 
		rhsSTentry=exp.getLastOperand().getOperands()[0].getValue()
		if self.tc.isTypeEquivalent(declEntry, rhsSTentry):
		    if self.isPredefinedType(rhsSTentry):
			return
		    rhsClassDef=rhsSTentry.getDefinition()
		    for lhsDecl in classInst.getPublicDeclarations():
			rhsDecl=rhsClassDef.getScope().localLookup(\
				str(lhsDecl.getName()))
			equalsModifExp=CompRefExp(-1, -1, \
					exp.getOperands()+ \
					[SingleCompRefExp(-1, -1, \
                                         [IdentExp(-1, -1, rhsDecl)])])
			equalsModif=EqualsModification(-1, -1, equalsModifExp)
			lhsDecl.addModif(equalsModif)
	    else: # exp is a normal expression, e.g, Real a=b+c;
		if not self.isPredefinedType(declEntry):
		    print 'Modification type error: ', \
			str(declEntry.getType())+' '+str(declEntry)+'='+\
			str(exp)+';'
		    raise 'Modification type error...'
		# if 'declEntry' is a predefined type, the modif will be left 
		# in the modif list until it is merged during flattening.
		# Type check will be performed during type checking pass.
        elif isinstance(modif, AssignModification):
            pass

    # create an instance for a DeclarationEntry
    # decl: ASTScope::DeclarationEntry		
    def createInstance(self, decl):
	inst=ClassInstance()
	if self.isBuiltinType(decl):
	    return inst
	for i in decl.getDefinition().getScope().getDeclarations():
	    inst.addDeclarationEntry(str(i.getName()), i.deepCopy())
	for i in decl.getDefinition().getInitEquations():
	    #inst.addInitEquation(copy.deepcopy(i))
	    inst.addEquation(self.copier.deepcopyEqStm(i))
	for i in decl.getDefinition().getEquations():
	    #inst.addEquation(copy.deepcopy(i))
	    inst.addEquation(self.copier.deepcopyEqStm(i))
	return inst

    # elem: DeclarationEntry
    def isPredefinedTypeOld(self, elem):
        found=self.ast.predefinedScope.localLookup(str(elem.getType()))
        return (not found==None)

    # elem: DeclarationEntry
    def isPredefinedType(self, elem):
	predefinedScope=self.ast.predefined.classDefinitions[0].scope
	found=predefinedScope.localLookup(str(elem.getType()))
	if not found==None:
	    return True
	else:
	    RealDefEntry=predefinedScope.localLookup('Real')
	    RealDef=RealDefEntry.getDefinition()
	    IntegerDefEntry=predefinedScope.localLookup('Integer')
	    IntegerDef=IntegerDefEntry.getDefinition()
	    StringDefEntry=predefinedScope.localLookup('String')
	    StringDef=StringDefEntry.getDefinition()
	    BooleanDefEntry=predefinedScope.localLookup('Boolean')
	    BooleanDef=BooleanDefEntry.getDefinition()
	    elemDef=elem.getDefinition()
	    if self.tc.isClassTypeEquivalent(elemDef, RealDef) or \
		self.tc.isClassTypeEquivalent(elemDef, IntegerDef) or \
		self.tc.isClassTypeEquivalent(elemDef, StringDef) or \
		self.tc.isClassTypeEquivalent(elemDef, BooleanDef):
		return True
	return False

    def isBuiltinType(self, elem):
	found=self.ast.builtinScope.localLookup(str(elem.getType()))
	return (not found==None)


