# MuModelica Compiler Project
# Author: Steven Xu
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University

# This is the visitor module

from AST.ASTDefinition import ClassDefinition
from AST.ASTScope import DefinitionEntry
from AST.ASTScope import DeclarationEntry, SymbolTable
from FrontEnd.Exception.NameException import NameNotDeclaredException, \
                                             UnencapsulatedElementException
from FrontEnd.Exception.ScopeException import EncapsulatedInheritanceException
from FrontEnd.Visitor import AbstractVisitor

# ---------------------------------------------------------------------
class Lookuper2(AbstractVisitor):
    "name lookup"
    def __init__(self, ast, modelNameP):
	# the debugging message
	self.msg=''
	# this global attribute also controls the differences between
        # the first and the second name lookup, e.g, names are
        # only linked to symbol table in the second lookup, and
        # names in "connect" will not be looked up in the second time
	self.ast=ast
	self.mainModel=None
	#if not astP==None:
        #    mainEntry=astP.getScope().localLookup(modelNameP)
        #    self.mainModel=mainEntry.getDefinition()
	# the current scope
        self.currentScope=None
	# the scope of base class of the currently visiting short class
	self.baseClassScopeOfCurrentShortClass=None
	# the scope node which stores all predefined primitive types
        self.predefinedScope=None
	# the scope node which stores all builtin primitive types
        self.builtinScope=None
	# the current declarationEntry that is being visited
	self.currentEntry=None
	# flag: indicates whether a ShortClassDefinition is being visited
	self.isVisitingShortClass=False
	# flag: indicates whether it is visiting a funtion name
	self.isVisitingFunCall=False

    def setCurrentEntry(self, e):
	self.currentEntry=e

    def setPredefinedScope(self, scope):
        self.predefinedScope=scope

    def setBuiltinScope(self, scope):
        self.builtinScope=scope

    # ---------------------ClassDefinition------------------
    def visitAST(self, ast):
	self.predefinedScope=self.ast.getPredefinedScope()
	self.builtinScope=self.ast.getBuiltinScope()
	for i in ast.classFiles:
	    i.accept(self)

    def visitClassFile(self, classFile):
	if self.mainModel==None:
	    self.currentScope=classFile.getScope()
            for i in classFile.getClassDefinitions():
	    	i.accept(self)
	else:
	    self.mainModel.accept(self)
	    

    def visitRegularClassDefinition(self, rClassDefinition):
	self.msg=self.msg+'\n In class: '+str(rClassDefinition.getName())
	if self.mainModel==None:
	    self.currentScope=rClassDefinition.getScope()
	    # lookup names in modifications
	    rClassDefinition.getScope().accept(self)
	    for i in rClassDefinition.getElementList():
		if isinstance(i, ClassDefinition):
	    	    i.accept(self)
	    for i in rClassDefinition.getInitEquations():
	    	i.accept(self)
	    for i in rClassDefinition.getEquations():
	    	i.accept(self)
	    self.currentScope=self.currentScope.getOuter()
	else:
	    # all names are looked up in the flat scope
	    self.currentScope=rClassDefinition.getFlatScope()
	    for i in rClassDefinition.getFlattenedInitEqs():
		i.accept(self)
	    for i in rClassDefinition.getFlattenedEqs():
		i.accept(self)
	self.msg=self.msg+'\n end of class: '+str(rClassDefinition.getName())
	    

    def visitShortClassDefinition(self, sClassDefinition):
	self.isVisitingShortClass=True
	self.msg=self.msg+'\n In class: '+str(sClassDefinition.getName())
	if self.mainModel==None:
	    # In Short Class definition, identifiers in modif 
	    # are looked up in outer scope
	    self.currentScope=sClassDefinition.getScope()
	    baseClass=self.baseClassLookup(\
		sClassDefinition.getScope(), sClassDefinition.getBaseName())
	    self.baseClassScopeOfCurrentShortClass=\
		baseClass.getDefinition().getScope()
            # lookup names in modifications
	    sClassDefinition.getModification().accept(self)
	    self.currentScope=self.currentScope.getOuter()
	else:
            # all names are looked up in the flat scope
            self.currentScope=sClassDefinition.getFlatScope()
            for i in sClassDefinition.getFlattenedInitEqs():
                i.accept(self)
            for i in sClassDefinition.getFlattenedEqs():
                i.accept(self)
	self.isVisitingShortClass=False
	self.msg=self.msg+'\n end of class: '+str(sClassDefinition.getName())

    def visitEnumClassDefinition(self, eClassDefinition):
	# EnumClassDefinition has been transformed to 
	# RegularClassDefinition
        pass

    def visitImportClause(self, importClause):
        pass

    def visitExtendsClause(self, extendsClause):
	#if not extendsClause.getModification()==None:
	    #extendsClause.modification.accept(self)
        pass

    def visitReplaceableClause(self, replaceableClause):
        raise "\"Replaceable\" found: not yet supported! Abort ... "

    def visitComponentClause(self, componentClause):
	# code here for Modification
	#if not componentClause.arrSub==None:
	#    componentClause.arrSub.accept(self)
	for i in componentClause.getComponentList():
	    i.accept(self)


    # --------------------Ssymbol Table--------------------------

    def visitSymbolTable(self, st):
        for i in st.getDeclarations():
	#for i in st.getAllElems():
	    #if not (str(i.getName())=='self' or str(i.getName())=='time'):
            i.accept(self)

    # only identfiers in modification are looked up in this pass
    def visitDeclarationEntry(self, e):
        self.setCurrentEntry(e)
	for i in e.getModif():
            i.accept(self) 
	self.setCurrentEntry(None)

    def visitDefinitionEntry(self, e):
	#e.getDefinition().accept(self)
	pass

    # -----------------------Component----------------------------
    def visitDeclaration(self, declaration):
	pass

    def visitClassModification(self, cm):
	tempScope=self.currentScope
	for i in cm.getArgumentList():
	    i.accept(self)
	equalsModifPart=cm.getEqualsModification()
	if not equalsModifPart==None:
	    equalsModifPart.accept(self)

    def visitEqualsModification(self, em):
	tempScope=self.currentScope
        # identifiers in modification of short class definition is
        # looked up in the outer scope
        if self.isVisitingShortClass:
            self.currentScope=self.currentScope.getOuter()
	em.getExp().accept(self)
	# reset currentScope
        if self.isVisitingShortClass:
            self.currentScope=tempScope

    def visitAssignModification(self, am):
	raise 'AssignModification is not supported ...'

    def visitElementModification(self, elemModif):
	# look up CompRef in ElementModification
	# only identifiers are looked up so far,
        # array subscripts are ignored
	if self.isVisitingShortClass:
	    scope=self.baseClassScopeOfCurrentShortClass
	else:
	    scope=self.currentEntry.getDefinition().getScope()
	compRef=elemModif.getCompRef()
        operands=compRef.getOperands()
	#found=elemScope.lookup(str(operands[0].getOperands()[0]))
	found=self.lookup(scope, str(operands[0].getOperands()[0]))
        # verify the found entry
	if found==None or (isinstance(found, DefinitionEntry) and \
			   not str(found.getName())=='self'):
	    raise NameNotDeclaredException(str(compRef))
	# lookup of the remaining part of the name
        index=1
        numOperands=len(operands)
        while (index<numOperands):
            # the rest of the name is looked up among declared classes
            found=self.lookupInScope(found.getDefinition().getScope(), \
                                operands[index].getOperands()[0])
            # verify the found entry
            if not isinstance(found, DeclarationEntry):
                raise NameNotDeclaredException(str(compRef))
            index=index+1
        self.msg=self.msg+'\n Element modification: '+str(compRef)+' is found;'
	# names in the RHS of ElementModification are looked up
	elemModif.getModification().accept(self)

    def visitElementRedeclaration(self, elemRedcl):
	pass

    # ------------------------------Equation----------------------
    def visitRegularEqStm(self, eqStm):
	self.msg=self.msg+'\n In equation: '+str(eqStm)
	eqStm.LHS.accept(self)
	eqStm.RHS.accept(self)

    def visitConnectEqStm(self, eqStm):
	self.msg=self.msg+'\n In equation: '+str(eqStm)
        eqStm.first.accept(self)
	eqStm.second.accept(self)

    def visitFunCallEqStm(self, eqStm):
        eqStm.funCall.accept(self)

    def visitConditionalEqStm(self, eqStm):
        eqStm.ifTest.accept(self)
	for i in eqStm.ifBody:
	    i.accept(self)
	for i in eqStm.elseifList:
	    i.accept(self)
	for i in eqStm.elseBody:
	    i.accept(self)

    def visitForEqStm(self, eqStm):
	eqStm.forIndices.accept(self)
	# enter new scope
	self.currentScope=eqStm.getScope()
	for i in eqStm.body:
	    i.accept(self)
	# reset current scope
	self.currentScope=self.currentScope.getOuter()

    def visitWhenEqStm(self, eqStm):
	eqStm.whenTest.accept(self)
	for i in eqStm.body:
	    i.accept(self)
	for i in elsewhenList:
	    i.accept(self)

    # ------------------------------Exp-----------------------------
    # lookup of simple names (identifiers), eg, a, b
    def visitIdentExp(self, exp):
	if str(exp)=='time':
            return
	found=self.lookup(self.currentScope, str(exp))
	if found == None:
            raise NameNotDeclaredException(str(exp))
	elif self.isVisitingFunCall:
	    if not isinstance(found, DefinitionEntry):
		raise NameNotDeclaredException(str(exp))
	else:
	    if not isinstance(found, DeclarationEntry):
		raise NameNotDeclaredException(str(exp))
	self.msg=self.msg+'\n '+str(exp)+' is found;'

    def visitRealExp(self, exp):
	pass

    def visitIntegerExp(self, exp):
	pass

    def visitBoolExp(self, exp):
	pass

    def visitStringExp(self, exp):
	pass

    # this method is not visited
    def visitNameExp(self, exp):
	pass

    def visitOrExp(self, exp):
	for i in exp.getOperands():
	    i.accept(self)

    def visitAndExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitNotExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitLessExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def vistLessEQExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitGreaterExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitGreaterEQExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitEQEQExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitLessGtExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitSumExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitMulExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitSubtractExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitDivExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitUnaryMinusExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitPowerExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitRangeExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitIfElseExp(self, exp):
        exp.ifTest.accept(self)
        exp.ifBody.accept(self)
        for i in exp.elseifList:
            i.accept(self)
        if not exp.elseBody==None:
            exp.elseBody.accept(self)

    def visitFunCallExp(self, exp):
	# set the flag
	self.isVisitingFunCall=True
	# a funtion name is first looked up in predefined scope
	found=self.predefinedScope.localLookup(str(exp.getOperands()[0]))
        if not found ==None:
	    self.msg=self.msg+'\n '+str(exp.getOperands()[0])+' is found;'
	else: # normal lookup
	    exp.getOperands()[0].accept(self)
	self.isVisitingFunCall=False
	# lookup of parameters
	exp.getOperands()[1].accept(self)

    def visitFunArgsExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitForFunArgExp(self, exp):
	operands=exp.getOperands()
	# enter new scope
	self.currentScope=exp.getScope()
	# only the first elem is looked up in the innner scope
	operands[0].accept(self)
	# exit scope
	self.currentScope=self.currentScope.getOuter()
	for i in range(1, len(operands)):
            operands[i].accept(self)

    def visitNamedArgExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitConnectRefExp(self, exp):
        #only identifiers are looked up so far,
        #array subscripts are ignored
        operands=exp.getOperands()
        numOperands=len(operands)
        # lookup of the first identifier
        found=self.lookup(self.currentScope, \
                          operands[0].getOperands()[0])
        if found == None or not isinstance(found, DeclarationEntry):
           raise NameNotDeclaredException(str(exp))
        operands[0].operands[0].setValue(found)
        # lookup of the remaining part of the name
        index=1
        numOperands=len(operands)
        while (index<numOperands):
            # the rest of the name is looked up among declared classes
            found=self.lookupInScope(found.getDefinition().getScope(), \
                                operands[index].getOperands()[0])
            if found == None or not isinstance(found, DeclarationEntry):
                raise NameNotDeclaredException(str(exp))
            operands[index].operands[0].setValue(found)
            index=index+1
        self.msg=self.msg+'\n '+str(exp)+' is found;'

    def visitSingleCompRefExp(self, exp):
	pass

    def visitCompRefExp(self, exp):
	#only identifiers are looked up so far, 
	#array subscripts are ignored
        operands=exp.getOperands()
        numOperands=len(operands)
	# lookup of the first identifier
	found=self.lookup(self.currentScope, \
			  operands[0].getOperands()[0])
	# verify the found entry
        if found == None:
            raise NameNotDeclaredException(str(exp))
        elif self.isVisitingFunCall:
            if not isinstance(found, DefinitionEntry):
                raise NameNotDeclaredException(str(exp))
        else:
            if not isinstance(found, DeclarationEntry):
                raise NameNotDeclaredException(str(exp))
	operands[0].operands[0].setValue(found)
	# lookup of the remaining part of the name
        index=1
        numOperands=len(operands)
        while (index<numOperands):
            # the rest of the name is looked up among declared classes
            found=self.lookupInScope(found.getDefinition().getScope(), \
                                operands[index].getOperands()[0])
	    # verify the found entry
	    if found == None:
            	raise NameNotDeclaredException(str(exp))
            elif self.isVisitingFunCall:
            	if not isinstance(found, DefinitionEntry):
                    raise NameNotDeclaredException(str(exp))
            else:
            	if not isinstance(found, DeclarationEntry):
                    raise NameNotDeclaredException(str(exp))
	    operands[index].operands[0].setValue(found)
            index=index+1
	self.msg=self.msg+'\n '+str(exp)+' is found;'

    def visitArrSubExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitCommentExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    def visitAnnotationExp(self, exp):
	pass

    def visitIdentInExpExp(self, exp):
	exp.getOperands()[1].accept(self)

    def visitForIndicesExp(self, exp):
	for i in exp.getOperands():
	    # i: IdentInExpExp
	    i.accept(self)

    def visitExpListExp(self, exp):
	for i in exp.getOperands():
            i.accept(self)

    # ----------------- lookup algorithms -----------------------

    # the lookup process of a simple name, starting from scope sp
    # sp: ASTScope::SymbolTable
    # exp: ASTExp::IdentExp
    def lookup(self, sp, exp):
        # lookup sequentially in the ordered set of parents
        while (True):
            found=self.lookupInScope(sp, exp)
            if not found==None:
                return found
	    if not sp.getOuter()==None:
		# lookup stops if an encapsulated parent is found
            	currentClass=sp.localLookup('self').getDefinition()
		if currentClass.isEncapsulated():
		    break
            	sp=sp.getOuter()	
	    # lookup stops if reaching the top parent 
	    else:
		break
        return None

    # the lookup precess in a scope sp
    # sp: ASTScope::SymbolTable
    # exp: ASTExp::IdentExp
    def lookupInScope(self, sp, exp):
        # lookup from locally declared elements
        found=sp.localLookup(str(exp))
        if not found ==None:
            return found
        # if not found, then perform lookup from inherited elements
        if sp.getOuter()==None: # has reached the top scope
            return None
        for i in sp.localLookup('self').getDefinition().getExtendsClauses():
            baseClass=self.baseClassLookup(sp, i.getName()).getDefinition()
            found=self.lookupInScope(baseClass.getScope(), exp)
            if not found==None:
                return found
        return None

    # lookup algorithm for the base class name in an extends clause
    # The base class name can be a simple name, e.g. A,
    # or composite name, e.g. A.B.C
    # sp: ASTScope::SymbolTable
    # exp: ASTExp::NameExp
    def baseClassLookup(self, sp, exp):
        operands=exp.getOperands()
        numOperands=len(operands)
        # look up the first identifier in a NameExp, eg, A in A.B.C
        firstIdent=operands[0]
        # names are first searched from pre-defined types
        found=self.predefinedScope.localLookup(str(firstIdent))
        if (not found==None) and numOperands==1 :
            return found
        #if self.isEncapsulated:
	if sp.localLookup('self').getDefinition().isEncapsulated():
            raise EncapsulatedInheritanceException(str(exp))
        # normal look up: lookup sequentially in each member of
        # the ordered set of parents
        else:
            found=sp.lookup(str(firstIdent))
            # the found entry must be a DefinitionEntry
            if isinstance(found, DefinitionEntry):
                localScope=found.getDefinition().getScope()
            else:
                raise NameNotDeclaredException(str(exp))
        # continue look up if exp is a composite name
        # eg, look up B, C in name A.B.C
        i=1
        while i<numOperands:
            isPackage=found.getDefinition().isPackage()
            found=localScope.localLookup(str(operands[i]))
            if isinstance(found, DefinitionEntry):
                if not isPackage:
                    # lookup is restricted to encapsulated elements only
                    if not found.getDefinition().isEncapsulated():
                        raise UnencapsulatedElementException(str(operands[i]))
                localScope=found.getDefinition().getScope()
            else:
                raise NameNotDeclaredException(str(exp))
            i=i+1
        return found



