# MuModelica Compiler Project
# Author: Steven Xu
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University



from AST.ASTExp import IdentExp
from AST.ASTScope import ComponentPrefix, DefinitionEntry
from AST.ASTScope import DeclarationEntry, SymbolTable
from FrontEnd.Visitor import AbstractVisitor
from FrontEnd.printers.EqExpPrinter import EqExpPrinter

# ---------------------------------------------------------------------

class FlatModelPrinter(AbstractVisitor):
    "print out AST to original the Modelica model"
    def __init__(self):
	self.indentLevel=0
	self.indent='  '
	self.output=''
	self.eqPrinter=EqExpPrinter()
	
    def getOutput(self):
	return self.output

    # ---------------------------ClassDefinition-----------------------
    def visitRegularClassDefinition(self, r):
	# dump class prefix, class name ...
	indentation=self.indentLevel*self.indent
	self.output=self.output+indentation+str(r.prefix)
	self.output=self.output+str(r.kind)+' '+str(r.className)
	if not r.stringComment==None:
            self.output=self.output+' '+str(r.stringComment)
	self.output=self.output+'\n'
	# dump class elements
	self.indentLevel=self.indentLevel+1
	indentation=self.indentLevel*self.indent
	for i in r.getFlattenedComponents():
	    i.accept(self)
	    self.output=self.output+'\n'
	self.indentLevel=self.indentLevel-1
	# dump initial equations
	indentation=self.indentLevel*self.indent
	if len(r.flattenedInitEqs)>0:
	    self.output=self.output+indentation+'initial equation \n'
	    self.indentLevel=self.indentLevel+1
	    for i in r.flattenedInitEqs:
		i.accept(self)
		self.output=self.output+';\n'
	    self.indentLevel=self.indentLevel-1
	# dump equations
	indentation=self.indentLevel*self.indent
        if len(r.flattenedEqs)>0:
            self.output=self.output+indentation+'equation \n'
            self.indentLevel=self.indentLevel+1
            for i in r.flattenedEqs:
                i.accept(self)
                self.output=self.output+';\n'
            self.indentLevel=self.indentLevel-1
	# dump initial alg
	# dumpl alg
	# dump 'end' & class name
	self.output=self.output+indentation+'end ' +str(r.className)+';'

    def visitShortClassDefinition(self, s):
	indentation=self.indentLevel*self.indent
	self.output=self.output+indentation+str(s)

    def visitEnumClassDefinition(self, eClassDefinition):	
	# EnumClassDefinition has been transformed to 
	# RegularClassDefinition
	pass

    def visitDeclarationEntry(self, e):
	indentation=self.indentLevel*self.indent
	self.output=self.output+indentation
	if e.typePrefix.getVisibility()=='PROTECTED':
	    self.output=self.output+'protected '
	self.output=self.output+str(e.typePrefix)+str(e.type)+'  '\
		+str(e.name)+';'
	for i in e.env.keys():
            if not e.env[i]==None:
		self.output=self.output+'\n    //'+i+'='+str(e.env[i])
	
    # ------------------------------EqStm-----------------------------
    def visitRegularEqStm(self, eq):
	indentation=self.indentLevel*self.indent
	self.output=self.output+indentation+self.eqPrinter.dumpEq(eq)
	#eqStm.LHS.accept(self)
	#eqStm.RHS.accept(self)

    def visitConnectEqStm(self, eq):
	indentation=self.indentLevel*self.indent
	self.output=self.output+indentation+str(eq)

    def visitFunCallEqStm(self, eq):
	indentation=self.indentLevel*self.indent
	self.output=self.output+indentation+str(eq)

    def visitConditionalEqStm(self, eq):
	# to be finished
	pass

    def visitForEqStm(self, eqStm):
	# to be finished
	pass

    def visitWhenEqStm(self, eqStm):
	# to be finished
	pass

