
//#include <antlr/Token.hpp>
#include "antlr/ANTLRException.hpp"

#include "modelica_lexer.hpp"
#include "modelica_parser.hpp"
#include "xml_dumper.hpp"

#include <cstdlib>
#include <fstream>
#include <iostream>

using namespace antlr;

int main(int argc, char* argv[])
{
    std::ifstream file;

    if (argc != 3)
    {
	std::cerr << "Incorrect number of arguments\n";
	return 1;
    }
    
    file.open(argv[1]);

    if (!file)
    {
	std::cerr << "Could not open file: " << argv[1] << "\n";
	return 2;
    }
    
    try 
    {
	modelica_lexer lexer(file);
	lexer.setFilename(argv[1]);

	modelica_parser parser(lexer);

        ASTFactory ast_factory;
        parser.initializeASTFactory(ast_factory);
        parser.setASTFactory(&ast_factory);

	parser.stored_definition();

	xml_dumper dumper(argv[2]);
 	dumper.go(parser.getAST(), argv[1]);
    }
    catch (ANTLR_USE_NAMESPACE(antlr)ANTLRException &e)
      {
	std::cerr << "ANTLRException: " << e.getMessage() << std::endl;
	file.close();
	return EXIT_FAILURE;
      }
    catch(std::exception& e) 
    {
	std::cerr << "Exception: " << e.what() << std::endl;
	file.close();
	return EXIT_FAILURE;
    }
  
    file.close();
    return EXIT_SUCCESS;
}
