
# DEVS Modelica Compiler Project
# Author: Hongyan Song
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University



from AST.ASTExp import IdentExp, CompRefExp, FunCallExp, FunArgsExp
from AST.ASTScope import ComponentPrefix, DefinitionEntry
from AST.ASTScope import DeclarationEntry, SymbolTable
from AST.ASTDefinition import RegularClassDefinition, ComponentClause
from FrontEnd.Visitor import AbstractVisitor
from devs.DevsEqExpPrinter import EqExpPrinter
from devs.mappingparser import MappingParser
import string
import dircache
from sets                import Set

# ---------------------------------------------------------------------

class DevsPrinter(AbstractVisitor):
    "print out AST to original the Modelica model"
    def __init__(self, modelicaPath):
	self.indentLevel  = 0
	self.indent       = '  '
	self.output       = ''
	self.eqPrinter    = EqExpPrinter()
        self.modelicaPath = modelicaPath
	self.inElseif     = False
	self.inClass      = False
	self.inFunction   = False
        self.inDevsState  = False
        self.inElLeft     = False
	self.inExperiment = False
	self.experimentName = ''
	self.inCompRef    = False	
	self.isSubModelClause=False
	self.currentScope = None
	self.currentClass = ''
	
    def getOutput(self):
	return self.output

    # write to a file
    def writeToFile(self, file):
        modelicaFile=open(file, 'w')
        modelicaFile.write(self.output)

    # ---------------------------ClassDefinition-----------------------
    def visitAST(self, ast):
	self.currentScope = ast.getScope()
	ast.classFiles[0].accept(self)

    def visitClassFile(self, classFile):
	self.currentScope = classFile.getScope()
	indentation = self.indentLevel * self.indent
	if not classFile.withinClause==None:
	    classFile.withinClause.accept(self)
        #self.readFuncMappings()
	for i in classFile.classDefinitions:
	  self.output = self.output + self.makeImports(indentation, i.getImports())
	  if(i.kind=='package'):
	    for el in i.elementList:
	      if(isinstance(el, RegularClassDefinition)):
	        self.output = self.output + self.makeImports(indentation, el.getImports())
	  
	for i in classFile.classDefinitions:
	    i.accept(self)
	    self.output=self.output+'\n\n'	
	self.currentScope = self.currentScope.getOuter()
	if(self.experimentName!=''):
	  indentation = self.indentLevel * self.indent
	  tmpoutput = ''
	  tmpoutput = tmpoutput + '\n' + indentation + 'import sys'
          tmpoutput = tmpoutput + '\n' + indentation + 'import os.path'
	  #tmpoutput = tmpoutput + '\n' + indentation + 'sys.path.append(os.path.expanduser(\'~/UA/1MINF/OS1/pydevs-mod/pydevs/\'))'
	  tmpoutput = tmpoutput + '\n' + indentation + 'sys.path.append(os.path.expanduser(\'pydevs/\'))'
          tmpoutput = tmpoutput + self.output
	  self.output = tmpoutput
          self.output = self.output + '\n' + indentation + 'def terminate_at(t):'
          self.output = self.output + '\n' + indentation + ' def f(model, clock):'
          self.output = self.output + '\n' + indentation + '  if clock >= t:'
          self.output = self.output + '\n' + indentation + '    return True'
          self.output = self.output + '\n' + indentation + '  else:'
          self.output = self.output + '\n' + indentation + '    return False'
          self.output = self.output + '\n' + indentation + ' return f'
	  self.output = self.output + '\n' + indentation + 'if __name__ == \'__main__\':'
	  self.output = self.output + '\n' + indentation + '  experiment = ' + self.experimentName + '()'
	  self.output = self.output + '\n' + indentation + '  experiment.simulate()'
	  #self.output = self.output + '\n' + indentation \
	  #            + '  experiment.sim.simulate(termination_condition=terminate_whenEndTimeReached, verbose=True, trace=True)'
	  
    def visitWithinClause(self, wClause):
	self.output=self.output+self.indentLevel*self.indent+str(wClause)+\
		';\n'
    #-------------------------------------------------------	
    #helper functions for devs code generation   
    #----------------------------------------------------------------------
    def makeImports(self, indentation, impList):
      impStr = ''
      mParser = self.getMappingParser()
      packMap = mParser.getPackageMap()
      #print packMap.keys()
      funcMap = mParser.getFunctionMap()
      for imp in impList:
	if(imp.qualified):
	  dp = str(imp.name).rfind('.')	
	  pack = str(imp.name)[0:dp]
	  className = str(imp.name)[dp+1:]
          if(pack=='devs' or pack=='externalfunctions'):
            impStr = impStr + '\n' + indentation + 'from ' + funcMap[className].mapPackage + ' import ' + funcMap[className].mapName
          else:
	    impStr = impStr + '\n' + indentation + 'from ' + pack + ' import ' + className
	else:
  	  pack = str(imp.name)
          if(pack=='devs'):
	    impStr = impStr + '\n' + indentation + 'from ' + packMap[pack] + ' import *'
	    impStr = impStr + '\n' + indentation + 'from infinity import *'
          elif(pack=='externalfunctions'):
            s = Set()
            for v in funcMap.values():
            #  s.add(v.mapPackage)
            #for p in s:
	      impStr = impStr + '\n' + indentation + 'from ' + v.mapPackage + ' import ' + v.mapName
          else:
	    impStr = impStr + '\n' + indentation + 'from ' + pack + ' import *'

      return impStr

    def getMappingParser(self):
      found = False
      for i in self.modelicaPath:
	# get all filenames in directory i
        a = dircache.listdir(i+'/')
        if 'funcmapping.xml' in a:
          found = True
          mappingParser = MappingParser(i+'/funcmapping.xml')
          #fMap = tParser.getFunctionMap()
          #print 'here', fMap.keys()
          #for f in fMap.keys():
          #  print f, fMap[f]
          #pMap = tParser.getPackageMap()
          #for p in pMap.keys():
          #  print p, pMap(p)
          break

	# if str(exp) is a directory name
      if(not found):          
        raise 'could not find the function mapping file: funcmapping.xml'

      return mappingParser
	 
    def makeParameters(self, paraList): #In Modelica, parameters are for classes
	paraStr = 'self';
        for dList in paraList:
	   for p in dList:
	     #print p.__class__
	     paraStr = paraStr + ', ' + str(p.getName())
	return paraStr
	   
    def makeParameterVariables(self, indentation, paraList): #In Modelica, parameters are for classes
	varStr = '';
        for dList in paraList:
	   for p in dList:
	     #print p.__class__
	     varStr = varStr + '\n' + indentation + 'self.' + str(p.getName()) + ' = ' + str(p.getName())
	return varStr

    def makeOutputInits(self, indentation, outputList): #In Modelica, parameters are for classes
	varStr = '';
        for dList in outputList:
	   for p in dList:
	     #print p.__class__
             if p.modification != None:
	        varStr = varStr + '\n' + indentation + str(p.getName()) + str(p.modification)
	return varStr


    def makeClassVariables(self, indentation, paraList): #In Modelica, parameters are for classes
	varStr = '';
        return varStr
        for dList in paraList:
	   for p in dList:
	     varStr = varStr + '\n' + indentation + 'self.' + str(p.getName()) + ' = ' + str(p.getName())
             continue
             if "=" not in str(p.modification):
	        varStr = varStr + '\n' + indentation + 'self.' + str(p.getName()) + ' = None'
             else:
	        varStr = varStr + '\n' + indentation + 'self.' + str(p.getName()) + ' ' + str(p.modification)
	return varStr

    def makeFunctionInputs(self, inList):  #In Modelica, inputs and outputs mean parameters for functions
	inStr = 'self';
        for inpt in inList:
	    for comp in inpt: #.getComponentList():
	        inStr = inStr + ", " + str(comp.getName())
	return inStr
  
  
    def makeClassInputs(self, indentation, inList):  #In Modelica, inputs and outputs mean parameters for functions
	inStr = '';
        for inpt in inList:
	    for comp in inpt: #.getComponentList():
	        inStr = inStr + '\n' + indentation + "self." + str(comp.getName()) + " = self.addInPort(" + "\"" + str(comp.getName())+ "\"" + ")"
	return inStr

    def makeClassOutputs(self, indentation, outList):  #In Modelica, inputs and outputs mean parameters for functions
	outStr = '';
        for outpt in outList:
	    for comp in outpt: #.getComponentList():
	        outStr = outStr + '\n' + indentation + "self." + str(comp.getName()) + " = self.addOutPort(" + "\"" + str(comp.getName())+ "\"" + ")"
	return outStr

    def makeSeqStates(self, indentation, sList):
	seqStr = ""
	#count = 0
	for sl in sList:
	    for s in sl:
		seqStr = seqStr + "\n" + indentation + str(s.getName()) + " = " + '\'' + str(s.getName()) + '\'' #str(count)
		#count = count + 1
	return seqStr
    
    #judge whether the declaration is a DEVS model declaration in coupledDEVS
    def isSubModelDeclaration(self, cClause):
        self.clauseType = str(cClause.getType())
	#print self.clauseType
	cEntry = self.currentScope.lookup(self.currentClass)
	if(cEntry!=None):
	  cDef = cEntry.getDefinition()
	  if(cDef.getExtends()=='CoupledDEVS'):
	    defEntry = self.currentScope.lookup(self.clauseType)	
	    if(defEntry != None):
	      defn = defEntry.getDefinition()	  
	      if(defn!=None) and (defn.__class__.__name__ == "RegularClassDefinition"):
	        if( (defn.getExtends()=='AtomicDEVS' or defn.getExtends()=='CoupledDEVS') ):
	          return True
	return False

    #-------------------------------------------------------------------------------------

    def visitExpListExp(self, r):
        # Hacked together...
        if self.output[-1] != "[":
                self.output += "["
        for i in r.getOperands():
                i.accept(self)
                self.output += ","
        # Cut of the trailing ,
        self.output = self.output[:-1]
        if self.output[-1] != "]":
                self.output += "]"
    
    def visitRegularClassDefinition(self, r):
	# dump class prefix, class name ...        	
        if(r.getExtends()=="DevsState"):
	  self.inDevsState = True
	if(r.getExtends()=='DevsExperiment'):
	  self.inExperiment = True
	  self.experimentName = str(r.className)
	self.currentScope = r.getScope()
	indentation=self.indentLevel*self.indent

	self.output=self.output+'\n'+indentation+str(r.prefix)

	#if(r.kind=="function" and string.upper(r.getFormalism())=="DEVS" and len(r.getSource())>0):
	#  self.output=self.output+ 'from ' + r.getSource() + ' import ' + str(r.className)
	if(r.kind=="class" or r.kind=="function" or r.kind=="package"):
	  if(r.kind=='function'):
	    self.inFunction = True
	    if(self.inClass):
              if(str(r.className)=="initialization"):
	        self.output = self.output + '\n' + indentation + 'self.initialization()' 

	      self.indentLevel = self.oldIndentLevel+1
              indentation=self.indentLevel*self.indent		
	      self.output=self.output+ '\n' + indentation + 'def '\
	                 +str(r.className) +'( ' + self.makeFunctionInputs( r.getInputs())+' ):'
              self.output += self.makeOutputInits(self.indent*(self.indentLevel+1), r.getOutputs())
	  elif(r.kind=='class'):
            self.inClass = True
	    self.currentClass = str(r.getName())
	    self.oldIndentLevel = self.indentLevel
	    if(len(r.getExtends()) > 0):
	      #if(self.inExperiment):
	      #	self.output=self.output + 'if __name__ == \'__main__\':'
	      #else:
		if(r.getExtends()=='DevsEvent' or r.getExtends()=='DevsState' or r.getExtends()=='DevsExperiment'):
		  self.output=self.output+str(r.kind)+' '+ str(r.className) + '( object ):'
		elif(r.getExtends()=='DevsPort'):
		  self.output=self.output+str(r.kind)+' '+ str(r.className) + '( Port ):'
		else:
	          self.output=self.output+str(r.kind)+' '+ str(r.className) + '( ' + r.getExtends() + ' ):'
	    else:
	      self.output=self.output+str(r.kind)+' '+str(r.className) + ':'
	      
            #if( string.upper(r.getFormalism())=="DEVS" and len(r.getBase())>0):		 
	    #  self.output=self.output+str(r.kind)+' '+ str(r.className) + '( ' + r.getBase() + ' ):'
       	    if not r.stringComment==None:
              self.output=self.output+' #'+str(r.stringComment)

            #if(r.isDevsModel() and len(r.getSeqStates())>0):
            if(len(r.getSeqStates())>0):
              self.indentLevel=self.indentLevel+1
              indentation=self.indentLevel*self.indent		
	      self.output=self.output+self.makeSeqStates(indentation,r.getSeqStates())
              self.indentLevel=self.indentLevel-1
                #indentation=self.indentLevel*self.indent
	
	  if(r.kind=='class'): # and not self.inExperiment):
	    if(r.hasIDDeclaration()[0]):
	      #print 'found ID'
              self.indentLevel=self.indentLevel+1
              indentation=self.indentLevel*self.indent		
	      self.output=self.output + '\n' + indentation + str(r.hasIDDeclaration()[1]) + 'Number = 0'
              self.indentLevel=self.indentLevel-1

            self.indentLevel=self.indentLevel+1
	    indentation=self.indentLevel*self.indent
	    #generate __init__ function for a class definition
	    paraStr = self.makeParameters(r.getParameters())
	    self.output = self.output + '\n' + indentation + 'def __init__(' + paraStr + '):'
            self.indentLevel=self.indentLevel+1
            indentation=self.indentLevel*self.indent
            if(r.getExtends()=='AtomicDEVS'):
		#self.output = self.output + '\n'+ indentation+'AtomicDEVS.__init__(self, name)'
		self.output = self.output + '\n'+ indentation+'AtomicDEVS.__init__(self)'
	    if(r.getExtends()=='CoupledDEVS'):
		#self.output = self.output + '\n'+ indentation+'CoupledDEVS.__init__(self, name)'
		self.output = self.output + '\n'+ indentation+'CoupledDEVS.__init__(self)'
	    varStr = self.makeParameterVariables(indentation, r.getParameters())
	    inpStr = self.makeClassInputs(indentation, r.getInputs())
	    outStr = self.makeClassOutputs(indentation, r.getOutputs())
	    self.output = self.output + varStr #'\n' + indentation + varStr
	    self.output = self.output + inpStr #'\n' + indentation + inpStr
	    self.output = self.output + outStr #'\n' + indentation + outStr
            #othStr = self.makeClassVariables(indentation, r.getVars())
            #self.output = self.output + othStr
            self.indentLevel = self.indentLevel - 1

	  if len(r.elementList)>0:
            #self.output = self.output + '\n'
#	    print r.__class__
#	    print r.className
            if(r.kind!='package'):
              self.indentLevel=self.indentLevel+1
            indentation=self.indentLevel*self.indent
	    visibility='DEFAULT'
	    for i in r.elementList:
		#print i.__class__
		v=i.getVisibility()
		if not (v==visibility):
		    if not v=='DEFAULT':
			if v=='PUBLIC':
			    self.output=self.output+indentation+'public\n'
			else:
			    self.output=self.output+indentation+'protected\n'
		    visibility=v
		i.accept(self)
                #self.output = self.output + '\n'
		#print i.__class__
	    if(r.kind!='package'):
	      self.indentLevel=self.indentLevel-1
	      
	      
          #Add sub models to coupledDEVS
          if(r.kind=='class' and r.getExtends=='CoupledDEVS'):
	    self.makeSubModels(r)
	
	
  	  # dump initial equations	    
	  if len(r.initEquationPart)>0:
	    indentation=self.indentLevel*self.indent
	    self.output=self.output+indentation+'initial equation \n'
	    self.indentLevel=self.indentLevel+1
	    for i in r.initEquationPart:
		i.accept(self)
		self.output=self.output+'\n'
	    self.indentLevel=self.indentLevel-1
	  # dump equations
          if len(r.equationPart)>0:
	    indentation=self.indentLevel*self.indent
            self.indentLevel=self.indentLevel+1
            for i in r.equationPart:
                self.output=self.output+indentation+'\n'
                i.accept(self)
            self.indentLevel=self.indentLevel-1
	  # dump initial alg
          if len(r.initAlgorithmPart)>0:
	    indentation=self.indentLevel*self.indent
            self.output=self.output+indentation+'initial algorithm \n'
            self.indentLevel=self.indentLevel+1
            for i in r.initAlgorithmPart:
                i.accept(self)
                self.output=self.output+'\n'
            self.indentLevel=self.indentLevel-1

          if len(r.algorithmPart)>0:
            self.indentLevel=self.indentLevel+1
            for i in r.algorithmPart:
                self.output=self.output+'\n'
                i.accept(self)

            self.indentLevel=self.indentLevel-1
	  # dump 'end' & class name
	  if(r.kind=='function'):
	    if(len(r.getOutputs())>0):	    
	      self.indentLevel = self.indentLevel+1
	      indentation = self.indentLevel* self.indent
	      self.output=self.output+indentation+'return '+str(r.getOutputs()[0][0].getName())
	      self.indentLevel=self.indentLevel-1
	    if('intTransition' in str(r.getName()) or 'extTransition' in str(r.getName())):
	      self.indentLevel = self.indentLevel+1
	      indentation = self.indentLevel* self.indent

	      if('intTransition' == str(r.getName()) or 'extTransition' == str(r.getName())):
                self.output=self.output+indentation+'return self.state' #+str(r.getOutputs()[0][0].getName())
              else:
                self.output=self.output+indentation+'return state' #+str(r.getOutputs()[0][0].getName())
	      self.indentLevel=self.indentLevel-1

	  
	  if(r.kind=='class' and self.indentLevel>0):
	    self.indentLevel = self.indentLevel -1
	  	
	  if(r.kind=='class' and not self.inExperiment):	      
	    #generate __str__ function for a class definition
	    self.indentLevel = self.indentLevel + 1
	    indentation=self.indentLevel*self.indent
	    self.output = self.output + '\n\n' + indentation + 'def __str__(self):'
	    self.indentLevel = self.indentLevel + 1
	    indentation=self.indentLevel*self.indent
	    self.output = self.output + r.toString(self.indent, self.indentLevel)
	    self.indentLevel = self.indentLevel - 1
	    #generate toXML function for a class definition
	    indentation=self.indentLevel*self.indent
	    self.output = self.output + '\n\n' + indentation + 'def toXML(self):'
	    self.indentLevel = self.indentLevel + 1
	    indentation=self.indentLevel*self.indent
	    self.output = self.output + r.toXML(self.indent, self.indentLevel)
	    self.indentLevel = self.indentLevel - 1
	    self.indentLevel = self.indentLevel - 1

          if(r.kind == 'class' and self.inExperiment):
            self.indentLevel += 1
            indentation = self.indentLevel * self.indent
	    self.output = self.output + '\n' + indentation + 'def simulate(self):'
            self.indentLevel += 1
            indentation = self.indentLevel * self.indent
	    #self.output = self.output + '\n' + indentation + 'self.sim.simulate(termination_condition=terminate_at(self.end_time), verbose=self.verbose, trace=self.trace)'
	    self.output = self.output + '\n' + indentation + 'self.sim.simulate(termination_condition=terminate_at(self.end_time), verbose=self.verbose)'
            self.indentLevel -= 1
            self.indentLevel -= 1

	  self.output=self.output+'\n'

          if(r.kind=='class'):
            self.inClass=False
	  if(r.kind=='function'):
            self.inFunction=False
	  if(self.inDevsState):
	    #print r.toXML()
	    self.inDevsState = False
	  if(self.inExperiment):
	    self.inExperiment = False
	  self.currentScope = self.currentScope.getOuter()



    def visitShortClassDefinition(self, s):
	indentation=self.indentLevel*self.indent
	self.output=self.output+indentation+str(s)+'\n'

    def visitEnumClassDefinition(self, eClassDefinition):	
	# EnumClassDefinition has been transformed to 
	# RegularClassDefinition
	pass

    def visitComponentClause(self, cClause):
        self.clauseType = str(cClause.getType())
	if(self.isSubModelDeclaration(cClause)):
	  self.isSubModelClause = True

	if(str(cClause.getTypePrefix())==''):
	  indentation=self.indentLevel*self.indent
	  dList=[]
	  dList = cClause.getComponentList()
	  for d in dList:
            self.output = self.output + '\n'
            self.output = self.output + indentation
            d.accept(self)
	
        if(self.isSubModelClause):
	  self.isSubModelClause=False

    def visitDeclaration(self, decl):
      #if(self.inClass and (not self.inFunction)): # and (not self.inExperiment)):
      #  self.output = self.output + 'self.'
      self.declName = str(decl.getName())
      decl.getName().accept(self)
      self.output = self.output + ' = '
      if(string.strip(str(decl.comment))=='\"ID\"'):
	self.output = self.output + self.currentClass + "." + str(decl.getName())  + 'Number + 1'
        indentation = self.indentLevel * self.indent
	self.output = self.output + '\n' + indentation + self.currentClass + "." + str(decl.getName())  + 'Number = ' \
	            + self.currentClass + "." + str(decl.getName()) + 'Number + 1'
      else:
        if(decl.getModification()!=None):
          decl.getModification().accept(self)
        else:
          if( self.clauseType=='Integer'):
	    self.output = self.output + "0"
	  elif( self.clauseType=='Real'):
	    self.output = self.output + "0.0" 
	  elif( self.clauseType=='Boolean'):
	    self.output = self.output + "False" 
	  elif( self.clauseType=='String'):
	    self.output = self.output + "\"\""
	  else:
            if(decl.getArrSub()!=None): #Array declaration
              self.output = self.output + "[]"
            else:
              self.output = self.output + "None"

    def visitClassModification(self, cm):
	#print cm
	argList = cm.getArgumentList()
        if( self.clauseType=='Integer'\
          or self.clauseType=='Real'\
          or self.clauseType=='Boolean'\
          or self.clauseType=='String'\
          or string.find(self.clauseType, 'SeqStates')!=-1):
 	  for arg in argList:
	    arg.accept(self)
        else:
          if(self.clauseType=='list'):
	    self.output = self.output + '[]'
          else:
            self.output = self.output + self.clauseType + '('
            if (len(argList) > 0) and ("name" in str(argList[0])) and (self.isSubModelClause):
                argList = argList[1:]
	    if(len(argList)>0):
	      argList[0].accept(self)
	    if(len(argList)>1):
	      for arg in argList[1:]:
	        self.output = self.output + ', '
	        arg.accept(self)

	    if(self.isSubModelClause):
	      if( len(argList)==0):
	        self.output = self.output + 'name=\'' + self.declName + '\''
	      if(len(argList)>0):
	        self.output = self.output + ', name=\'' + self.declName + '\''

            self.output = self.output + ')'
	    if(self.isSubModelClause):
	      indentation = self.indentLevel * self.indent
	      self.output = self.output + '\n' + indentation + 'self.addSubModel( self.' + self.declName + ' )'
    #self.GDEVS = self.addSubModel(Generator(ia,ib,sa,sb, name="GeneratorDEVS"))
	  
    def visitElementModification(self, elm):
	#print elm
        if( self.clauseType=='Integer'\
          or self.clauseType=='Real'\
          or self.clauseType=='Boolean'\
          or self.clauseType=='String'\
          or string.find(self.clauseType, 'SeqStates')!=-1):
          if(str(elm.getCompRef())=='start'):
            elm.getModification().accept(self)
        else:
          self.inElLeft = True
	  #if(str(elm.getCompRef())=='simMode'):
	  elm.getCompRef().accept(self)
          self.inElLeft = False
          self.output = self.output + ' = '	
	  elm.getModification().accept(self)
	  #print elm.getModification()
	  #print elm.getModification().__class__
	
    def visitEqualsModification(self, eqm):
	eqm.getExp().accept(self)
	#print eqm.getExp()
	#print eqm.getExp().__class__
	    
    def visitReplaceableClause(self, rc):
	indentation=self.indentLevel*self.indent
	if not rc.classDefinition==None:
	    rc.classDefinition.accept(self)
	elif not rc.componentClause==None:
	    rc.componentClause.accept(self)
	elif not rc.constrainingClause==None:
	    self.output=self.output+str(rc.constrainingClause)+'\n' 
	if not rc.comment==None:
	    self.output=self.output+str(rc.comment)+'\n'
	
	
    def visitImportClause(self, importClause):
	pass

    def visitExtendsClause(self, extendsClause):
	pass

    # ------------------------------EqStm-----------------------------
    def visitRegularEqStm(self, eq):
	self.output=self.output+self.eqPrinter.dumpIndentEq(eq, \
		self.indentLevel, self.indent)+'\n'

    def visitConnectEqStm(self, eq):
        indentation=self.indentLevel*self.indent
	self.output = self.output + indentation + 'self.connectPorts( '
	eq.getFirstConnector().accept(self)
	self.output = self.output + ', '
	eq.getSecondConnector().accept(self)
	self.output = self.output + ')'

    def visitConnectRefExp(self, exp):
      self.output = self.output + 'self.'+str(exp)
      

    def visitFunCallEqStm(self, eq):
        #print eq
	self.output=self.output+self.eqPrinter.dumpIndentEq(eq, \
                self.indentLevel, self.indent)+'\n'

    def visitConditionalEqStm(self, eq):
        indentation = self.indentLevel * self.indent
	if(self.inElseif):
          self.output=self.output + '('
	  self.inElseif = False
	else:
          self.output=self.output+indentation+'if('
	
        eq.getIfTest().accept(self)
	self.output=self.output + '):\n'
	
	for i in eq.getIfBody():
	   self.indentLevel=self.indentLevel+1
	   i.accept(self) 
	   self.indentLevel=self.indentLevel-1
	if len(eq.getElseifList())>0:
	   self.output=self.output+indentation+'elif'
	   #self.indentLevel=self.indentLevel+1
	   for j in eq.getElseifList():
	     self.inElseif=True
	     j.accept(self)
	     self.inElseif=False
	   #self.indentLevel=self.indentLevel-1
	if len(eq.getElseBody())>0:
	   self.output=self.output+indentation+'else:\n'
	   self.indentLevel=self.indentLevel+1
	   for k in eq.getElseBody():
	     k.accept(self)
	   self.indentLevel=self.indentLevel-1

    def visitForEqStm(self, eq):
	#print eq
	#print eq.__class__
	ind = eq.getForIndices()
	ind.accept(self)
        self.indentLevel = self.indentLevel + 1
	for b in eq.getBody():
	  b.accept(self)
	self.indentLevel = self.indentLevel - 1


    def visitForIndicesExp(self, exp):
	indentation = self.indentLevel * self.indent
	self.output=self.output+indentation+'for '  #str(exp)+'\n'
	for o in exp.getOperands():
	  o.accept(self)
	self.output=self.output+':\n'
	
    def visitWhileAlgStm(self, exp):
	indentation = self.indentLevel * self.indent
	self.output=self.output+indentation+'while '  #str(exp)+'\n'
	#for o in exp.whileTest:
	#  o.accept(self)
        exp.whileTest.accept(self)
	self.output=self.output+':\n'
        self.indentLevel += 1
        indentation = self.indentLevel * self.indent
        for o in exp.body:
          o.accept(self)
        self.indentLevel -= 1
        indentation = self.indentLevel * self.indent
	
    def visitIdentInExpExp(self, exp):
	oper = exp.getOperands()
	if oper!=None:
	  self.output=self.output+str(oper[0])+' in '
	  oper[1].accept(self)
	  
    def visitRangeExp(self, exp):
        self.output=self.output+'range('
        opList = exp.getOperands()
        opList[0].accept(self)
        self.output=self.output+', '
        opList[1].accept(self)
        self.output=self.output+')'
	
    def visitWhenEqStm(self, eq):
	self.output=self.output+self.eqPrinter.dumpIndentEq(eq, \
                self.indentLevel, self.indent)+'\n'

    def visitAtomicAlgStm1(self, eq):  #Added by Hongyan Song, Mar 27, 2006
	#print eq
	indentation = self.indentLevel * self.indent

	self.output = self.output + indentation
	eq.compRef.accept(self)
        if(eq.exp!=None):
	  #print eq.exp
	  #print eq.exp.__class__
	  self.output = self.output + ' = '
	  eq.exp.accept(self)
        if(eq.funCall!=None):
	  #print eq.funCall
          #print eq.funCall.__class__
	  self.output = self.output + '('
	  eq.funCall.accept(self)
	  self.output = self.output + ')'

        if not eq.comment==None:
            self.output=self.output + ' #'+str(eq.comment)
        self.output = self.output+'\n'

    def visitCompRefExp(self, exp):
      self.inCompRef = True
      if(str(exp).find('.SeqStates.')!=-1):
        self.output = self.output + str(exp).replace('.SeqStates.', '.')
	return
      if(self.inFunction):
        #if(self.currentScope.lookup(str(exp))==None and str(exp).find('SeqStates')==-1):
        baseexp = str(exp.operands[0].operands[0])
        #if(self.currentScope.lookup(str(exp))==None and
        if(self.currentScope.lookup(str(baseexp))==None and
                not str(exp).startswith("state") and
                "State" not in str(exp)
                ):
          #TODO only self here?
          self.output = self.output + 'self' #+ str(exp)      
      for op in exp.getOperands():
	op.accept(self)
      self.inCompRef = False
	
    def visitSingleCompRefExp(self, exp):
      #if(self.inDevsState and str(exp)=='SeqStates'):
      if(self.inDevsState and 'SeqStates' in str(exp)):
	return
      else:
        for op in exp.getOperands():
	  op.accept(self)

    def visitArrSubExp(self, exp):
	self.output = self.output + '['
	for op in exp.getOperands():
          op.accept(self)
	self.output = self.output + ']'
	
    def visitLessGtExp(self, exp):
      #print exp
      opList = exp.getOperands()
      opList[0].accept(self)
      self.output = self.output + ' != '
      opList[1].accept(self)

    def visitLessExp(self, exp):
      #print exp
      opList = exp.getOperands()
      opList[0].accept(self)
      self.output = self.output + ' < '
      opList[1].accept(self)

    def visitLessEQExp(self, exp):
      #print exp
      opList = exp.getOperands()
      opList[0].accept(self)
      self.output = self.output + ' <= '
      opList[1].accept(self)

    def visitEQEQExp(self, exp):
      #print exp
      opList = exp.getOperands()
      opList[0].accept(self)
      self.output = self.output + ' == '
      opList[1].accept(self)

    def visitGreaterExp(self, exp):
      #print exp
      opList = exp.getOperands()
      opList[0].accept(self)
      self.output = self.output + ' > '
      opList[1].accept(self)

    def visitGreaterEQExp(self, exp):
      #print exp
      opList = exp.getOperands()
      opList[0].accept(self)
      self.output = self.output + ' >= '
      opList[1].accept(self)


    def visitFunCallExp(self, exp):
      #print exp      
      opList = exp.getOperands()
      #strList = []
      #print opList[0].__class__
      pos = string.find(str(opList[0]), '.')
      #if(pos!=-1):
      #	print opList[0]
      #  strList = string.split(str(opList[0]), '.')
      #  for s in strList:
      #	  print s
      #if(isExternalFunction(opList[0])):
      #	self.output = self.output + str(opList[0])
      if(pos!=-1):
	strList = string.split(str(opList[0]), '.')
	found = self.classMemberLookup(strList[0])
      else:
	found = self.classMemberLookup(str(opList[0]))
      #found = self.classMemberLookup(str(opList[0]))
      if(found != None) and ("X" != str(opList[0])[0]):
        self.output = self.output + 'self.' + str(opList[0]) #.accept(self)
      else:
        self.output = self.output + str(opList[0]) #.accept(self)
      #else:
      #  opList[0].accept(self)
	
      self.output = self.output + '('      
      opList[1].accept(self)
      self.output = self.output + ')'

    def visitSumExp(self, exp):
      opList = exp.getOperands()
      opList[0].accept(self)
      self.output = self.output + ' + '
      opList[1].accept(self)

    def visitIdentExp(self, exp):
      if(self.inExperiment and str(exp)=='simModel'):
	self.output = self.output + 'model'
      else:
        if(string.lower(string.strip(str(exp)))=='null'):
	  self.output = self.output + 'None'
        else:
          #if(self.currentScope.localLookup(str(exp))==None):
	  #  self.output = self.output + 'self.'+str(exp)
          #else:
	  if(string.strip(str(exp)) == 'SeqStates'):
	    self.output = self.output + self.currentClass
	  else:
            if(self.currentScope.localLookup(str(exp))==None):
              if(self.inCompRef):
                atomDevs = False
                defEntry = self.currentScope.lookup(str(exp))
                if defEntry != None:
                  defn = defEntry.getDefinition()
                  if defn.kind == 'class' and defn.getExtends()=='AtomicDEVS':
                    atomDevs = True
                if(atomDevs):
                  self.output = self.output + str(exp)
                else:
                  if(self.inElLeft):
                    self.output = self.output+ str(exp)
                  else:
                    if str(exp).endswith("State"):
                        if self.output[-1] in [" ", "(", ")", ","]:
                                self.output = self.output + str(exp)
                        else:
                                self.output = self.output + '.' + str(exp)
                    else:
                        if self.output[-1] in [" ", "(", ")", ","]:
                                self.output = self.output + 'self.' + str(exp)
                        else:
                                self.output = self.output + '.'  + str(exp)
              else:
                if(self.inFunction and str(exp)!='INFINITY' and str(exp) != "None"): #ad-hoc method
                  self.output = self.output + 'self.' + str(exp)
                else:
                  if "seqState" in str(exp):
                        self.output += "self."
                  self.output = self.output + str(exp)            
            else:
	      if(self.inFunction):
	        self.output = self.output + str(exp)
	      else:
	        #if(not self.inElLeft):
	        self.output = self.output + 'self.' + str(exp)
	        #else:
	        #  self.output = self.output + str(exp)

    def visitSubtractExp(self, exp):
      opList = exp.getOperands()
      opList[0].accept(self)
      self.output = self.output + ' - '
      opList[1].accept(self)

    def visitRealExp(self, exp):
      self.output = self.output + str(exp)

    def visitBoolExp(self, exp):
      self.output = self.output + str(exp).title()

    def visitIntegerExp(self, exp):
      self.output = self.output + str(exp)

    def visitFunArgsExp(self, exp):
      #print exp
      opList = exp.getOperands()
      if(len(opList)>0):
	#print opList[0]
	opList[0].accept(self)
      if(len(opList)>1):
        for op in opList[1:]:
	  self.output = self.output + ', '
	  op.accept(self)

	#pass
    def visitNamedArgExp(self, exp):
	#print exp
	#print dir(exp)
	self.output = self.output + str(exp)
		
    def visitAnnotationExp(self, exp): #Added by Hongyan Song, Mar 28, 2006
	#print dir(exp)
	#self.output=self.output+exp.dump(self.indentLevel)
	pass

    def visitStringExp(self, exp):
      self.output = self.output + str(exp)

    def classMemberLookup(self, mName):
      cDef = self.currentScope.lookup(self.currentClass).getDefinition()      
      found = None

      while( cDef != None and found == None):
	scope = cDef.getScope()
	found = scope.localLookup(mName)
	if(found!=None):
	  return found
	else:
	  if(len(cDef.getExtends())>0):
	    cDef = self.currentScope.lookup(cDef.getExtends()).getDefinition()
	  else:
	    cDef = None
      return found
