
from xml.dom.minidom     import *
from Tkinter             import *                  # get widget classes
from sets                import Set

class StateParser(object):
  def getString(self, xmlState):
    pass

  def getSeqState(self, xmlState):
    pass

class SimpleStateParser(object):
  def __init__(self, attrName):
    self.__attrName = attrName

  def getString(self, xmlState):
    for node in xmlState.childNodes:
      if node.nodeType == Node.CDATA_SECTION_NODE:
        return node.nodeValue  #node.childNodes[0].data
    return "No state string"

  def getSeqState(self, xmlState):
    attributes = xmlState.getElementsByTagName("attribute")
    for attribute in attributes:      
      attrName = attribute.getElementsByTagName("name")[0]
      txtName=attrName.childNodes[0].data
      #print txtName, "--", self.attrName
      if(txtName==self.__attrName):
        attrValue = attribute.getElementsByTagName("value")[0]
        txtValue=attrValue.childNodes[0].data
        return txtValue
    return "Error"

#--------------------------------------------------------------
# DEVS event used by the visual plotter
#------------------------------------------------------------------------------
class DevsEvent(object):
  def __init__(self, time=0.0, type='', state='', content=''):
    self.__time     = time         #the time when the event occurs
    self.__type     = type         #the type of the event, internal or external
    self.__state    = state        #at which phase/state when the event happens
    self.__content  = content      #the content of the event
    
  def __str__(self):
    return '\nEvent: \n\t%s\n\t%s\n\t%s\n\t%s' % (str(self.__time), self.__type, self.__state, self.__content) 

  def getTime(self):
    return self.__time

  def getType(self):
    return self.__type

  def getState(self):
    return self.__state

  def getContent(self):
    return self.__content

#--------------------------------------------------------------
# save the intermediate format of events
# this format cannot be viewed by DEVS Plotter, because the state part are in
# raw xml format
#------------------------------------------------------------------------------
class TmpEvent(object):
  def __init__(self, time=0.0, type='', portInfo='', state=''):
    self.__time     = time         
    self.__type     = type
    self.__portInfo = portInfo     
    self.__xmlState = state     
    
  def __str__(self):
    return ( '\nEvent:\n\t%s\n\t%s\n\t%s\n\t%10s'
           % (str(self.__time), self.__type, self.__portInfo, self.__xmlState )
           )

  def getTime(self):
    return self.__time

  def getType(self):
    return self.__type

  def getPortInfo(self):
    return self.__portInfo

  def getXmlState(self):
    return self.__xmlState

#--------------------------------------------------------------
#Parse the whole trace event
#------------------------------------------------------------------------------    
class TraceParser(object):
  def __init__(self, fileName):
    self.__fileName     = fileName
    self.__modelSet     = Set()
    self.__modelEvents  = {}  #maps between model name and model parser
    self.__modelState   = {}
    self.__modelType    = {}
#    self.__stateParsers = {} 
    self.__coupleComp   = {}
    self.__initialize()

  def __initialize(self):
    doc   = parse(self.__fileName)
    trace = doc.getElementsByTagName("trace")[0]
    self.__parseEvents(trace)


  def __getAttributes(self, node):
    attrList=[]
    for attribute in node.childNodes:
      if(attribute.nodeType==Node.ELEMENT_NODE):
        attrName = attribute.getElementsByTagName("name")[0]
        attrType = attribute.getElementsByTagName("type")[0]
        attrCategory = attribute.getAttribute("category")
        txtName=attrName.childNodes[0].data
        txtType=attrType.childNodes[0].data
        attrList.append(txtName) # + " " + txtType + " " + attrCategory)
    return attrList

#--------------------------------------------------------------
# traverse the trace file
# separate the trace events into according their model name
# the results are saved in a dictionary keyed by the model names
#-------------------------------------------------------------------------------
  def __parseEvents(self, trace):  
    events = trace.getElementsByTagName("event")
    for event in events:
      model = event.getElementsByTagName("model")[0]
      txtModel=model.childNodes[0].data
      self.__modelSet.add(txtModel)
      self.__modelType[txtModel]='A'
      num = txtModel.count('.')
      i = 0
      #print txtModel
      while (i < num):
        i +=1
        pModel = txtModel.split('.', i)[0]
        if(not pModel in self.__modelSet):
          self.__modelSet.add(pModel)
          self.__modelType[pModel] = 'C'
          self.__coupleComp[pModel] = Set()
          self.__coupleComp[pModel].add(txtModel)
        else:
          self.__coupleComp[pModel].add(txtModel)

    for model in self.__modelSet:
      self.__modelEvents[model]  = []
      self.__modelState[model]   = []
#      self.__stateParsers[model] = None

    events = trace.getElementsByTagName("event")
    for event in events:
      model = event.getElementsByTagName("model")[0]
      time  = event.getElementsByTagName("time")[0]
      kind  = event.getElementsByTagName("kind")[0]
      ports = event.getElementsByTagName("port")
      state = event.getElementsByTagName("state")[0]

      txtModel = model.childNodes[0].data
      txtKind  = kind.childNodes[0].data
      txtTime  = time.childNodes[0].data

      portInfo = ""
      for port in ports:
        portInfo += port.getAttribute("category")
        portInfo += " Port " + port.getAttribute("name") + ": "
        msg = port.getElementsByTagName("message")[0]
        portInfo += msg.childNodes[0].data 

      tEvent = TmpEvent(float(txtTime), txtKind, portInfo, state)
      self.__modelEvents[txtModel].append(tEvent)

      if(len(self.__modelState[txtModel])==0):
        self.__modelState[txtModel] = self.__getAttributes(state)

      num = txtModel.count('.')
      i = 0
      while (i < num):
        i +=1
        pModel = txtModel.split('.', i)[0]
        dEvt = DevsEvent(float(txtTime), '', txtModel, '') 
        self.__modelEvents[pModel].append(dEvt)

# -------------------------------------------------------------
# return a the type of a model, 'C' means a coupled model, 'A' means a atomic model
# -----------------------------------------------------------------------------
  def getModelType(self, model):  
    return self.__modelType[model]

# -------------------------------------------------------------
# return a List of sub-models of a coupled model
# -----------------------------------------------------------------------------
  def getSubComponents(self, model):
    sModels = self.__coupleComp[model]
    mList = []
    for m in sModels:
      mList.append(m)
    return mList

# -------------------------------------------------------------
# return a set of full model names included in the trace file
# -----------------------------------------------------------------------------
  def getModels(self):  
    return self.__modelSet

# -------------------------------------------------------------
# return a list of tmp events for the model specified by model name
# -----------------------------------------------------------------------------
  def getModelEvents(self, modelName):
    return self.__modelEvents[modelName]

# -------------------------------------------------------------
# return a list of model state attributes for the model specified by 
# the model name
# -----------------------------------------------------------------------------
  def getModelStateAttributes(self, modelName):
    return self.__modelState[modelName]

#  def getStateParsers(self):
#    return self.__stateParsers

#  def getStateParser(self, modelName):
#    return self.__stateParsers[modelName]

#  def setStateParser(self, modelName, stateParser):
#    self.__stateParsers[modelName] = stateParser

#-------------------------------------------------------------
# parse events of a specific model from intermediate format to the format that
# can be recognized by the visual plotter
#------------------------------------------------------------------------------
class ModelParser(object):
  def __init__(self, modelEvents, stateParser):
    self.__stateParser = stateParser
    self.__events      = []
    self.__states      = []

    stateSet = Set()
    for tEvt in modelEvents:
      time  = tEvt.getTime()
      type  = tEvt.getType()
      state = self.__stateParser.getSeqState(tEvt.getXmlState())
      stateSet.add(state)
      content = tEvt.getPortInfo() \
              + self.__stateParser.getString(tEvt.getXmlState())
      
      dEvt = DevsEvent(time, type, state, content)
      self.__events.append(dEvt)


    # convert from set to list
    for state in stateSet:
      self.__states.append(state)

  def getStateParser(self):
    return self.__stateParser

  def getEvents(self):
    return self.__events

  def getStates(self):
    return self.__states
    
if __name__ == '__main__':
  import sys  
  if(len(sys.argv)<2):
    print "\nUsage: python eventparser.py XMLFILE"
    sys.exit(2)

  tParser = TraceParser(sys.argv[1])
  tParser.setStateParser('theSystem.Processor1', SimpleStateParser('processorStatus'))
  tParser.setStateParser('theSystem.Processor0', SimpleStateParser('processorStatus'))
  tParser.setStateParser('theSystem.GeneratorDEVS', SimpleStateParser('first'))
  modelSet = tParser.getModels()
  for model in modelSet:
    modelEvents=tParser.getModelEvents(model)
    print model
    modelParser =  ModelParser(modelEvents, tParser.getStateParser(model))
    devsEvents  =  modelParser.getEvents()
    i = 0
    for event in devsEvents:
      if(i < 10):
        print "\t%s" % event
        i = i+1
