# coding: utf-8
# MuModelica Compiler Project
# Author: Steven Xu
# Modeling, Simulation and Design Lab (MSDL)
# School of Computer Science
# McGill University
# Modified by Hongyan Song for supporting DEVS formalism

import os,sys
from optparse import OptionParser
from string import replace

# from the Front End
from FrontEnd.XMLReader import XMLReader
from FrontEnd.ASTbuilder import ASTbuilder
from AST.ASTDefinition import AST
from FrontEnd.Parser import Parser
from FrontEnd.Scoper import Scoper
from FrontEnd.InheritanceExpander import InheritanceExpander
from FrontEnd.Lookuper1 import Lookuper1
from FrontEnd.Lookuper2 import Lookuper2
from FrontEnd.Instantiater import Instantiater
from FrontEnd.Flattener import Flattener
from FrontEnd.FlatModelicaNameLookuper import FlatModelicaNameLookuper
from FrontEnd.TypeCheckVisitor import TypeCheckVisitor
from FrontEnd.printers.ModelicaPrinter import ModelicaPrinter
from FrontEnd.printers.FlatModelPrinter import FlatModelPrinter
from devs.DevsPrinter import DevsPrinter
from devs.DevsFlattener import DevsFlattener

# from the Back End
from BackEnd.CanonicalTransformer import CanonicalTransformer
from BackEnd.causality.CausalityAssigner import CausalityAssigner
from BackEnd.causality.Sorter import Sorter
from BackEnd.causality.EquationRewriter import EquationRewriter

# from Code Generator
from CodeGenerator.OctaveGenerator import OctaveGenerator


# ----------------------------main()-----------------------------
def main():
    # -------------argument handling-------------------
    usage = "usage: python %prog [options] arg \n \
		where: 	arg: either a Modelica source file *.mo or \n\
                             an XML source file *.agl"
    parser = OptionParser(usage, version="MuModelica Compiler: mc 1.0")
    #parser.add_option("-e", action="store_false", dest="aliases", 
	#	default=True, help="switch off aliases-elimination")
    #parser.add_option("-t", action="store_false", dest="tearing", 
	#	default=True, help="switch off tearing")
    #parser.add_option("-i", action="store_false", dest="inline", 
	#	default=True, help="switch off inline integration")
    parser.add_option("-d", action="store_true", dest="dumpIntmd", 
		default=False, help="display intermediate result \
				     at each compilation step")
    parser.add_option("-f", action="store_true", dest="flatten",
                default=False)

    (options, args) = parser.parse_args()
    if len(args)!=1:
	parser.error("incorrect number of arguments!")
    if args[0][(len(args[0])-3):(len(args[0]))]=='.mo':
	moFilename=args[0]
	xmlFilename=args[0][0:(len(args[0])-3)]+'.agl'
	octaveFilename=args[0][0:(len(args[0])-3)]+'.m'
    elif args[0][(len(args[0])-4):(len(args[0]))]=='.agl':
	xmlFilename=args[0]
	moFilename=None
	octaveFilename=args[0][0:(len(args[0])-4)]+'.m'
    else:
	parser.error("The name of the file to be compiled is illegal!")

    # set the dump flag
    dumpFlag = options.dumpIntmd


    # -----------------------The Front End -----------------------
    
    parserPath = os.path.abspath(os.path.dirname(sys.argv[0]))
    #print "parserPath: "+parserPath
    filePath = os.path.abspath(os.path.dirname(sys.argv[1]))
    #print "filePath: "+filePath
    parser=Parser()
    ast=AST()
    # load main model
    classFile=parser.parse(parserPath, 'xmlDumper', filePath, \
                moFilename, xmlFilename, moFilename)
    ast.addClassFile(classFile)

    moFilenameFlat="flatten.mo"
    xmlFilenameFlat="flatten.agl"
    filePath+="/lib"
    parser = Parser()
    flatClassFile=parser.parse(parserPath, 'xmlDumper', filePath, \
                moFilenameFlat, xmlFilenameFlat, moFilenameFlat)
    ast.addClassFile(flatClassFile)

    # load predefined package
    predefinedLibPath=parserPath+os.sep+'lib/predefined/Predefined.agl'
    pyGKPath=parserPath
    classFileOfPredefined=parser.loadXML(predefinedLibPath, \
					parserPath, 'Predefined')
    ast.setPredefined(classFileOfPredefined)
    #print parser.msg

    
#    print ast.classFiles
    
    # get MODELICAPATH
    stringPath=os.getenv("MODELICAPATH")
    # the following code is to separate the paths joined by ":"s
    MODELICAPATH=[]
    path=''
    for i in range(0, len(stringPath)):
  	if stringPath[i]==":":
    	    MODELICAPATH.append(path)
    	    path=''
  	else:
    	    path=path+stringPath[i]
	    if i==len(stringPath)-1:
		MODELICAPATH.append(path)	

#    print 'here'
    # scoping analysis: build symbol tables
    scopingVisitor=Scoper(ast, MODELICAPATH, parserPath, 'xmlDumper')
    ast.accept(scopingVisitor)
#    print 'here 1'
    # name lookup
    lookuper1=Lookuper1(ast, MODELICAPATH, parserPath, 'xmlDumper')
    ast.classFiles[0].accept(lookuper1)
    i=1
    while (i<len(ast.classFiles)):
	ast.classFiles[i].accept(lookuper1)
	i=i+1
    #if dumpFlag:
    #	print '\n****************** Type_specifiers Lookup *****************'
    #	print lookuper1.msg


    # write out Modelica file from AST
    modelicaPrinter1=ModelicaPrinter()
    ast.accept(modelicaPrinter1)
    if dumpFlag:
	#print "\n************** Original Modelica model *************"
	#print modelicaPrinter1.getOutput()
    	modelicaOut=moFilename[0:(len(moFilename)-3)]+'.out.mo'
    	modelicaPrinter1.writeToFile(modelicaOut)

    if options.flatten:
        # Flatten for DEVS models
        devsFlatten = DevsFlattener(ast)
        ast = devsFlatten.go()

        modelicaOut = moFilename[0:(len(moFilename)-3)]+'.flat.mo'
        modelicaPrinterDEVSFlat = ModelicaPrinter()
        ast.accept(modelicaPrinterDEVSFlat)
        modelicaPrinterDEVSFlat.writeToFile(modelicaOut)

    if not options.flatten:
        # Generate PythonDEVS code; Added by Hongyan Song, Mar 28, 2006
        devsPrinter=DevsPrinter(MODELICAPATH)
        ast.accept(devsPrinter)
        #if dumpFlag:
	        #print "\n************** Original Modelica model *************"
	        #print modelicaPrinter1.getOutput()
        devsOut=moFilename[0:(len(moFilename)-3)]+'.py'
        devsPrinter.writeToFile(devsOut)

    # name lookup
#    lookuper2=Lookuper2(ast, None)
#    ast.accept(lookuper2)
    #if dumpFlag:
  	#print '\n********************** Name Lookup *********************'
    	#print lookuper2.msg

'''    # expanding inheritance
    inheritanceExpander=InheritanceExpander(ast)
    ast.accept(inheritanceExpander)

    # instantiation
    instantiater=Instantiater(ast)
    instantiater.go()

    # flattning
    flattener=Flattener(ast)
    flatClassDef=flattener.go()

    # dump Flat Modelica
    flatModelicaPrinter=FlatModelPrinter()
    flatClassDef.accept(flatModelicaPrinter)
    if dumpFlag:
    	print '\n**************** Flat Modelica model ***************'
    	print flatModelicaPrinter.getOutput()

    # name lookup in Flat Modelica
    lookuper3=FlatModelicaNameLookuper()
    ast.accept(lookuper3)
    #if dumpFlag:
	#print '\n************ Name Lookup in Flat Modelica *************'
    	#print lookuper3.msg

    # type check
    #print '\n*********************** Type Check *********************'
    typeChecker=TypeCheckVisitor(ast)
    typeCheckMsg=typeChecker.go()
    #if dumpFlag:
	#print typeCheckMsg
    #except:
        #print sys.exc_type

    # -----------------------The Back End -----------------------
    flatMoClassDef=ast.classFiles[0].classDefinitions[0]
    canonical=CanonicalTransformer(flatClassDef)
    canonical.go()
    #print '\n*********************Canonical Form********************'
    #canonicalprinter=FlatModelPrinter()
    #flatClassDef.accept(canonicalprinter)
    #print canonicalprinter.getOutput()

    # causality assignment
    ca = CausalityAssigner(ast)
    G = ca.go()
    #if dumpFlag:
	#print '\n****************Causality Assignment*******************'
	#print ca.msg

    # sort equations and detect algebraic loops
    s = Sorter()
    strComps = s.go(G)
    #if dumpFlag:
	#print s.msg

    # rewrite equations into explicit form
    ER = EquationRewriter()
    strComps=ER.go(strComps)
    if dumpFlag:
	print '\n************* Causal Model *************'
    	print ER.dump(strComps)

    # need to transform the explicit equations into canonical form again
    # to be finished ...

    # ----------------------- Code Generation ----------------------
    #print '************************octave code***********************'
    octaveGen=OctaveGenerator()
    octaveGen.go(flatClassDef, strComps, octaveFilename)
    if dumpFlag:
	print '\n************* Octave Model **************'
    print octaveFilename+" has been generated!"
'''
#-----------------------------end of main()------------------------------

main()

