model Circuit
  Resistor R1(r=1);
  Capacitor C(c=1);
  Resistor R2(r=1);
  VsourceAC AC;
  Ground G;
equation 
  connect(AC.p, R1.p);
  connect(C.n, AC.n);
  connect(R1.n, R2.p);
  connect(R2.n, C.p);
  connect(AC.n, G.p);
end Circuit;

model Capacitor "Ideal electrical capacitor"
  extends TwoPin;
  parameter Predefined.Real c(unit="F") "Capacitance";
equation 
  c*der(v)=i;
end Capacitor;

model Resistor "Ideal electrical resistor"
  extends TwoPin;
  parameter Real r(unit="Ohm") "Resistance";
equation 
  r*i=v;
end Resistor;

model Ground "Ground"
  Pin p;
equation 
  p.v=0;
end Ground;

model VsourceAC "Sin-wave voltage source"
  extends TwoPin;
  parameter Voltage VA=110 "Amplitude";
  parameter Real f(unit="Hz")=1 "Frequency";
  constant Real pi=3.14;
equation 
  v=VA*sin(2*f*pi*time);
end VsourceAC;

partial model TwoPin "Superclass of elements with two electrical pins"
  Pin p, n;
  Voltage v;
  Current i;
equation 
  v=p.v-n.v;
  0=p.i+n.i;
  i=p.i;
end TwoPin;

connector Pin
  Voltage v;
  flow Current i;
end Pin;

type Voltage = Real(unit="V");

type Current = Real(unit="A");

