
import sys
import os.path
sys.path.append(os.path.expanduser('pydevs/'))
from DEVS import *
from infinity import *
from random import randint
from simulator import *

class GeneratorState:
  def __init__(self):
    self.seqState = Generator.generating

  def __str__(self):
    strRep = ''
    strRep = strRep + "\nseqState: " + str(self.seqState)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"P\">"
    strRep = strRep + "\n\t<name>seqState</name>"
    strRep = strRep + "\n\t<type>Generator.SeqStates</type>"
    strRep = strRep + "\n\t<value>"+str(self.seqState)+"</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class Generator( AtomicDEVS ):
  generating = 'generating'
  def __init__(self, name):
    AtomicDEVS.__init__(self)
    self.name = name
    self.p_out = self.addOutPort("p_out")
    self.state = GeneratorState()
    
    
  def intTransition( self ):
    if(self.state.seqState == Generator.generating):
      self.state.seqState = Generator.generating
    return self.state

  
  def outputFnc( self ):
    evt = None
    if(self.state.seqState == Generator.generating):
      evt = Job(0.3)
      self.poke(self.p_out, evt)


  
  def timeAdvance( self ):
    if(self.state.seqState == Generator.generating):
      timespan = 1
    return timespan


  def __str__(self):
    strRep = ''
    strRep = strRep + "\nstate: " + str(self.state)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>state</name>"
    strRep = strRep + "\n\t<type>GeneratorState</type>"
    if(self.state!=None):
      strRep = strRep + "\n\t<value>"+self.state.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class ProcessorState:
  def __init__(self):
    self.seqState = Processor.idle

  def __str__(self):
    strRep = ''
    strRep = strRep + "\nseqState: " + str(self.seqState)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"P\">"
    strRep = strRep + "\n\t<name>seqState</name>"
    strRep = strRep + "\n\t<type>Processor.SeqStates</type>"
    strRep = strRep + "\n\t<value>"+str(self.seqState)+"</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class Processor( AtomicDEVS ):
  idle = 'idle'
  state_2 = 'state_2'
  def __init__(self, name):
    AtomicDEVS.__init__(self)
    self.name = name
    self.p_in = self.addInPort("p_in")
    self.p_out = self.addOutPort("p_out")
    self.state = ProcessorState()
    
    
  def extTransition( self ):
    if(self.state.seqState == Processor.idle):
      self.state.seqState = Processor.state_2
      self.event = self.peek(self.p_in)
    return self.state

  
  def intTransition( self ):
    if(self.state.seqState == Processor.state_2):
      self.state.seqState = Processor.idle
    return self.state

  
  def outputFnc( self ):
    evt = None
    if(self.state.seqState == Processor.state_2):
      self.poke(self.p_out, self.event)


  
  def timeAdvance( self ):
    if(self.state.seqState == Processor.idle):
      timespan = INFINITY

    if(self.state.seqState == Processor.state_2):
      timespan = self.event.jobSize
    return timespan


  def __str__(self):
    strRep = ''
    strRep = strRep + "\nstate: " + str(self.state)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>state</name>"
    strRep = strRep + "\n\t<type>ProcessorState</type>"
    if(self.state!=None):
      strRep = strRep + "\n\t<value>"+self.state.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class Root( CoupledDEVS ):
  def __init__(self, name):
    CoupledDEVS.__init__(self)
    self.name = name
    self.p_out = self.addOutPort("p_out")
    self.ins_0 = Generator(name='ins_0')
    self.addSubModel( self.ins_0 )
    self.ins_1 = Processor(name='ins_1')
    self.addSubModel( self.ins_1 )
    self.ins_2 = Processor(name='ins_2')
    self.addSubModel( self.ins_2 )
    self.ins_3 = Processor(name='ins_3')
    self.addSubModel( self.ins_3 )
    self.ins_4 = Processor(name='ins_4')
    self.addSubModel( self.ins_4 )
    self.ins_5 = Processor(name='ins_5')
    self.addSubModel( self.ins_5 )
    self.ins_6 = Processor(name='ins_6')
    self.addSubModel( self.ins_6 )
    self.ins_7 = Processor(name='ins_7')
    self.addSubModel( self.ins_7 )
    self.ins_8 = Processor(name='ins_8')
    self.addSubModel( self.ins_8 )
    self.ins_9 = Processor(name='ins_9')
    self.addSubModel( self.ins_9 )
    self.ins_10 = Processor(name='ins_10')
    self.addSubModel( self.ins_10 )
    self.ins_11 = Processor(name='ins_11')
    self.addSubModel( self.ins_11 )
    self.ins_12 = Processor(name='ins_12')
    self.addSubModel( self.ins_12 )
    self.ins_13 = Processor(name='ins_13')
    self.addSubModel( self.ins_13 )
    self.ins_14 = Processor(name='ins_14')
    self.addSubModel( self.ins_14 )
    self.ins_15 = Processor(name='ins_15')
    self.addSubModel( self.ins_15 )
    self.ins_16 = Processor(name='ins_16')
    self.addSubModel( self.ins_16 )
    self.ins_17 = Processor(name='ins_17')
    self.addSubModel( self.ins_17 )
    self.ins_18 = Processor(name='ins_18')
    self.addSubModel( self.ins_18 )
    self.ins_19 = Processor(name='ins_19')
    self.addSubModel( self.ins_19 )
    self.ins_20 = Processor(name='ins_20')
    self.addSubModel( self.ins_20 )
    self.ins_21 = Processor(name='ins_21')
    self.addSubModel( self.ins_21 )
    self.ins_22 = Processor(name='ins_22')
    self.addSubModel( self.ins_22 )
    self.ins_23 = Processor(name='ins_23')
    self.addSubModel( self.ins_23 )
    self.ins_24 = Processor(name='ins_24')
    self.addSubModel( self.ins_24 )
    self.ins_25 = Processor(name='ins_25')
    self.addSubModel( self.ins_25 )
    self.ins_26 = Processor(name='ins_26')
    self.addSubModel( self.ins_26 )
    self.ins_27 = Processor(name='ins_27')
    self.addSubModel( self.ins_27 )
    self.ins_28 = Processor(name='ins_28')
    self.addSubModel( self.ins_28 )
    self.ins_29 = Processor(name='ins_29')
    self.addSubModel( self.ins_29 )
    self.ins_30 = Processor(name='ins_30')
    self.addSubModel( self.ins_30 )
    self.ins_31 = Processor(name='ins_31')
    self.addSubModel( self.ins_31 )
    self.ins_32 = Processor(name='ins_32')
    self.addSubModel( self.ins_32 )
    self.ins_33 = Processor(name='ins_33')
    self.addSubModel( self.ins_33 )
    self.ins_34 = Processor(name='ins_34')
    self.addSubModel( self.ins_34 )
    self.ins_35 = Processor(name='ins_35')
    self.addSubModel( self.ins_35 )
    self.ins_36 = Processor(name='ins_36')
    self.addSubModel( self.ins_36 )
    self.ins_37 = Processor(name='ins_37')
    self.addSubModel( self.ins_37 )
    self.ins_38 = Processor(name='ins_38')
    self.addSubModel( self.ins_38 )
    self.ins_39 = Processor(name='ins_39')
    self.addSubModel( self.ins_39 )
    self.ins_40 = Processor(name='ins_40')
    self.addSubModel( self.ins_40 )
    self.ins_41 = Processor(name='ins_41')
    self.addSubModel( self.ins_41 )
    self.ins_42 = Processor(name='ins_42')
    self.addSubModel( self.ins_42 )
    self.ins_43 = Processor(name='ins_43')
    self.addSubModel( self.ins_43 )
    self.ins_44 = Processor(name='ins_44')
    self.addSubModel( self.ins_44 )
    self.ins_45 = Processor(name='ins_45')
    self.addSubModel( self.ins_45 )
    self.ins_46 = Processor(name='ins_46')
    self.addSubModel( self.ins_46 )
    self.ins_47 = Processor(name='ins_47')
    self.addSubModel( self.ins_47 )
    self.ins_48 = Processor(name='ins_48')
    self.addSubModel( self.ins_48 )
    self.ins_49 = Processor(name='ins_49')
    self.addSubModel( self.ins_49 )
    self.ins_50 = Processor(name='ins_50')
    self.addSubModel( self.ins_50 )  
    self.connectPorts( self.ins_0.p_out, self.ins_1.p_in)  
    self.connectPorts( self.ins_1.p_out, self.ins_2.p_in)  
    self.connectPorts( self.ins_2.p_out, self.ins_3.p_in)  
    self.connectPorts( self.ins_3.p_out, self.ins_4.p_in)  
    self.connectPorts( self.ins_4.p_out, self.ins_5.p_in)  
    self.connectPorts( self.ins_5.p_out, self.ins_6.p_in)  
    self.connectPorts( self.ins_6.p_out, self.ins_7.p_in)  
    self.connectPorts( self.ins_7.p_out, self.ins_8.p_in)  
    self.connectPorts( self.ins_8.p_out, self.ins_9.p_in)  
    self.connectPorts( self.ins_9.p_out, self.ins_10.p_in)  
    self.connectPorts( self.ins_10.p_out, self.ins_11.p_in)  
    self.connectPorts( self.ins_11.p_out, self.ins_12.p_in)  
    self.connectPorts( self.ins_12.p_out, self.ins_13.p_in)  
    self.connectPorts( self.ins_13.p_out, self.ins_14.p_in)  
    self.connectPorts( self.ins_14.p_out, self.ins_15.p_in)  
    self.connectPorts( self.ins_15.p_out, self.ins_16.p_in)  
    self.connectPorts( self.ins_16.p_out, self.ins_17.p_in)  
    self.connectPorts( self.ins_17.p_out, self.ins_18.p_in)  
    self.connectPorts( self.ins_18.p_out, self.ins_19.p_in)  
    self.connectPorts( self.ins_19.p_out, self.ins_20.p_in)  
    self.connectPorts( self.ins_20.p_out, self.ins_21.p_in)  
    self.connectPorts( self.ins_21.p_out, self.ins_22.p_in)  
    self.connectPorts( self.ins_22.p_out, self.ins_23.p_in)  
    self.connectPorts( self.ins_23.p_out, self.ins_24.p_in)  
    self.connectPorts( self.ins_24.p_out, self.ins_25.p_in)  
    self.connectPorts( self.ins_25.p_out, self.ins_26.p_in)  
    self.connectPorts( self.ins_26.p_out, self.ins_27.p_in)  
    self.connectPorts( self.ins_27.p_out, self.ins_28.p_in)  
    self.connectPorts( self.ins_28.p_out, self.ins_29.p_in)  
    self.connectPorts( self.ins_29.p_out, self.ins_30.p_in)  
    self.connectPorts( self.ins_30.p_out, self.ins_31.p_in)  
    self.connectPorts( self.ins_31.p_out, self.ins_32.p_in)  
    self.connectPorts( self.ins_32.p_out, self.ins_33.p_in)  
    self.connectPorts( self.ins_33.p_out, self.ins_34.p_in)  
    self.connectPorts( self.ins_34.p_out, self.ins_35.p_in)  
    self.connectPorts( self.ins_35.p_out, self.ins_36.p_in)  
    self.connectPorts( self.ins_36.p_out, self.ins_37.p_in)  
    self.connectPorts( self.ins_37.p_out, self.ins_38.p_in)  
    self.connectPorts( self.ins_38.p_out, self.ins_39.p_in)  
    self.connectPorts( self.ins_39.p_out, self.ins_40.p_in)  
    self.connectPorts( self.ins_40.p_out, self.ins_41.p_in)  
    self.connectPorts( self.ins_41.p_out, self.ins_42.p_in)  
    self.connectPorts( self.ins_42.p_out, self.ins_43.p_in)  
    self.connectPorts( self.ins_43.p_out, self.ins_44.p_in)  
    self.connectPorts( self.ins_44.p_out, self.ins_45.p_in)  
    self.connectPorts( self.ins_45.p_out, self.ins_46.p_in)  
    self.connectPorts( self.ins_46.p_out, self.ins_47.p_in)  
    self.connectPorts( self.ins_47.p_out, self.ins_48.p_in)  
    self.connectPorts( self.ins_48.p_out, self.ins_49.p_in)  
    self.connectPorts( self.ins_49.p_out, self.ins_50.p_in)  
    self.connectPorts( self.ins_50.p_out, self.p_out)

  def __str__(self):
    strRep = ''
    strRep = strRep + "\nins_0: " + str(self.ins_0)
    strRep = strRep + "\nins_1: " + str(self.ins_1)
    strRep = strRep + "\nins_2: " + str(self.ins_2)
    strRep = strRep + "\nins_3: " + str(self.ins_3)
    strRep = strRep + "\nins_4: " + str(self.ins_4)
    strRep = strRep + "\nins_5: " + str(self.ins_5)
    strRep = strRep + "\nins_6: " + str(self.ins_6)
    strRep = strRep + "\nins_7: " + str(self.ins_7)
    strRep = strRep + "\nins_8: " + str(self.ins_8)
    strRep = strRep + "\nins_9: " + str(self.ins_9)
    strRep = strRep + "\nins_10: " + str(self.ins_10)
    strRep = strRep + "\nins_11: " + str(self.ins_11)
    strRep = strRep + "\nins_12: " + str(self.ins_12)
    strRep = strRep + "\nins_13: " + str(self.ins_13)
    strRep = strRep + "\nins_14: " + str(self.ins_14)
    strRep = strRep + "\nins_15: " + str(self.ins_15)
    strRep = strRep + "\nins_16: " + str(self.ins_16)
    strRep = strRep + "\nins_17: " + str(self.ins_17)
    strRep = strRep + "\nins_18: " + str(self.ins_18)
    strRep = strRep + "\nins_19: " + str(self.ins_19)
    strRep = strRep + "\nins_20: " + str(self.ins_20)
    strRep = strRep + "\nins_21: " + str(self.ins_21)
    strRep = strRep + "\nins_22: " + str(self.ins_22)
    strRep = strRep + "\nins_23: " + str(self.ins_23)
    strRep = strRep + "\nins_24: " + str(self.ins_24)
    strRep = strRep + "\nins_25: " + str(self.ins_25)
    strRep = strRep + "\nins_26: " + str(self.ins_26)
    strRep = strRep + "\nins_27: " + str(self.ins_27)
    strRep = strRep + "\nins_28: " + str(self.ins_28)
    strRep = strRep + "\nins_29: " + str(self.ins_29)
    strRep = strRep + "\nins_30: " + str(self.ins_30)
    strRep = strRep + "\nins_31: " + str(self.ins_31)
    strRep = strRep + "\nins_32: " + str(self.ins_32)
    strRep = strRep + "\nins_33: " + str(self.ins_33)
    strRep = strRep + "\nins_34: " + str(self.ins_34)
    strRep = strRep + "\nins_35: " + str(self.ins_35)
    strRep = strRep + "\nins_36: " + str(self.ins_36)
    strRep = strRep + "\nins_37: " + str(self.ins_37)
    strRep = strRep + "\nins_38: " + str(self.ins_38)
    strRep = strRep + "\nins_39: " + str(self.ins_39)
    strRep = strRep + "\nins_40: " + str(self.ins_40)
    strRep = strRep + "\nins_41: " + str(self.ins_41)
    strRep = strRep + "\nins_42: " + str(self.ins_42)
    strRep = strRep + "\nins_43: " + str(self.ins_43)
    strRep = strRep + "\nins_44: " + str(self.ins_44)
    strRep = strRep + "\nins_45: " + str(self.ins_45)
    strRep = strRep + "\nins_46: " + str(self.ins_46)
    strRep = strRep + "\nins_47: " + str(self.ins_47)
    strRep = strRep + "\nins_48: " + str(self.ins_48)
    strRep = strRep + "\nins_49: " + str(self.ins_49)
    strRep = strRep + "\nins_50: " + str(self.ins_50)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_0</name>"
    strRep = strRep + "\n\t<type>Generator</type>"
    if(self.ins_0!=None):
      strRep = strRep + "\n\t<value>"+self.ins_0.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_1</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_1!=None):
      strRep = strRep + "\n\t<value>"+self.ins_1.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_2</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_2!=None):
      strRep = strRep + "\n\t<value>"+self.ins_2.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_3</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_3!=None):
      strRep = strRep + "\n\t<value>"+self.ins_3.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_4</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_4!=None):
      strRep = strRep + "\n\t<value>"+self.ins_4.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_5</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_5!=None):
      strRep = strRep + "\n\t<value>"+self.ins_5.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_6</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_6!=None):
      strRep = strRep + "\n\t<value>"+self.ins_6.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_7</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_7!=None):
      strRep = strRep + "\n\t<value>"+self.ins_7.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_8</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_8!=None):
      strRep = strRep + "\n\t<value>"+self.ins_8.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_9</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_9!=None):
      strRep = strRep + "\n\t<value>"+self.ins_9.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_10</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_10!=None):
      strRep = strRep + "\n\t<value>"+self.ins_10.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_11</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_11!=None):
      strRep = strRep + "\n\t<value>"+self.ins_11.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_12</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_12!=None):
      strRep = strRep + "\n\t<value>"+self.ins_12.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_13</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_13!=None):
      strRep = strRep + "\n\t<value>"+self.ins_13.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_14</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_14!=None):
      strRep = strRep + "\n\t<value>"+self.ins_14.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_15</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_15!=None):
      strRep = strRep + "\n\t<value>"+self.ins_15.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_16</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_16!=None):
      strRep = strRep + "\n\t<value>"+self.ins_16.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_17</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_17!=None):
      strRep = strRep + "\n\t<value>"+self.ins_17.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_18</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_18!=None):
      strRep = strRep + "\n\t<value>"+self.ins_18.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_19</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_19!=None):
      strRep = strRep + "\n\t<value>"+self.ins_19.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_20</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_20!=None):
      strRep = strRep + "\n\t<value>"+self.ins_20.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_21</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_21!=None):
      strRep = strRep + "\n\t<value>"+self.ins_21.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_22</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_22!=None):
      strRep = strRep + "\n\t<value>"+self.ins_22.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_23</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_23!=None):
      strRep = strRep + "\n\t<value>"+self.ins_23.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_24</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_24!=None):
      strRep = strRep + "\n\t<value>"+self.ins_24.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_25</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_25!=None):
      strRep = strRep + "\n\t<value>"+self.ins_25.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_26</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_26!=None):
      strRep = strRep + "\n\t<value>"+self.ins_26.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_27</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_27!=None):
      strRep = strRep + "\n\t<value>"+self.ins_27.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_28</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_28!=None):
      strRep = strRep + "\n\t<value>"+self.ins_28.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_29</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_29!=None):
      strRep = strRep + "\n\t<value>"+self.ins_29.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_30</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_30!=None):
      strRep = strRep + "\n\t<value>"+self.ins_30.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_31</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_31!=None):
      strRep = strRep + "\n\t<value>"+self.ins_31.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_32</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_32!=None):
      strRep = strRep + "\n\t<value>"+self.ins_32.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_33</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_33!=None):
      strRep = strRep + "\n\t<value>"+self.ins_33.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_34</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_34!=None):
      strRep = strRep + "\n\t<value>"+self.ins_34.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_35</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_35!=None):
      strRep = strRep + "\n\t<value>"+self.ins_35.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_36</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_36!=None):
      strRep = strRep + "\n\t<value>"+self.ins_36.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_37</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_37!=None):
      strRep = strRep + "\n\t<value>"+self.ins_37.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_38</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_38!=None):
      strRep = strRep + "\n\t<value>"+self.ins_38.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_39</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_39!=None):
      strRep = strRep + "\n\t<value>"+self.ins_39.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_40</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_40!=None):
      strRep = strRep + "\n\t<value>"+self.ins_40.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_41</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_41!=None):
      strRep = strRep + "\n\t<value>"+self.ins_41.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_42</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_42!=None):
      strRep = strRep + "\n\t<value>"+self.ins_42.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_43</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_43!=None):
      strRep = strRep + "\n\t<value>"+self.ins_43.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_44</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_44!=None):
      strRep = strRep + "\n\t<value>"+self.ins_44.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_45</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_45!=None):
      strRep = strRep + "\n\t<value>"+self.ins_45.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_46</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_46!=None):
      strRep = strRep + "\n\t<value>"+self.ins_46.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_47</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_47!=None):
      strRep = strRep + "\n\t<value>"+self.ins_47.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_48</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_48!=None):
      strRep = strRep + "\n\t<value>"+self.ins_48.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_49</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_49!=None):
      strRep = strRep + "\n\t<value>"+self.ins_49.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_50</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_50!=None):
      strRep = strRep + "\n\t<value>"+self.ins_50.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class Job( object ):
  def __init__(self, jobSize):
    self.jobSize = jobSize

  def __str__(self):
    strRep = ''
    return strRep

  def toXML(self):
    strRep = ''
    return strRep

class Experiment( object ):
  def __init__(self):
    self.rootModel = Root(name = "Experiment")
    self.sim = Simulator(model = self.rootModel)
    self.verbose = False
    self.trace = False
    self.end_time = 1000
  def simulate(self):
    self.sim.simulate(termination_condition=terminate_at(self.end_time), verbose=self.verbose)




def terminate_at(t):
 def f(model, clock):
  if clock >= t:
    return True
  else:
    return False
 return f
if __name__ == '__main__':
  experiment = Experiment()
  experiment.simulate()