
import sys
import os.path
sys.path.append(os.path.expanduser('../../pydevs-old/pydevs/'))
from DEVS import *
from infinity import *
from random import randint
from simulator import *

class GeneratorState:
  def __init__(self):
    self.seqState = Generator.generating

  def __str__(self):
    strRep = ''
    strRep = strRep + "\nseqState: " + str(self.seqState)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"P\">"
    strRep = strRep + "\n\t<name>seqState</name>"
    strRep = strRep + "\n\t<type>Generator.SeqStates</type>"
    strRep = strRep + "\n\t<value>"+str(self.seqState)+"</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class Generator( AtomicDEVS ):
  generating = 'generating'
  def __init__(self, name):
    AtomicDEVS.__init__(self)
    self.name = name
    self.p_out = self.addOutPort("p_out")
    self.state = GeneratorState()
    
    
  def intTransition( self ):
    if(self.state.seqState == Generator.generating):
      self.state.seqState = Generator.generating
    return self.state

  
  def outputFnc( self ):
    evt = None
    if(self.state.seqState == Generator.generating):
      evt = Job(0.3)
      self.poke(self.p_out, evt)


  
  def timeAdvance( self ):
    if(self.state.seqState == Generator.generating):
      timespan = 1
    return timespan


  def __str__(self):
    strRep = ''
    strRep = strRep + "\nstate: " + str(self.state)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>state</name>"
    strRep = strRep + "\n\t<type>GeneratorState</type>"
    if(self.state!=None):
      strRep = strRep + "\n\t<value>"+self.state.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class ProcessorState:
  def __init__(self):
    self.seqState = Processor.idle

  def __str__(self):
    strRep = ''
    strRep = strRep + "\nseqState: " + str(self.seqState)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"P\">"
    strRep = strRep + "\n\t<name>seqState</name>"
    strRep = strRep + "\n\t<type>Processor.SeqStates</type>"
    strRep = strRep + "\n\t<value>"+str(self.seqState)+"</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class Processor( AtomicDEVS ):
  idle = 'idle'
  state_2 = 'state_2'
  def __init__(self, name):
    AtomicDEVS.__init__(self)
    self.name = name
    self.p_in = self.addInPort("p_in")
    self.p_out = self.addOutPort("p_out")
    self.state = ProcessorState()
    
    
  def extTransition( self ):
    if(self.state.seqState == Processor.idle):
      self.state.seqState = Processor.state_2
      self.event = self.peek(self.p_in)
    return self.state

  
  def intTransition( self ):
    if(self.state.seqState == Processor.state_2):
      self.state.seqState = Processor.idle
    return self.state

  
  def outputFnc( self ):
    evt = None
    if(self.state.seqState == Processor.state_2):
      self.poke(self.p_out, self.event)


  
  def timeAdvance( self ):
    if(self.state.seqState == Processor.idle):
      timespan = INFINITY

    if(self.state.seqState == Processor.state_2):
      timespan = self.event.jobSize
    return timespan


  def __str__(self):
    strRep = ''
    strRep = strRep + "\nstate: " + str(self.state)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>state</name>"
    strRep = strRep + "\n\t<type>ProcessorState</type>"
    if(self.state!=None):
      strRep = strRep + "\n\t<value>"+self.state.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class Root( CoupledDEVS ):
  def __init__(self, name):
    CoupledDEVS.__init__(self)
    self.name = name
    self.p_out = self.addOutPort("p_out")
    self.ins_2 = Processor(name='ins_2')
    self.addSubModel( self.ins_2 )
    self.ins_6 = CoupledProcessor(name='ins_6')
    self.addSubModel( self.ins_6 )
    self.ins_7 = Generator(name='ins_7')
    self.addSubModel( self.ins_7 )  
    self.connectPorts( self.ins_2.p_out, self.p_out)  
    self.connectPorts( self.ins_6.p_out, self.ins_2.p_in)  
    self.connectPorts( self.ins_7.p_out, self.ins_6.p_in)

  def __str__(self):
    strRep = ''
    strRep = strRep + "\nins_2: " + str(self.ins_2)
    strRep = strRep + "\nins_6: " + str(self.ins_6)
    strRep = strRep + "\nins_7: " + str(self.ins_7)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_2</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_2!=None):
      strRep = strRep + "\n\t<value>"+self.ins_2.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_6</name>"
    strRep = strRep + "\n\t<type>CoupledProcessor</type>"
    if(self.ins_6!=None):
      strRep = strRep + "\n\t<value>"+self.ins_6.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_7</name>"
    strRep = strRep + "\n\t<type>Generator</type>"
    if(self.ins_7!=None):
      strRep = strRep + "\n\t<value>"+self.ins_7.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class CoupledProcessor( CoupledDEVS ):
  def __init__(self, name):
    CoupledDEVS.__init__(self)
    self.name = name
    self.p_in = self.addInPort("p_in")
    self.p_out = self.addOutPort("p_out")
    self.ins_3 = Processor(name='ins_3')
    self.addSubModel( self.ins_3 )
    self.ins_4 = Processor(name='ins_4')
    self.addSubModel( self.ins_4 )  
    self.connectPorts( self.ins_3.p_out, self.ins_4.p_in)  
    self.connectPorts( self.ins_4.p_out, self.p_out)  
    self.connectPorts( self.p_in, self.ins_3.p_in)

  def __str__(self):
    strRep = ''
    strRep = strRep + "\nins_3: " + str(self.ins_3)
    strRep = strRep + "\nins_4: " + str(self.ins_4)
    return strRep

  def toXML(self):
    strRep = ''
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_3</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_3!=None):
      strRep = strRep + "\n\t<value>"+self.ins_3.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    strRep = strRep + "\n<attribute category=\"C\">"
    strRep = strRep + "\n\t<name>ins_4</name>"
    strRep = strRep + "\n\t<type>Processor</type>"
    if(self.ins_4!=None):
      strRep = strRep + "\n\t<value>"+self.ins_4.toXML()+"</value>"
    else:
      strRep = strRep + "\n\t<value>None</value>"
    strRep = strRep + "\n</attribute>"
    return strRep

class Job( object ):
  def __init__(self, jobSize):
    self.jobSize = jobSize

  def __str__(self):
    strRep = ''
    return strRep

  def toXML(self):
    strRep = ''
    return strRep

class Experiment( object ):
  def __init__(self):
    self.rootModel = Root(name = "Experiment")
    self.sim = Simulator(model = self.rootModel)
    self.verbose = False
    self.end_time = 10000
  def simulate(self):
    self.sim.simulate(termination_condition=terminate_at(self.end_time), verbose=self.verbose)




def terminate_at(t):
 def f(model, clock):
  if clock >= t:
    return True
  else:
    return False
 return f
if __name__ == '__main__':
  experiment = Experiment()
  experiment.simulate()
