#Simple example showing symboltables and their xml
#representation.
#Author: Marc Provost
#Dec 18, 2003

from GraphRepr.Node import Node
from GraphRepr.Graph import Graph

#importing the symboltable type:
from GraphRepr.Types.MappingType.SymbolTable import SymbolTable

#importing the simpletypes:
from GraphRepr.Types.SimpleType.Int import Int
from GraphRepr.Types.SimpleType.String import String
from GraphRepr.Types.SimpleType.Bool import Bool
from GraphRepr.Types.SimpleType.Float import Float


#AGLLoader,AGLGenerator are repeated twice because
#the directory in which they are located has the same
#name as them.
from GraphManager.AGLLoader.AGLLoader import AGLLoader
from GraphManager.AGLGenerator.AGLGenerator import AGLGenerator

import os,sys

#A catalog file is used by the xmlparser.  It basically
#map a given http reference for a dtd to a local file.
#e.g. In our case :

# SYSTEM http://agl.dtd C:\mcgill\thesis\pyGK\XMLTools\dtd\agl.dtd

#Indeed, usually, the dtd are specified as hyper-references,
#but the parser is opening a local file (will be changed
#at some point, as suggested by Hans).

#Now, on my system the catalog is located at:

# C:\mcgill\thesis\pyGK\XMLTools\catalog\catalog.txt


#But this path dependent on the system where pyGK is installed!  Thus,
#we must locate the path independently of the directory
#structure.  We know that this part of the path is always
#the same:

#  ..\pyGK\XMLTools\catalog\catalog.txt

#But how can we locate the first part of the path ?
#Fortunately, there is a dirty python trick (took me hours to find it) to find
#the path of the python script that is executed.
#Since we know that the python script is located in:

#  ..\pyGK\Examples\SymbolTable

#We can replace this part of the path:

# Examples\SymbolTable by

# XMLTools\catalog\catalog.txt

catalog=""

pathname = os.path.dirname(sys.argv[0])
scriptDir = os.path.abspath(pathname)

#now rootDir = ..\pyGK\Examples\SymbolTable\

#Getting the os specific separator (/ or \)
sep=os.sep

from string import replace


#removing the end of the path, replacing it by XMLTools\catalog\catalog.txt
rootDir = replace(scriptDir, "Examples"+sep+"SymbolTable", "")
catalog=rootDir+"XMLTools"+sep+"catalog"+sep+"catalog.txt"

print ""
print ""
print "Symbol Table Example!"
print "*********************"
print ""
print ""

print "The catalog is located at: "
print catalog
print ""


graph=Graph("AST")

#Say we parse the following java method and we create an ast for it:

#public print(boolean itemA, string itemB)

#creating a symbol table for the actual method
method=SymbolTable("method1")

#adding a type attribute:
#(note that python dictionnary syntax was overloaded)
method["type"]=String(value="method")

#adding a name attribute:
method["name"]=String(value="print")

#adding a visibility attribute
method["visibility"]=String(value="public")

#adding a line number attribute
method["lineno"]=Int(102)

#adding the symboltable to the graph
graph.addElement(method)

#Now, we use two symboltable nodes to represent
#the arguments.

#first argument:

arg1=SymbolTable("arg1")

arg1["type"]=String(value="bool")
arg1["name"]=String(value="itemA")

#second argument:

arg2=SymbolTable("arg2")

arg2["type"]=String(value="string")
arg2["name"]=String(value="itemB")

#add the two elements to the graph:

graph.addElement(arg1)
graph.addElement(arg2)

#connecting method to arg1, arg2, using ORDERING.
#arg1 is considered to be the first connection:

#the two last arguments of connect are:
#fromOrdering, toOrdering

graph.connect("method1","arg1",0,0)
graph.connect("method1","arg2",1,0) #note 1>0, thus arg2 is second


#Ok, generating code for graph!

#In order to generate code, of course the path where the file will
#be located must be specified.  Since pyGK supports higraphs,
#AGLGenerator takes a dictionnary as an attribute specifying a file
#for each graph.  In our case, we only have one graph, so the mapping
#is very simple:

mapping={}
#mapping the graph with ID AST to the file AST.agl
#in the directory where this script is located:
mapping["AST"]=scriptDir+sep+"AST.agl"

print ""
print "Generating code for the sample graph at:"
print mapping["AST"]

#creating a generator object:
gen=AGLGenerator(graph, mapping)
gen.genCode()

print ""
print "loading the generated file, checking if its equal to the original"
print ""

#the loader takes the same mapping as an argument, along with the
#catalog path

loader=AGLLoader(mapping,catalog)

#loading the graph with ID AST
loadedGraph=loader.load("AST")

if loadedGraph.equals(graph):
    print "OK!", "The graphs are equal!"
    print ""

print "Iterating over the outgoing connections of method"
print "itemA should appear before itemB!"
print ""

#getting the method:

loadedMethod=loadedGraph.getElement("method1")

#getting the outgoing connections..

outC=loadedMethod.outC()

#creating an iterator over the connections:
outCIter=outC.iterate()

argno=1
while not outCIter.isDone():
    curArg=outCIter.currentElement()
    outCIter.next()

    print loadedMethod["name"],"argument", argno, "is", curArg["name"]
    argno+=1

print ""
print "Example is finished, please email me with questions"


















