from GraphRepr.GraphUtils.Visitor import Visitor
from GraphRepr.GraphUtils.Search import DFS

from string import replace

# first pass for GraphElements.
# Call the Secondpass at the end for connections

class AGLFirstPass(Visitor):

    def __init__(self, fileMapping, indentSize="    "):

        self.__indentLevel={}
        for key in fileMapping.keys():
            self.__indentLevel[key]=0
        self.__indentSize=indentSize
        self.__outputf=fileMapping

    def __pushIndent(self, graphId):
        self.__indentLevel[graphId] += 1

    def __popIndent(self, graphId):
        self.__indentLevel[graphId] -= 1

    def __indent(self, graphId):
        for i in range(self.__indentLevel[graphId]):
            self.__outputf[graphId].write(self.__indentSize)

    def __newline(self, graphId, qty=1):

        for i in range(qty):
            self.__outputf[graphId].write('\n')

    def __writeTag(self, graphId, strTag, attrList=None, toClose=0, tagData=""):

        self.__indent(graphId)
        self.__outputf[graphId].write('<' + strTag)

        if attrList != None:
            for attr,value in attrList:
                self.__outputf[graphId].write(" " + str(attr) + '=' + '\"' + str(value) + '\"')

        if toClose:
            self.__outputf[graphId].write('/>')
        else:
            self.__outputf[graphId].write('>')
            self.__outputf[graphId].write(tagData)
            self.__pushIndent(graphId)
        self.__newline(graphId)

    def __writeCloseTag(self, graphId, strTag):
        self.__popIndent(graphId)
        self.__indent(graphId)
        self.__outputf[graphId].write('</' + strTag + '>')
        self.__newline(graphId)

    def __writeData(self, graphId, strData):
        self.__outputf[graphId].write(strData)

    def enterNode(self, elem):
        self.__writeTag(elem.getParent().getGlobalId(),
                        "node", [("id", elem.getGlobalId(elem.getDepth())),
                                 ("type",elem.getType().getGlobalId(elem.getType().getDepth()))
                                ],1)
    def exitNode(self, elem):
        pass

    def enterBool(self, elem):
        self.__writeTag(elem.getParent().getGlobalId(),
                        "bool", [("id", elem.getGlobalId(elem.getDepth())),
                                 ("type",elem.getType().getGlobalId(elem.getType().getDepth())),
                                 ("value",elem.getValue())
                                ],1)
    def exitBool(self, elem):
        pass

    def enterFloat(self, elem):
        self.__writeTag(elem.getParent().getGlobalId(),
                        "float", [("id", elem.getGlobalId(elem.getDepth())),
                                  ("type",elem.getType().getGlobalId(elem.getType().getDepth())),
                                  ("value",elem.getValue())
                                 ],1)
    def exitFloat(self, elem):
        pass

    def enterInt(self, elem):
        self.__writeTag(elem.getParent().getGlobalId(),
                        "int", [("id", elem.getGlobalId(elem.getDepth())),
                                ("type",elem.getType().getGlobalId(elem.getType().getDepth())),
                                ("value",elem.getValue())
                               ],1)
    def exitInt(self, elem):
        pass

    def enterString(self, elem):
        value=replace(elem.getValue(), '\n', '\\n')
        value=replace(value, '\t', '\\t')
        self.__writeTag(elem.getParent().getGlobalId(),
                        "string", [("id", elem.getGlobalId(elem.getDepth())),
                                   ("type",elem.getType().getGlobalId(elem.getType().getDepth())),
                                   ("value", value)
                                  ],1)
    def exitString(self, elem):
        pass

    def enterGraph(self, elem):
        self.__outputf[elem.getGlobalId()].write('<?xml version="1.0"?>')
        self.__outputf[elem.getGlobalId()].write('\n')
        self.__outputf[elem.getGlobalId()].write('<!DOCTYPE agl SYSTEM "http://agl.dtd">')
        self.__outputf[elem.getGlobalId()].write('\n')
        self.__outputf[elem.getGlobalId()].write('<!-- GraphElements -->')
        self.__outputf[elem.getGlobalId()].write('\n')
        self.__outputf[elem.getGlobalId()].write('<agl>')
        self.__outputf[elem.getGlobalId()].write('\n')
        self.__pushIndent(elem.getGlobalId())
        self.__writeTag(elem.getGlobalId(),
                        "graph", [("id", elem.getGlobalId(elem.getDepth())),
                                  ("type",elem.getType().getGlobalId(elem.getType().getDepth()))
                                 ])
        if elem.getParent()!=None:
            self.__writeTag(elem.getParent().getGlobalId(),
                           "graph", [("id", elem.getGlobalId(elem.getDepth())),
                                     ("type",elem.getType().getGlobalId(elem.getType().getDepth()))
                                    ],1)



    def exitGraph(self, elem):
        conns=elem.getLocalConn()

        for conn in conns:
            self.__writeTag(elem.getGlobalId(),
                            "edge", [("from", conn[0].getGlobalId(elem.getDepth()+1)),
                                     ("to", conn[1].getGlobalId(elem.getDepth()+1)),
                                     ("fromOrd",conn[2]),
                                     ("toOrd",conn[3])
                            ],1)


        self.__writeCloseTag(elem.getGlobalId(),"graph")
        self.__popIndent(elem.getGlobalId())
        self.__outputf[elem.getGlobalId()].write('</agl>')
        self.__outputf[elem.getGlobalId()].write('\n')
        self.__outputf[elem.getGlobalId()].close()

    def enterList(self, elem):
        self.__writeTag(elem.getParent().getGlobalId(),
                        "list", [("id", elem.getGlobalId(elem.getDepth())),
                                 ("type", elem.getType().getGlobalId(elem.getType().getDepth()))
                                ])
        for el in elem:
            el.enter(self)
            el.exit(self)


    def exitList(self, elem):
        self.__writeCloseTag(elem.getParent().getGlobalId(),"list")

    def enterSymbolTable(self, elem):
        self.__writeTag(elem.getParent().getGlobalId(),
                        "symb", [("id", elem.getGlobalId(elem.getDepth())),
                                 ("type", elem.getType().getGlobalId(elem.getType().getDepth()))
                                ])
        for key in elem.keys():
            self.__writeTag(elem.getParent().getGlobalId(),
                            "map", [("key", key)])
            elem[key].enter(self)
            elem[key].exit(self)
            self.__writeCloseTag(elem.getParent().getGlobalId(),"map")


    def exitSymbolTable(self, elem):
        self.__writeCloseTag(elem.getParent().getGlobalId(),"symb")

    def preAction(self):
        pass

    def postAction(self):
        pass
