class GraphManagerException(Exception):
    """Base class for exception in this module"""
    pass

class UnknownGraphException(GraphManagerException):

    def __init__(self, graphName):
        self.graphName=graphName

    def __str__(self):
        return "\nUnknown graph:" +\
                str(self.graphName) + "\n"

class SingletonException(GraphManagerException):

    def __str__(self):
        return "\nThis object is a singleton " +\
               "(only one instance is allowed) \n"

class GraphIdAlreadyExistsException(GraphManagerException):

    def __init__(self, graphId):
        self.graphId=graphId

    def __str__(self):
        return "\nGraph with id " + str(self.graphId) +\
                  " is already member of the system \n"

class InvalidPathException(GraphManagerException):

    def __init__(self, path):
        self.path=path

    def __str__(self):
        return "\nSpecified file is invalid or unreachable " + str(self.path) + "\n"

class PathAlreadyInUseException(GraphManagerException):

    def __init__(self, path):
        self.path=path

    def __str__(self):
        return "\nSpecified file is already in use by another graph " +\
               str(self.path) + "\n"