from GraphRepr.Types.MappingType.MappingType import MappingType
from GraphRepr.Node import Node
from GraphRepr.GraphUtils.GraphConnCollection import GraphConnCollection

import weakref

class SymbolTable(MappingType, Node):

    TYPE = MappingType.TYPE.getElement("SymbolTable")

    def __init__(self, ID="",parent=None):

        self.__dict={}

        self.__id=ID
        self.__typeId=self.TYPE.getLocalId()
        self.__type=self.TYPE

        self.__parent=None
        self.__root=None
        self.__globalId=str(ID)
        self.__depth=0

        if parent != None:
            parent.addElement(self)

    def addElement(self, key, elem):
        self.__dict[key]=elem

    def removeElement(self, key):
        del self.__dict[key]

    def hasElement(self, key):
        return self.__dict.has_key(key)

    def getElement(self, key):
        return self.__dict[key]

    def size(self):
        return len(self.__dict)

    def keys(self):
        return self.__dict.keys()

    def has_key(self,key):
        return self.__dict.has_key(key)

    def valueStr(self):
        return str(self.__dict)

    # OVERLOADED METHODS

    def __getitem__(self, key):
        return self.__dict[key]

    def __setitem__(self, key, item):
        self.__dict[key]=item

    def __delitem__(self, key):
        del self.__dict[key]

    def __contains__(self, key):
        return self.__dict.has_key(key)

    def __len__(self):
        return len(self.__dict)

    def __str__(self):
        return str(self.__id) + " :: " + str(self.__type.getId()) + " :: " +\
               "value == " + str(self.__dict)

    #Node implementation... (due to lack of protected types in python
    #we must copy // paste the implementation there..)

    #the following methods are meant to be protected
    def setRoot(self, root):
        if root==None:
            self.__root=root
        else:
            self.__root=weakref.ref(root)

    def setGlobalId(self, path):
        self.__globalId=path

    def setDepth(self, depth):
        self.__depth=depth

    def setParent(self, parent):
        if parent==None:
            self.__parent=parent
        else:
            self.__parent=weakref.ref(parent)
        for key in self.__dict.keys():
            self.__dict[key].setParent(parent)

    def createConnInfo(self):
        conns=self.getRoot().getConnInfo()
        conns[self.getGlobalId()]=[{}, {}]

    #the following methods are accessible members of the interface
    def getTypeLocalId(self):
        return self.__typeId

    def getType(self):
        return self.__type

    def getTypeGlobalId(self):
        return self.__type.getGlobalId()

    def getDepth(self):
        return self.__depth

    def getGlobalId(self, relativeToDepth=0):
        if not relativeToDepth:
            return self.__globalId
        ind=0
        i=0
        for j in range(len(self.__globalId)):
            if self.__globalId[j]==".":
                ind=j
                i+=1
            if i==relativeToDepth:
                break
        return self.__globalId[(ind+1):]

    def getRoot(self):
        if self.__root==None:
            return self.__root
        else:
            return self.__root()

    def getParent(self):
        if self.__parent==None:
            return self.__parent
        else:
            return self.__parent()

    def getLocalId(self):
        return self.__id

    def hasElementAnyDepth(self, globalId):
        conns=self.getRoot().getConnInfo()
        return conns.has_key(globalId)

    def outC(self):
        conns=self.getRoot().getConnInfo()
        globalId=self.getGlobalId()

        return GraphConnCollection(conns[globalId][1])

    def inC(self):
        conns=self.getRoot().getConnInfo()
        globalId=self.getGlobalId()

        return GraphConnCollection(conns[globalId][0])

    def disconnectAllRecurse(self):
        conns = self.getRoot.getConnInfo()
        globalId=self.getGlobalId()

        for fromKey in conns[globalId][0].keys():
            fromElem=conns[globalId][0][fromKey][0]
            fromElemId=fromElem.getGlobalId()
            del conns[fromElemId][1][globalId]
            if globalId==fromElemId:
                del conns[fromElemId][0][globalId]

        for toKey in conns[globalId][1].keys():
            toElem=conns[globalId][1][toKey][0]
            toElemId=toElem.getGlobalId()
            del conns[toElemId][0][globalId]
        del conns[globalId]


    def __str__(self):
        return str(self.__id) + " :: " + str(self.getType().getLocalId())

    # enter action perfomed by a visitor on this element
    def enter(self, visitor):
        visitor.enterSymbolTable(self)

    # exit action perfomed by a visitor on this element
    def exit(self, visitor):
        visitor.exitSymbolTable(self)

    def equals(self, elem):
        if not isinstance(elem,SymbolTable):
            return 0

        if self.getLocalId() != elem.getLocalId():
            return 0

        if not (self.getType()==self and elem.getType()==elem):
            if not self.getType().equals(elem.getType()):
                return 0

        for key in self.keys():
            if not elem.has_key(key):
                return 0
            elif not self[key].equals(elem[key]):
                return 0

        return 1

    def clone(self):
        symb = SymbolTable(self.getLocalId(),
                           parent=None)

        for key in self.keys():
            symb[key]=self[key].clone()

        return symb

    def __repr__(self):
        return str(self.__globalId)

    def __del__(self):
        #print "del node", self
        del self.__id
        del self.__type
        del self.__parent
        del self.__depth
        del self.__globalId
        del self.__root
        del self.__dict
