from GraphRepr.Types.SimpleType.SimpleType import SimpleType
from GraphRepr.Types.SimpleType.Error.SimpleTypeException\
                                 import TypeInvalidityException

from GraphRepr.Node import Node
from GraphRepr.GraphUtils.GraphConnCollection import GraphConnCollection

import weakref

class String(SimpleType,Node):

    TYPE=SimpleType.TYPE.getElement("String")

    def __init__(self, value, ID="",parent=None):
        if type(value)!=str:
            raise TypeInvalidityException(type(value), [type("string")])
        else:
            self.__value=str(value)

        self.__id=ID
        self.__type=self.TYPE
        self.__typeId=self.TYPE.getLocalId()

        self.__parent=None
        self.__root=None
        self.__globalId=str(ID)
        self.__depth=0

        if parent != None:
            parent.addElement(self)

    def setValue(self,value):
        if type(value)!=str:
            raise TypeInvalidityException(type(value), [type("string")])
        else:
            self.__value=str(value)

    def getValue(self):
        return self.__value

    def valueStr(self):
        return str(self.__value)

    def __str__(self):
        return str(self.__value)

    def __repr__(self):
        return self.valueStr()


    #Node implementation... (due to lack of protected types in python
    #we must copy // paste the implementation there..)

    #the following methods are meant to be protected
    def setRoot(self, root):
        if root==None:
            self.__root=root
        else:
            self.__root=weakref.ref(root)

    def setGlobalId(self, path):
        self.__globalId=path

    def setDepth(self, depth):
        self.__depth=depth

    def setParent(self, parent):
        if parent==None:
            self.__parent=parent
        else:
            self.__parent=weakref.ref(parent)

    def createConnInfo(self):
        conns=self.getRoot().getConnInfo()
        conns[self.getGlobalId()]=[{}, {}]

    #the following methods are accessible members of the interface
    def getTypeLocalId(self):
        return self.__typeId

    def getType(self):
        return self.__type

    def getTypeGlobalId(self):
        return self.__type.getGlobalId()

    def getDepth(self):
        return self.__depth

    def getGlobalId(self, relativeToDepth=0):
        if not relativeToDepth:
            return self.__globalId
        ind=0
        i=0
        for j in range(len(self.__globalId)):
            if self.__globalId[j]==".":
                ind=j
                i+=1
            if i==relativeToDepth:
                break
        return self.__globalId[(ind+1):]

    def getRoot(self):
        if self.__root==None:
            return self.__root
        else:
            return self.__root()

    def getParent(self):
        if self.__parent==None:
            return self.__parent
        else:
            return self.__parent()

    def getLocalId(self):
        return self.__id

    def hasElementAnyDepth(self, globalId):
        conns=self.getRoot().getConnInfo()
        return conns.has_key(globalId)

    def outC(self):
        conns=self.getRoot().getConnInfo()
        globalId=self.getGlobalId()

        return GraphConnCollection(conns[globalId][1])

    def inC(self):
        conns=self.getRoot().getConnInfo()
        globalId=self.getGlobalId()

        return GraphConnCollection(conns[globalId][0])

    def disconnectAllRecurse(self):
        conns = self.getRoot().getConnInfo()
        globalId=self.getGlobalId()

        for fromKey in conns[globalId][0].keys():
            fromElem=conns[globalId][0][fromKey][0]
            fromElemId=fromElem.getGlobalId()
            del conns[fromElemId][1][globalId]
            if globalId==fromElemId:
                del conns[fromElemId][0][globalId]

        for toKey in conns[globalId][1].keys():
            toElem=conns[globalId][1][toKey][0]
            toElemId=toElem.getGlobalId()
            del conns[toElemId][0][globalId]
        del conns[globalId]

    # enter action perfomed by a visitor on this element
    def enter(self, visitor):
        visitor.enterString(self)

    # exit action perfomed by a visitor on this element
    def exit(self, visitor):
        visitor.exitString(self)

    def equals(self, elem):
        if not isinstance(elem,String):
            return 0

        if self.getLocalId() != elem.getLocalId():
            return 0

        if not (self.getType()==self and elem.getType()==elem):
            if not self.getType().equals(elem.getType()):
                return 0

        if not self.getValue()==elem.getValue():
            return 0

        return 1

    def clone(self):
        return String(self.getValue(),
                      self.getLocalId(),
                      parent=None)

    def __repr__(self):
        return str(self.__globalId)

    def __del__(self):
        #print "del node", self
        del self.__id
        del self.__type
        del self.__parent
        del self.__depth
        del self.__globalId
        del self.__root
        del self.__value









