
package test2
  import devs.*;
  import externalfunctions.*;
  import simulator.*;

  class GeneratorState
    Generator.SeqStates seqState(start=Generator.SeqStates.generating);
  end GeneratorState;

  class Generator
    extends AtomicDEVS;
    parameter String name;
    output DevsPort p_out;
    GeneratorState state();
    type SeqStates = enumeration(generating);

    function intTransition
    algorithm
      if( state.seqState == SeqStates.generating ) then
        // Use Modelica syntax
        state.seqState := SeqStates.generating;
      end if;
    end intTransition;

    function outputFnc
      DevsEvent evt = null;
    algorithm
      if( state.seqState == SeqStates.generating) then
        evt := Job(0.1);
        poke(p_out, evt);
      end if;
    end outputFnc;

    function timeAdvance
      output Integer timespan;
    algorithm
      if( state.seqState == SeqStates.generating) then
        timespan := 1;
      end if;
    end timeAdvance;

  end Generator;

  class ProcessorState
    Processor.SeqStates seqState(start=Processor.SeqStates.idle);
  end ProcessorState;

  class Processor
    extends AtomicDEVS;
    parameter String name;
    input DevsPort p_in;
    output DevsPort p_out;
    ProcessorState state();
    type SeqStates = enumeration(idle, state_2);

    function extTransition
    algorithm
      if( state.seqState == SeqStates.idle ) then
        // Use Modelica syntax
        state.seqState := SeqStates.state_2;
      event := peek(p_in);
      end if;
    end extTransition;


    function intTransition
    algorithm
      // Use Modelica syntax
      if( state.seqState == SeqStates.state_2 ) then
        // Use Modelica syntax
        state.seqState := SeqStates.idle;
      end if;
    end intTransition;

    function outputFnc
      DevsEvent evt = null;
    algorithm
      if( state.seqState == SeqStates.state_2) then
        poke(p_out, event);
      end if;
    end outputFnc;

    function timeAdvance
      output Integer timespan;
    algorithm
      if( state.seqState == SeqStates.idle) then
        timespan := INFINITY;
      end if;
      if( state.seqState == SeqStates.state_2) then
        timespan := event.jobSize;
      end if;
    end timeAdvance;

  end Processor;

  class Root
    extends CoupledDEVS;
    parameter String name;
    output DevsPort p_out;
    Generator ins_0();
    Processor ins_2();
    CoupledProcessor ins_6();
  equation
    connect( ins_0.p_out, ins_6.p_in);
    connect( ins_2.p_out, p_out);
    connect( ins_6.p_out, ins_2.p_in);
  end Root;

  class CoupledProcessor
    extends CoupledDEVS;
    parameter String name;
    output DevsPort p_out;
    input DevsPort p_in;
    Processor ins_3();
    Processor ins_4();
  equation
    connect( ins_3.p_out, ins_4.p_in);
    connect( ins_4.p_out, p_out);
    connect( p_in, ins_3.p_in);
  end CoupledProcessor;

  class Job
    extends DevsEvent;
    parameter Real jobSize;
  end Job;

  class Experiment
    extends DevsExperiment;
    Root rootModel(name="Experiment");
    Simulator sim(simModel=rootModel);
    Boolean verbose = True;
    Boolean trace = False;
    Real end_time = 100.0;
  end Experiment;

end test2;
