package test2
  import devs.*;
  import externalfunctions.*;
  import simulator.*;
  class GeneratorState
    Generator.SeqStates seqState(start=Generator.SeqStates.generating);
  end GeneratorState;
  class Generator
    extends AtomicDEVS;
    parameter String name;
    output DevsPort p_out;
    GeneratorState state();
    type SeqStates
      EnumType value;
      parameter StringType quantity="";
      parameter EnumType min=generating, max=generating;
      parameter EnumType start=generating;
      parameter BooleanType fixed=false;
      parameter BooleanType enable=true;
      constant EnumType generating;
    end SeqStates;
    function intTransition
    algorithm 
      if state.seqState == SeqStates.generating then
        state.seqState := SeqStates.generating;
      end if;
    end intTransition;
    function outputFnc
      DevsEvent evt=null;
    algorithm 
      if state.seqState == SeqStates.generating then
        evt := Job(0.1);
        poke(p_out,evt);
      end if;
    end outputFnc;
    function timeAdvance
      output Integer timespan;
    algorithm 
      if state.seqState == SeqStates.generating then
        timespan := 1;
      end if;
    end timeAdvance;
  end Generator;
  class ProcessorState
    Processor.SeqStates seqState(start=Processor.SeqStates.idle);
  end ProcessorState;
  class Processor
    extends AtomicDEVS;
    parameter String name;
    input DevsPort p_in;
    output DevsPort p_out;
    ProcessorState state();
    type SeqStates
      EnumType value;
      parameter StringType quantity="";
      parameter EnumType min=idle, max=state_2;
      parameter EnumType start=idle;
      parameter BooleanType fixed=false;
      parameter BooleanType enable=true;
      constant EnumType idle;
      constant EnumType state_2;
    end SeqStates;
    function extTransition
    algorithm 
      if state.seqState == SeqStates.idle then
        state.seqState := SeqStates.state_2;
        event := peek(p_in);
      end if;
    end extTransition;
    function intTransition
    algorithm 
      if state.seqState == SeqStates.state_2 then
        state.seqState := SeqStates.idle;
      end if;
    end intTransition;
    function outputFnc
      DevsEvent evt=null;
    algorithm 
      if state.seqState == SeqStates.state_2 then
        poke(p_out,event);
      end if;
    end outputFnc;
    function timeAdvance
      output Integer timespan;
    algorithm 
      if state.seqState == SeqStates.idle then
        timespan := INFINITY;
      end if;
      if state.seqState == SeqStates.state_2 then
        timespan := event.jobSize;
      end if;
    end timeAdvance;
  end Processor;
  class Root
    extends CoupledDEVS;
    parameter String name;
    output DevsPort p_out;
    Generator ins_0();
    Processor ins_2();
    CoupledProcessor ins_6();
  equation 
    connect(ins_0.p_out, ins_6.p_in);
    connect(ins_2.p_out, p_out);
    connect(ins_6.p_out, ins_2.p_in);
  end Root;
  class CoupledProcessor
    extends CoupledDEVS;
    parameter String name;
    output DevsPort p_out;
    input DevsPort p_in;
    Processor ins_3();
    Processor ins_4();
  equation 
    connect(ins_3.p_out, ins_4.p_in);
    connect(ins_4.p_out, p_out);
    connect(p_in, ins_3.p_in);
  end CoupledProcessor;
  class Job
    extends DevsEvent;
    parameter Real jobSize;
  end Job;
  class Experiment
    extends DevsExperiment;
    Root rootModel(name="Experiment");
    Simulator sim(simModel=rootModel);
    Boolean verbose=True;
    Boolean trace=False;
    Real end_time=100.0;
  end Experiment;
end test2;

package flatten
  import devs.*;
  class NewAtomic
    dict X=dict();
    list timeLefts;
    Real elapsed=0;
    function get_dstar
      output list dStar;
      Real minimal=0;
      list elems=list();
    algorithm 
      minimal := timeAdvance();
      if instA_timeLeft < minimal+1e-10 then
        elems.append(instA_state);
      end if;
      if len(elems) > 1 then
        dStar := select(elems);
      else
        dStar := elems[0];
      end if;
    end get_dstar;
    function new_intTransition
      Integer counter=0;
      list new=list();
      list transitioned=list();
      list first=list();
      list elems=list();
      Real ta=0.0;
      dict Y_inst=dict();
      list dStar;
    algorithm 
      new := list();
      counter := 0;
      instA_timeLeft := instA_timeLeft-elapsed;
      dStar := get_dstar();
      if dStar == instantiationA then
        Y := outputFnc_modelA(instantiationA_state);
        instantiationA_state := intTransition_modelA(instantiationA_state);
        Y := renamePorts(Y,instantiationA_name);
        Y := mapConnections(Y);
        instA_timeLeft := timeAdvance_modelA(instantiationA_state);
        Y_inst := getInstances(Y,"inst_A");
        elapsed := instA_totalWait-instA_timeLeft;
        instantiationB_state := extTransition_modelB(instantiationB_state,Y_inst,elapsed);
        instB_timeLeft := timeAdvance_modelB(instantiationB_state);
        instB_totalWait := instB_timeLeft;
      end if;
    end new_intTransition;
    function new_select
      input list immChildren;
      output list child;
    algorithm 
      child := immChildren[0];
    end new_select;
    function new_extTransition
      input dict X;
      input Real elapsed;
      dict Y=dict();
      list keys=list();
      dict Y_inst=dict();
    algorithm 
      instA_timeLeft := instA_timeLeft-elapsed;
      Y := mapConnections(X);
      keys := Y.keys();
      if keys.contains("inst_name") then
        Y_inst := getInstances(Y,"inst_A");
        elapsed := instA_totalWait-instA_timeLeft;
        instantiationB_state := extTransition_modelA(state_inst,Y_inst,elapsed);
        instA_timeLeft := timeAdvance_modelA(state_inst);
        instA_totalWait := instA_timeLeft;
      end if;
    end new_extTransition;
    function new_timeAdvance
      output Real timeSpan;
    algorithm 
      timeLefts := list();
      timeLefts.append(instA_timeLeft);
      timeSpan := min(timeLefts);
    end new_timeAdvance;
    function new_outputFnc
      dict outp=dict();
      list dStar;
    algorithm 
      dStar := get_dstar();
      if dStar == instantiationA then
        outp := outputFnc_modelA(instantiationA_state);
        outp := renamePorts(outp,instantiationA_name);
      end if;
      outp := mapConnections(outp);
      poke(outportA,outp.get("a",None));
    end new_outputFnc;
    function mapConnections
      input dict X;
      output dict Y;
    algorithm 
      Y := dict();
      if X.__contains__("b") then
        Y["a"] := X["b"];
      end if;
    end mapConnections;
    function getInstances
      input dict X;
      input String name;
      output dict Y;
      list keys;
      Integer counter;
      String key;
      String new_key;
      String searchname;
      Integer length;
    algorithm 
      Y := dict();
      keys := X.keys();
      counter := 0;
      length := keys.__len__();
      while counter<length loop
        key := keys[counter];
        if key.__contains__(name) then
          searchname := name+"_";
          new_key := key.split(searchname);
          new_key := new_key[1];
          Y[new_key] := X[key];
        end if;
        counter := counter+1;
      end while;
    end getInstances;
    function renamePorts
      input dict X;
      input String name;
      output dict Y=dict();
      list keys=list();
      String newname="";
      String key="";
      Integer counter=0;
      Integer length=0;
    algorithm 
      keys := X.keys();
      counter := 0;
      length := keys.__len__();
      while counter<length loop
        key := keys[counter];
        newname := name+"_"+key;
        Y[newname] := X[key];
        counter := counter+1;
      end while;
    end renamePorts;
  end NewAtomic;
end flatten;

;